@@ -56,8 +56,10 @@ public class TestPassProperties {
   private static String[] input;
   private static HiveConf hiveConf;
 
-  public void Initialize() throws Exception {
+  public void initialize() throws Exception {
     hiveConf = new HiveConf(this.getClass());
+    //TODO: HIVE-27998: hcatalog tests on Tez
+    hiveConf.setVar(HiveConf.ConfVars.HIVE_EXECUTION_ENGINE, "mr");
     hiveConf.set(HiveConf.ConfVars.PRE_EXEC_HOOKS.varname, "");
     hiveConf.set(HiveConf.ConfVars.POST_EXEC_HOOKS.varname, "");
     hiveConf.set(HiveConf.ConfVars.HIVE_SUPPORT_CONCURRENCY.varname, "false");
@@ -82,7 +84,7 @@ public class TestPassProperties {
 
   @Test
   public void testSequenceTableWriteReadMR() throws Exception {
-    Initialize();
+    initialize();
     String createTable = "CREATE TABLE bad_props_table(a0 int, a1 String, a2 String) STORED AS SEQUENCEFILE";
     driver.run("drop table bad_props_table");
     driver.run(createTable);
