@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -27,6 +27,8 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.concurrent.ExecutionException;
+import java.util.concurrent.TimeoutException;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import javax.servlet.http.HttpServletRequest;
@@ -46,8 +48,8 @@ import javax.ws.rs.core.SecurityContext;
 import javax.ws.rs.core.UriInfo;
 
 import org.apache.commons.exec.ExecuteException;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.apache.hadoop.hive.shims.ShimLoader;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.hadoop.security.authentication.client.PseudoAuthenticator;
@@ -123,7 +125,7 @@ public class Server {
   private @QueryParam(DO_AS_PARAM) String doAs;
   private @Context HttpServletRequest request;
 
-  private static final Log LOG = LogFactory.getLog(Server.class);
+  private static final Logger LOG = LoggerFactory.getLogger(Server.class);
 
   /**
    * Check the status of this server.  Always OK.
@@ -363,7 +365,7 @@ public class Server {
     verifyUser();
     verifyDdlParam(db, ":db");
     verifyDdlParam(table, ":table");
-    verifyDdlParam(property, ":property");
+    verifyPropertyParam(property, ":property");
 
     HcatDelegator d = new HcatDelegator(appConf, execService);
     return d.descTableProperty(getDoAsUser(), db, table, property);
@@ -402,7 +404,7 @@ public class Server {
     verifyUser();
     verifyDdlParam(db, ":db");
     verifyDdlParam(table, ":table");
-    verifyDdlParam(property, ":property");
+    verifyPropertyParam(property, ":property");
     desc.name = property;
 
     HcatDelegator d = new HcatDelegator(appConf, execService);
@@ -650,7 +652,7 @@ public class Server {
                       @FormParam("enablelog") boolean enablelog,
                       @FormParam("enablejobreconnect") Boolean enablejobreconnect)
     throws NotAuthorizedException, BusyException, BadParam, QueueException,
-    ExecuteException, IOException, InterruptedException {
+    ExecuteException, IOException, InterruptedException, TooManyRequestsException {
     verifyUser();
     verifyParam(inputs, "input");
     verifyParam(mapper, "mapper");
@@ -704,7 +706,7 @@ public class Server {
                   @FormParam("enablelog") boolean enablelog,
                   @FormParam("enablejobreconnect") Boolean enablejobreconnect)
     throws NotAuthorizedException, BusyException, BadParam, QueueException,
-    ExecuteException, IOException, InterruptedException {
+    ExecuteException, IOException, InterruptedException, TooManyRequestsException {
     verifyUser();
     verifyParam(jar, "jar");
     verifyParam(mainClass, "class");
@@ -754,7 +756,7 @@ public class Server {
                @FormParam("enablelog") boolean enablelog,
                @FormParam("enablejobreconnect") Boolean enablejobreconnect)
     throws NotAuthorizedException, BusyException, BadParam, QueueException,
-    ExecuteException, IOException, InterruptedException {
+    ExecuteException, IOException, InterruptedException, TooManyRequestsException {
     verifyUser();
     if (execute == null && srcFile == null) {
       throw new BadParam("Either execute or file parameter required");
@@ -805,7 +807,7 @@ public class Server {
               @FormParam("enablelog") boolean enablelog,
               @FormParam("enablejobreconnect") Boolean enablejobreconnect)
     throws NotAuthorizedException, BusyException, BadParam, QueueException,
-    IOException, InterruptedException {
+    IOException, InterruptedException, TooManyRequestsException {
     verifyUser();
     if (command == null && optionsFile == null)
       throw new BadParam("Must define Sqoop command or a optionsfile contains Sqoop command to run Sqoop job.");
@@ -859,7 +861,7 @@ public class Server {
               @FormParam("enablelog") boolean enablelog,
               @FormParam("enablejobreconnect") Boolean enablejobreconnect)
     throws NotAuthorizedException, BusyException, BadParam, QueueException,
-    ExecuteException, IOException, InterruptedException {
+    ExecuteException, IOException, InterruptedException, TooManyRequestsException {
     verifyUser();
     if (execute == null && srcFile == null) {
       throw new BadParam("Either execute or file parameter required");
@@ -891,7 +893,8 @@ public class Server {
   @Path("jobs/{jobid}")
   @Produces({MediaType.APPLICATION_JSON})
   public QueueStatusBean showJobId(@PathParam("jobid") String jobid)
-    throws NotAuthorizedException, BadParam, IOException, InterruptedException {
+    throws NotAuthorizedException, BadParam, IOException, InterruptedException,
+           BusyException, TimeoutException, ExecutionException, TooManyRequestsException {
 
     verifyUser();
     verifyParam(jobid, ":jobid");
@@ -923,16 +926,16 @@ public class Server {
    * 1. curl -s 'http://localhost:50111/templeton/v1/jobs?user.name=hsubramaniyan'
    * Return all the Job IDs submitted by hsubramaniyan
    * 2. curl -s
-   * 'http://localhost:50111/templeton/v1/jobs?user.name=hsubramaniyan&showall=true'
+   * 'http://localhost:50111/templeton/v1/jobs?user.name=hsubramaniyan%26showall=true'
    * Return all the Job IDs that are visible to hsubramaniyan
    * 3. curl -s
-   * 'http://localhost:50111/templeton/v1/jobs?user.name=hsubramaniyan&jobid=job_201312091733_0003'
+   * 'http://localhost:50111/templeton/v1/jobs?user.name=hsubramaniyan%26jobid=job_201312091733_0003'
    * Return all the Job IDs for hsubramaniyan after job_201312091733_0003.
    * 4. curl -s 'http://localhost:50111/templeton/v1/jobs?
-   * user.name=hsubramaniyan&jobid=job_201312091733_0003&numrecords=5'
+   * user.name=hsubramaniyan%26jobid=job_201312091733_0003%26numrecords=5'
    * Return the first 5(atmost) Job IDs submitted by hsubramaniyan after job_201312091733_0003.
    * 5.  curl -s
-   * 'http://localhost:50111/templeton/v1/jobs?user.name=hsubramaniyan&numrecords=5'
+   * 'http://localhost:50111/templeton/v1/jobs?user.name=hsubramaniyan%26numrecords=5'
    * Return the first 5(atmost) Job IDs submitted by hsubramaniyan after sorting the Job ID list
    * lexicographically.
    * </p>
@@ -968,7 +971,8 @@ public class Server {
                                        @QueryParam("showall") boolean showall,
                                        @QueryParam("jobid") String jobid,
                                        @QueryParam("numrecords") String numrecords)
-    throws NotAuthorizedException, BadParam, IOException, InterruptedException {
+    throws NotAuthorizedException, BadParam, IOException, InterruptedException,
+    BusyException, TimeoutException, ExecutionException, TooManyRequestsException {
 
     verifyUser();
 
@@ -980,19 +984,14 @@ public class Server {
       showDetails = true;
     }
 
-    ListDelegator ld = new ListDelegator(appConf);
-    List<String> list = ld.run(getDoAsUser(), showall);
-    List<JobItemBean> detailList = new ArrayList<JobItemBean>();
-    int currRecord = 0;
     int numRecords;
-
     // Parse numrecords to an integer
     try {
       if (numrecords != null) {
         numRecords = Integer.parseInt(numrecords);
-  if (numRecords <= 0) {
-    throw new BadParam("numrecords should be an integer > 0");
-  }
+        if (numRecords <= 0) {
+          throw new BadParam("numrecords should be an integer > 0");
+        }
       }
       else {
         numRecords = -1;
@@ -1002,44 +1001,8 @@ public class Server {
       throw new BadParam("Invalid numrecords format: numrecords should be an integer > 0");
     }
 
-    // Sort the list lexicographically
-    Collections.sort(list);
-
-    for (String job : list) {
-      // If numRecords = -1, fetch all records.
-      // Hence skip all the below checks when numRecords = -1.
-      if (numRecords != -1) {
-        // If currRecord >= numRecords, we have already fetched the top #numRecords
-        if (currRecord >= numRecords) {
-          break;
-        }
-        // If the current record needs to be returned based on the
-        // filter conditions specified by the user, increment the counter
-        else if ((jobid != null && job.compareTo(jobid) > 0) || jobid == null) {
-          currRecord++;
-        }
-        // The current record should not be included in the output detailList.
-        else {
-          continue;
-        }
-      }
-      JobItemBean jobItem = new JobItemBean();
-      jobItem.id = job;
-      if (showDetails) {
-        StatusDelegator sd = new StatusDelegator(appConf);
-        try {
-          jobItem.detail = sd.run(getDoAsUser(), job);
-        }
-        catch(Exception ex) {
-          /*if we could not get status for some reason, log it, and send empty status back with
-          * just the ID so that caller knows to even look in the log file*/
-          LOG.info("Failed to get status detail for jobId='" + job + "'", ex);
-          jobItem.detail = new QueueStatusBean(job, "Failed to retrieve status; see WebHCat logs");
-        }
-      }
-      detailList.add(jobItem);
-    }
-    return detailList;
+    ListDelegator ld = new ListDelegator(appConf);
+    return ld.run(getDoAsUser(), showall, jobid, numRecords, showDetails);
   }
 
   /**
@@ -1106,6 +1069,8 @@ public class Server {
   }
 
   public static final Pattern DDL_ID = Pattern.compile("[a-zA-Z]\\w*");
+  public static final Pattern PROPERTY_ID =
+      Pattern.compile("[a-zA-Z0-9][\\w\\.\\-]*(?<!\\-)(?<!\\.)(?<!\\_)$");
 
   /**
    * Verify that the parameter exists and is a simple DDL identifier
@@ -1121,6 +1086,21 @@ public class Server {
       throw new BadParam("Invalid DDL identifier " + name);
     }
   }
+
+  /**
+   * Verify that the parameter exists and is a valid property
+   * name.  Throw an exception if invalid.
+   *
+   */
+  public void verifyPropertyParam(String param, String name)
+    throws BadParam {
+    verifyParam(param, name);
+    Matcher m = PROPERTY_ID.matcher(param);
+    if (!m.matches()) {
+      throw new BadParam("Invalid property name " + name);
+    }
+  }
+
   /**
    * Get the user name from the security context, i.e. the user making the HTTP request.
    * With simple/pseudo security mode this should return the
