@@ -27,14 +27,16 @@ import org.apache.commons.httpclient.methods.PutMethod;
 import org.apache.commons.httpclient.methods.StringRequestEntity;
 import org.apache.hadoop.hive.metastore.MetaStoreTestUtils;
 import org.apache.hadoop.hive.ql.ErrorMsg;
-import org.codehaus.jackson.map.ObjectMapper;
-import org.codehaus.jackson.type.TypeReference;
 import org.junit.AfterClass;
 import org.junit.BeforeClass;
 import org.junit.Ignore;
 import org.junit.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+
+import com.fasterxml.jackson.core.type.TypeReference;
+import com.fasterxml.jackson.databind.ObjectMapper;
+
 import org.eclipse.jetty.http.HttpStatus;
 
 import java.io.IOException;
@@ -43,7 +45,7 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.TreeMap;
-import junit.framework.Assert;
+import org.junit.Assert;
 
 /**
  * A set of tests exercising e2e WebHCat DDL APIs.  These tests are somewhat
@@ -59,6 +61,7 @@ import junit.framework.Assert;
  *
  * It may be possible to extend this to more than just DDL later.
  */
+@Ignore("HIVE-26343")
 public class TestWebHCatE2e {
   private static final Logger LOG =
       LoggerFactory.getLogger(TestWebHCatE2e.class);
@@ -276,7 +279,7 @@ public class TestWebHCatE2e {
     Assert.assertEquals(HttpStatus.NOT_IMPLEMENTED_501, p.httpStatusCode);
     Map<String, Object> props = JsonBuilder.jsonToMap(p.responseBody);
     Assert.assertEquals(p.getAssertMsg(), "Pig version request not yet " +
-        "implemented", (String)props.get("error"));
+        "implemented", props.get("error"));
   }
 
   /**
