@@ -21,10 +21,10 @@ package org.apache.iceberg.hive;
 
 import java.io.IOException;
 import java.nio.file.Path;
-import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.UUID;
+import java.util.concurrent.TimeUnit;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hive.conf.HiveConf;
 import org.apache.hadoop.hive.metastore.api.Database;
@@ -36,7 +36,6 @@ import org.apache.iceberg.CatalogUtil;
 import org.apache.iceberg.DataFile;
 import org.apache.iceberg.DataFiles;
 import org.apache.iceberg.FileFormat;
-import org.apache.iceberg.HasTableOperations;
 import org.apache.iceberg.PartitionSpec;
 import org.apache.iceberg.PartitionSpecParser;
 import org.apache.iceberg.Schema;
@@ -48,10 +47,10 @@ import org.apache.iceberg.Table;
 import org.apache.iceberg.TableMetadata;
 import org.apache.iceberg.TableOperations;
 import org.apache.iceberg.TableProperties;
-import org.apache.iceberg.TestHelpers;
 import org.apache.iceberg.Transaction;
 import org.apache.iceberg.UpdateSchema;
 import org.apache.iceberg.catalog.Catalog;
+import org.apache.iceberg.catalog.CatalogTests;
 import org.apache.iceberg.catalog.Namespace;
 import org.apache.iceberg.catalog.TableIdentifier;
 import org.apache.iceberg.exceptions.AlreadyExistsException;
@@ -66,7 +65,11 @@ import org.apache.iceberg.transforms.Transforms;
 import org.apache.iceberg.types.Types;
 import org.apache.iceberg.util.JsonUtil;
 import org.apache.thrift.TException;
+import org.junit.Assert;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.extension.RegisterExtension;
 import org.junit.jupiter.api.io.TempDir;
 import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.ValueSource;
@@ -88,8 +91,11 @@ import static org.assertj.core.api.Assertions.assertThatThrownBy;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.when;
 
-public class TestHiveCatalog extends HiveMetastoreTest {
-  private static ImmutableMap meta = ImmutableMap.of(
+/**
+ * Run all the tests from abstract of {@link CatalogTests} with few specific tests related to HIVE.
+ */
+public class TestHiveCatalog extends CatalogTests<HiveCatalog> {
+  private static final ImmutableMap meta = ImmutableMap.of(
       "owner", "apache",
       "group", "iceberg",
       "comment", "iceberg  hiveCatalog test");
@@ -97,12 +103,95 @@ public class TestHiveCatalog extends HiveMetastoreTest {
   @TempDir
   private Path temp;
 
+  private HiveCatalog catalog;
+  private static final String DB_NAME = "hivedb";
+
+  @RegisterExtension
+  private static final HiveMetastoreExtension HIVE_METASTORE_EXTENSION =
+      HiveMetastoreExtension.builder().build();
+
+  @BeforeEach
+  public void before() throws TException {
+    catalog = initCatalog("hive", ImmutableMap.of());
+    String dbPath = HIVE_METASTORE_EXTENSION.metastore().getDatabasePath(DB_NAME);
+    Database db = new Database(DB_NAME, "description", dbPath, Maps.newHashMap());
+    HIVE_METASTORE_EXTENSION.metastoreClient().createDatabase(db);
+  }
+
+  @Override
+  protected HiveCatalog initCatalog(String catalogName, Map<String, String> additionalProperties) {
+    Map<String, String> properties =
+            ImmutableMap.of(
+                    CatalogProperties.CLIENT_POOL_CACHE_EVICTION_INTERVAL_MS,
+                    String.valueOf(TimeUnit.SECONDS.toMillis(10)),
+                    CatalogProperties.TABLE_DEFAULT_PREFIX + "default-key1",
+                    "catalog-default-key1",
+                    CatalogProperties.TABLE_DEFAULT_PREFIX + "default-key2",
+                    "catalog-default-key2",
+                    CatalogProperties.TABLE_DEFAULT_PREFIX + "override-key3",
+                    "catalog-default-key3",
+                    CatalogProperties.TABLE_OVERRIDE_PREFIX + "override-key3",
+                    "catalog-override-key3",
+                    CatalogProperties.TABLE_OVERRIDE_PREFIX + "override-key4",
+                    "catalog-override-key4");
+
+    return (HiveCatalog)
+            CatalogUtil.loadCatalog(
+                    HiveCatalog.class.getName(),
+                    catalogName,
+                    ImmutableMap.<String, String>builder()
+                            .putAll(properties)
+                            .putAll(additionalProperties)
+                            .build(),
+                    HIVE_METASTORE_EXTENSION.hiveConf());
+  }
+
+  @AfterEach
+  public void cleanup() throws Exception {
+    HIVE_METASTORE_EXTENSION.metastore().reset();
+  }
+
+  @Override
+  protected boolean requiresNamespaceCreate() {
+    return true;
+  }
+
+  @Override
+  protected boolean supportsNamesWithSlashes() {
+    return false;
+  }
+
+  @Override
+  protected boolean supportsNamesWithDot() {
+    return false;
+  }
+
+  @Override
+  protected HiveCatalog catalog() {
+    return catalog;
+  }
+
   private Schema getTestSchema() {
     return new Schema(
         required(1, "id", Types.IntegerType.get(), "unique ID"),
         required(2, "data", Types.StringType.get()));
   }
 
+  @Test
+  public void testInvalidIdentifiersWithRename() {
+    TableIdentifier invalidFrom = TableIdentifier.of(Namespace.of("l1", "l2"), "table1");
+    TableIdentifier validTo = TableIdentifier.of(Namespace.of("l1"), "renamedTable");
+    assertThatThrownBy(() -> catalog.renameTable(invalidFrom, validTo))
+            .isInstanceOf(IllegalArgumentException.class)
+            .hasMessageContaining("Invalid identifier: " + invalidFrom);
+
+    TableIdentifier validFrom = TableIdentifier.of(Namespace.of("l1"), "table1");
+    TableIdentifier invalidTo = TableIdentifier.of(Namespace.of("l1", "l2"), "renamedTable");
+    assertThatThrownBy(() -> catalog.renameTable(validFrom, invalidTo))
+            .isInstanceOf(IllegalArgumentException.class)
+            .hasMessageContaining("Invalid identifier: " + invalidTo);
+  }
+
   @Test
   public void testCreateTableBuilder() throws Exception {
     Schema schema = getTestSchema();
@@ -165,8 +254,8 @@ public class TestHiveCatalog extends HiveMetastoreTest {
     assertThatNoException()
         .isThrownBy(
             () -> {
-              HiveCatalog catalog = new HiveCatalog();
-              catalog.initialize("hive", Maps.newHashMap());
+              HiveCatalog hiveCatalog = new HiveCatalog();
+              hiveCatalog.initialize("hive", Maps.newHashMap());
             });
   }
 
@@ -175,8 +264,8 @@ public class TestHiveCatalog extends HiveMetastoreTest {
     assertThatNoException()
         .isThrownBy(
             () -> {
-              HiveCatalog catalog = new HiveCatalog();
-              catalog.toString();
+              HiveCatalog hiveCatalog = new HiveCatalog();
+              hiveCatalog.toString();
             });
   }
 
@@ -185,11 +274,12 @@ public class TestHiveCatalog extends HiveMetastoreTest {
     Map<String, String> properties = Maps.newHashMap();
     properties.put("uri", "thrift://examplehost:9083");
     properties.put("warehouse", "/user/hive/testwarehouse");
-    HiveCatalog catalog = new HiveCatalog();
-    catalog.initialize("hive", properties);
+    HiveCatalog hiveCatalog = new HiveCatalog();
+    hiveCatalog.initialize("hive", properties);
 
-    assertThat(catalog.getConf().get("hive.metastore.uris")).isEqualTo("thrift://examplehost:9083");
-    assertThat(catalog.getConf().get("hive.metastore.warehouse.dir"))
+    assertThat(hiveCatalog.getConf().get("hive.metastore.uris"))
+        .isEqualTo("thrift://examplehost:9083");
+    assertThat(hiveCatalog.getConf().get("hive.metastore.warehouse.dir"))
         .isEqualTo("/user/hive/testwarehouse");
   }
 
@@ -288,7 +378,8 @@ public class TestHiveCatalog extends HiveMetastoreTest {
     String location = temp.resolve(tbl).toString();
     try {
       Table table = catalog.createTable(tableIdent, schema, spec, location, properties);
-      org.apache.hadoop.hive.metastore.api.Table hmsTable = metastoreClient.getTable(db, tbl);
+      org.apache.hadoop.hive.metastore.api.Table hmsTable =
+          HIVE_METASTORE_EXTENSION.metastoreClient().getTable(db, tbl);
       assertThat(hmsTable.getOwner()).isEqualTo(owner);
       Map<String, String> hmsTableParams = hmsTable.getParameters();
       assertThat(hmsTableParams).doesNotContainKey(HiveCatalog.HMS_TABLE_OWNER);
@@ -348,10 +439,11 @@ public class TestHiveCatalog extends HiveMetastoreTest {
   }
 
   @Test
-  public void testCreateNamespace() throws Exception {
+  public void testDatabaseAndNamespaceWithLocation() throws Exception {
     Namespace namespace1 = Namespace.of("noLocation");
     catalog.createNamespace(namespace1, meta);
-    Database database1 = metastoreClient.getDatabase(namespace1.toString());
+    Database database1 =
+        HIVE_METASTORE_EXTENSION.metastoreClient().getDatabase(namespace1.toString());
 
     assertThat(database1.getParameters()).containsEntry("owner", "apache");
     assertThat(database1.getParameters()).containsEntry("group", "iceberg");
@@ -362,7 +454,7 @@ public class TestHiveCatalog extends HiveMetastoreTest {
 
     assertThatThrownBy(() -> catalog.createNamespace(namespace1))
         .isInstanceOf(AlreadyExistsException.class)
-        .hasMessage("Namespace '" + namespace1 + "' already exists!");
+        .hasMessage(String.format("Namespace already exists: %s", namespace1));
     String hiveLocalDir = temp.toFile().toURI().toString();
     // remove the trailing slash of the URI
     hiveLocalDir = hiveLocalDir.substring(0, hiveLocalDir.length() - 1);
@@ -373,7 +465,8 @@ public class TestHiveCatalog extends HiveMetastoreTest {
     Namespace namespace2 = Namespace.of("haveLocation");
 
     catalog.createNamespace(namespace2, newMeta);
-    Database database2 = metastoreClient.getDatabase(namespace2.toString());
+    Database database2 =
+        HIVE_METASTORE_EXTENSION.metastoreClient().getDatabase(namespace2.toString());
     assertThat(hiveLocalDir)
         .as("There no same location for db and namespace")
         .isEqualTo(database2.getLocationUri());
@@ -453,27 +546,12 @@ public class TestHiveCatalog extends HiveMetastoreTest {
     Namespace namespace = Namespace.of(name);
 
     catalog.createNamespace(namespace, prop);
-    Database db = metastoreClient.getDatabase(namespace.toString());
+    Database db = HIVE_METASTORE_EXTENSION.metastoreClient().getDatabase(namespace.toString());
 
     assertThat(db.getOwnerName()).isEqualTo(expectedOwner);
     assertThat(db.getOwnerType()).isEqualTo(expectedOwnerType);
   }
 
-  @Test
-  public void testListNamespace() throws TException {
-    List<Namespace> namespaces;
-    Namespace namespace1 = Namespace.of("dbname1");
-    catalog.createNamespace(namespace1, meta);
-    namespaces = catalog.listNamespaces(namespace1);
-    assertThat(namespaces).as("Hive db not hive the namespace 'dbname1'").isEmpty();
-
-    Namespace namespace2 = Namespace.of("dbname2");
-    catalog.createNamespace(namespace2, meta);
-    namespaces = catalog.listNamespaces();
-
-    assertThat(namespaces).as("Hive db not hive the namespace 'dbname2'").contains(namespace2);
-  }
-
   @Test
   public void testLoadNamespaceMeta() throws TException {
     Namespace namespace = Namespace.of("dbname_load");
@@ -489,7 +567,7 @@ public class TestHiveCatalog extends HiveMetastoreTest {
   }
 
   @Test
-  public void testNamespaceExists() throws TException {
+  public void testNamespaceExists() {
     Namespace namespace = Namespace.of("dbname_exists");
 
     catalog.createNamespace(namespace, meta);
@@ -501,7 +579,7 @@ public class TestHiveCatalog extends HiveMetastoreTest {
   }
 
   @Test
-  public void testSetNamespaceProperties() throws TException {
+  public void testSetNamespaceProperties() {
     Namespace namespace = Namespace.of("dbname_set");
 
     catalog.createNamespace(namespace, meta);
@@ -513,7 +591,12 @@ public class TestHiveCatalog extends HiveMetastoreTest {
             "comment", "iceberg test")
     );
 
-    Database database = metastoreClient.getDatabase(namespace.level(0));
+    Database database = null;
+    try {
+      database = HIVE_METASTORE_EXTENSION.metastoreClient().getDatabase(namespace.level(0));
+    } catch (TException e) {
+      Assert.fail();
+    }
     assertThat(database.getParameters()).containsEntry("owner", "alter_apache");
     assertThat(database.getParameters()).containsEntry("test", "test");
     assertThat(database.getParameters()).containsEntry("group", "iceberg");
@@ -698,22 +781,26 @@ public class TestHiveCatalog extends HiveMetastoreTest {
         name, propToCreate, expectedOwnerPostCreate, expectedOwnerTypePostCreate);
 
     catalog.setProperties(Namespace.of(name), propToSet);
-    Database database = metastoreClient.getDatabase(name);
+    Database database = HIVE_METASTORE_EXTENSION.metastoreClient().getDatabase(name);
 
     assertThat(database.getOwnerName()).isEqualTo(expectedOwnerPostSet);
     assertThat(database.getOwnerType()).isEqualTo(expectedOwnerTypePostSet);
   }
 
   @Test
-  public void testRemoveNamespaceProperties() throws TException {
+  public void testRemoveNamespaceProperties() {
     Namespace namespace = Namespace.of("dbname_remove");
 
     catalog.createNamespace(namespace, meta);
 
     catalog.removeProperties(namespace, ImmutableSet.of("comment", "owner"));
 
-    Database database = metastoreClient.getDatabase(namespace.level(0));
-
+    Database database = null;
+    try {
+      database = HIVE_METASTORE_EXTENSION.metastoreClient().getDatabase(namespace.level(0));
+    } catch (TException e) {
+      Assert.fail();
+    }
     assertThat(database.getParameters()).doesNotContainKey("owner");
     assertThat(database.getParameters()).containsEntry("group", "iceberg");
 
@@ -844,14 +931,14 @@ public class TestHiveCatalog extends HiveMetastoreTest {
 
     catalog.removeProperties(Namespace.of(name), propToRemove);
 
-    Database database = metastoreClient.getDatabase(name);
+    Database database = HIVE_METASTORE_EXTENSION.metastoreClient().getDatabase(name);
 
     assertThat(database.getOwnerName()).isEqualTo(expectedOwnerPostRemove);
     assertThat(database.getOwnerType()).isEqualTo(expectedOwnerTypePostRemove);
   }
 
   @Test
-  public void testDropNamespace() throws TException {
+  public void dropNamespace() {
     Namespace namespace = Namespace.of("dbname_drop");
     TableIdentifier identifier = TableIdentifier.of(namespace, "table");
     Schema schema = getTestSchema();
@@ -917,7 +1004,7 @@ public class TestHiveCatalog extends HiveMetastoreTest {
   }
 
   private String defaultUri(Namespace namespace) throws TException {
-    return metastoreClient.getConfigValue(
+    return HIVE_METASTORE_EXTENSION.metastoreClient().getConfigValue(
         "hive.metastore.warehouse.external.dir", "") +  "/" + namespace.level(0) + ".db";
   }
 
@@ -950,8 +1037,8 @@ public class TestHiveCatalog extends HiveMetastoreTest {
           .create();
 
       String tableName = tableIdentifier.name();
-      org.apache.hadoop.hive.metastore.api.Table hmsTable =
-          metastoreClient.getTable(tableIdentifier.namespace().level(0), tableName);
+      org.apache.hadoop.hive.metastore.api.Table hmsTable = HIVE_METASTORE_EXTENSION.metastoreClient().getTable(
+          tableIdentifier.namespace().level(0), tableName);
 
       // check whether parameters are in expected state
       Map<String, String> parameters = hmsTableParameters();
@@ -989,9 +1076,6 @@ public class TestHiveCatalog extends HiveMetastoreTest {
 
   @Test
   public void testSetSnapshotSummary() throws Exception {
-    Configuration conf = new Configuration();
-    conf.set("iceberg.hive.table-property-max-size", "4000");
-    HiveTableOperations ops = new HiveTableOperations(conf, null, null, catalog.name(), DB_NAME, "tbl");
     Snapshot snapshot = mock(Snapshot.class);
     Map<String, String> summary = Maps.newHashMap();
     when(snapshot.summary()).thenReturn(summary);
@@ -1002,7 +1086,8 @@ public class TestHiveCatalog extends HiveMetastoreTest {
     }
     assertThat(JsonUtil.mapper().writeValueAsString(summary).length()).isLessThan(4000);
     Map<String, String> parameters = Maps.newHashMap();
-    ops.setSnapshotSummary(parameters, snapshot);
+    final long maxHiveTablePropertySize = 4000;
+    HMSTablePropertyHelper.setSnapshotSummary(parameters, snapshot, maxHiveTablePropertySize);
     assertThat(parameters).as("The snapshot summary must be in parameters").hasSize(1);
 
     // create a snapshot summary whose json string size exceeds the limit
@@ -1013,7 +1098,7 @@ public class TestHiveCatalog extends HiveMetastoreTest {
     // the limit has been updated to 4000 instead of the default value(32672)
     assertThat(summarySize).isGreaterThan(4000).isLessThan(32672);
     parameters.remove(CURRENT_SNAPSHOT_SUMMARY);
-    ops.setSnapshotSummary(parameters, snapshot);
+    HMSTablePropertyHelper.setSnapshotSummary(parameters, snapshot, maxHiveTablePropertySize);
     assertThat(parameters)
         .as("The snapshot summary must not be in parameters due to the size limit")
         .isEmpty();
@@ -1021,9 +1106,7 @@ public class TestHiveCatalog extends HiveMetastoreTest {
 
   @Test
   public void testNotExposeTableProperties() {
-    Configuration conf = new Configuration();
-    conf.set("iceberg.hive.table-property-max-size", "0");
-    HiveTableOperations ops = new HiveTableOperations(conf, null, null, catalog.name(), DB_NAME, "tbl");
+    final long maxHiveTablePropertySize = 0;
     TableMetadata metadata = mock(TableMetadata.class);
     Map<String, String> parameters = Maps.newHashMap();
     parameters.put(CURRENT_SNAPSHOT_SUMMARY, "summary");
@@ -1033,19 +1116,19 @@ public class TestHiveCatalog extends HiveMetastoreTest {
     parameters.put(DEFAULT_PARTITION_SPEC, "partitionSpec");
     parameters.put(DEFAULT_SORT_ORDER, "sortOrder");
 
-    ops.setSnapshotStats(metadata, parameters);
+    HMSTablePropertyHelper.setSnapshotStats(metadata, parameters, maxHiveTablePropertySize);
     assertThat(parameters)
         .doesNotContainKey(CURRENT_SNAPSHOT_SUMMARY)
         .doesNotContainKey(CURRENT_SNAPSHOT_ID)
         .doesNotContainKey(CURRENT_SNAPSHOT_TIMESTAMP);
 
-    ops.setSchema(metadata, parameters);
+    HMSTablePropertyHelper.setSchema(metadata.schema(), parameters, maxHiveTablePropertySize);
     assertThat(parameters).doesNotContainKey(CURRENT_SCHEMA);
 
-    ops.setPartitionSpec(metadata, parameters);
+    HMSTablePropertyHelper.setPartitionSpec(metadata, parameters, maxHiveTablePropertySize);
     assertThat(parameters).doesNotContainKey(DEFAULT_PARTITION_SPEC);
 
-    ops.setSortOrder(metadata, parameters);
+    HMSTablePropertyHelper.setSortOrder(metadata, parameters, maxHiveTablePropertySize);
     assertThat(parameters).doesNotContainKey(DEFAULT_SORT_ORDER);
   }
 
@@ -1095,7 +1178,8 @@ public class TestHiveCatalog extends HiveMetastoreTest {
   }
 
   private Map<String, String> hmsTableParameters() throws TException {
-    org.apache.hadoop.hive.metastore.api.Table hmsTable = metastoreClient.getTable(DB_NAME, "tbl");
+    org.apache.hadoop.hive.metastore.api.Table hmsTable =
+        HIVE_METASTORE_EXTENSION.metastoreClient().getTable(DB_NAME, "tbl");
     return hmsTable.getParameters();
   }
 
@@ -1128,7 +1212,7 @@ public class TestHiveCatalog extends HiveMetastoreTest {
             HiveCatalog.class.getName(),
             CatalogUtil.ICEBERG_CATALOG_TYPE_HIVE,
             catalogProps,
-            hiveConf);
+            HIVE_METASTORE_EXTENSION.hiveConf());
 
     try {
       Table table =
@@ -1168,42 +1252,11 @@ public class TestHiveCatalog extends HiveMetastoreTest {
     conf.set("hive.metastore.warehouse.dir", "s3://bucket/");
     conf.set("hive.metastore.warehouse.external.dir", "s3://bucket/");
 
-    HiveCatalog catalog = new HiveCatalog();
-    catalog.setConf(conf);
+    HiveCatalog hiveCatalog = new HiveCatalog();
+    hiveCatalog.setConf(conf);
 
-    Database database = catalog.convertToDatabase(Namespace.of("database"), ImmutableMap.of());
+    Database database = hiveCatalog.convertToDatabase(Namespace.of("database"), ImmutableMap.of());
 
     assertThat(database.getLocationUri()).isEqualTo("s3://bucket/database.db");
   }
-
-  @Test
-  public void testRegisterTable() {
-    TableIdentifier identifier = TableIdentifier.of(DB_NAME, "t1");
-    catalog.createTable(identifier, getTestSchema());
-    Table registeringTable = catalog.loadTable(identifier);
-    catalog.dropTable(identifier, false);
-    TableOperations ops = ((HasTableOperations) registeringTable).operations();
-    String metadataLocation = ((HiveTableOperations) ops).currentMetadataLocation();
-    Table registeredTable = catalog.registerTable(identifier, metadataLocation);
-    assertThat(registeredTable).isNotNull();
-    TestHelpers.assertSerializedAndLoadedMetadata(registeringTable, registeredTable);
-    String expectedMetadataLocation =
-        ((HasTableOperations) registeredTable).operations().current().metadataFileLocation();
-    assertThat(metadataLocation).isEqualTo(expectedMetadataLocation);
-    assertThat(catalog.loadTable(identifier)).isNotNull();
-    assertThat(catalog.dropTable(identifier)).isTrue();
-  }
-
-  @Test
-  public void testRegisterExistingTable() {
-    TableIdentifier identifier = TableIdentifier.of(DB_NAME, "t1");
-    catalog.createTable(identifier, getTestSchema());
-    Table registeringTable = catalog.loadTable(identifier);
-    TableOperations ops = ((HasTableOperations) registeringTable).operations();
-    String metadataLocation = ((HiveTableOperations) ops).currentMetadataLocation();
-    assertThatThrownBy(() -> catalog.registerTable(identifier, metadataLocation))
-        .isInstanceOf(AlreadyExistsException.class)
-        .hasMessage("Table already exists: hivedb.t1");
-    assertThat(catalog.dropTable(identifier, true)).isTrue();
-  }
 }
