@@ -22,9 +22,12 @@ package org.apache.iceberg.mr;
 import java.io.File;
 import java.io.IOException;
 import java.io.UncheckedIOException;
+import java.lang.reflect.Method;
+import java.security.PrivilegedAction;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.concurrent.ExecutorService;
 import java.util.function.Function;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.Path;
@@ -38,6 +41,7 @@ import org.apache.hadoop.mapreduce.InputSplit;
 import org.apache.hadoop.mapreduce.RecordReader;
 import org.apache.hadoop.mapreduce.TaskAttemptContext;
 import org.apache.hadoop.mapreduce.task.TaskAttemptContextImpl;
+import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.iceberg.AppendFiles;
 import org.apache.iceberg.CatalogProperties;
 import org.apache.iceberg.CatalogUtil;
@@ -64,6 +68,7 @@ import org.apache.iceberg.relocated.com.google.common.collect.Lists;
 import org.apache.iceberg.relocated.com.google.common.collect.Sets;
 import org.apache.iceberg.types.TypeUtil;
 import org.apache.iceberg.types.Types;
+import org.apache.iceberg.util.ThreadPools;
 import org.assertj.core.api.Assertions;
 import org.junit.Assert;
 import org.junit.Assume;
@@ -76,6 +81,7 @@ import org.junit.runner.RunWith;
 import org.junit.runners.Parameterized;
 
 import static org.apache.iceberg.types.Types.NestedField.required;
+import static org.assertj.core.api.Assertions.assertThat;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
 
@@ -112,7 +118,7 @@ public class TestIcebergInputFormats {
 
   @Before
   public void before() throws IOException {
-    conf = new Configuration();
+    conf = new JobConf();
     conf.set(CatalogUtil.ICEBERG_CATALOG_TYPE, Catalogs.LOCATION);
     HadoopTables tables = new HadoopTables(conf);
 
@@ -232,8 +238,6 @@ public class TestIcebergInputFormats {
             "Filter expression ref(name=\"id\") == 0 is not completely satisfied. Additional rows can be returned " +
                     "not satisfied by the filter expression");
 
-    builder.usePigTuples();
-
     Assertions.assertThatThrownBy(() -> testInputFormat.create(builder.conf()))
         .isInstanceOf(UnsupportedOperationException.class)
         .hasMessage(
@@ -410,6 +414,45 @@ public class TestIcebergInputFormats {
         mapWork.getCacheAffinity());
   }
 
+  @Test
+  public void testWorkerPool() throws Exception {
+    Table table = helper.createUnpartitionedTable();
+    UserGroupInformation user1 =
+            UserGroupInformation.createUserForTesting("user1", new String[] {});
+    UserGroupInformation user2 =
+            UserGroupInformation.createUserForTesting("user2", new String[] {});
+    final ExecutorService workerPool1 = ThreadPools.newFixedThreadPool("iceberg-plan-worker-pool", 1);
+    final ExecutorService workerPool2 = ThreadPools.newFixedThreadPool("iceberg-plan-worker-pool", 1);
+    try {
+      assertThat(getUserFromWorkerPool(user1, table, workerPool1)).isEqualTo("user1");
+      assertThat(getUserFromWorkerPool(user2, table, workerPool1)).isEqualTo("user1");
+      assertThat(getUserFromWorkerPool(user2, table, workerPool2)).isEqualTo("user2");
+    } finally {
+      workerPool1.shutdown();
+      workerPool2.shutdown();
+    }
+  }
+
+  private String getUserFromWorkerPool(
+          UserGroupInformation user, Table table, ExecutorService workerpool) throws Exception {
+    Method method =
+            IcebergInputFormat.class.getDeclaredMethod(
+                    "planInputSplits", Table.class, Configuration.class, ExecutorService.class);
+    method.setAccessible(true);
+    return user.doAs(
+            (PrivilegedAction<String>)
+                    () -> {
+                      try {
+                        method.invoke(new IcebergInputFormat<>(), table, conf, workerpool);
+                        return workerpool
+                                .submit(() -> UserGroupInformation.getCurrentUser().getUserName())
+                                .get();
+                      } catch (Exception e) {
+                        throw new RuntimeException("Failed to get user from worker pool", e);
+                      }
+                    });
+  }
+
   // TODO - Capture template type T in toString method: https://github.com/apache/iceberg/issues/1542
   public abstract static class TestInputFormat<T> {
 
