@@ -41,6 +41,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.UUID;
 import java.util.stream.Collectors;
+import java.util.stream.Stream;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hive.common.type.HiveDecimal;
 import org.apache.hadoop.hive.common.type.TimestampTZUtil;
@@ -284,10 +285,14 @@ public class HiveIcebergTestUtils {
    * @param dataFileNum The expected number of data files (TABLE_LOCATION/data/*)
    */
   public static void validateFiles(Table table, Configuration conf, JobID jobId, int dataFileNum) throws IOException {
-    List<Path> dataFiles = Files.walk(Paths.get(table.location() + "/data"))
-        .filter(Files::isRegularFile)
-        .filter(path -> !path.getFileName().toString().startsWith("."))
-        .collect(Collectors.toList());
+    List<Path> dataFiles;
+    try (Stream<Path> files = Files.walk(Paths.get(table.location() + "/data"))) {
+      dataFiles =
+          files
+              .filter(Files::isRegularFile)
+              .filter(path -> !path.getFileName().toString().startsWith("."))
+              .collect(Collectors.toList());
+    }
 
     Assert.assertEquals(dataFileNum, dataFiles.size());
     Assert.assertFalse(
