@@ -20,10 +20,13 @@
 package org.apache.iceberg.mr.hive;
 
 import java.io.IOException;
+import java.util.Collection;
 import java.util.List;
+import org.apache.commons.lang3.ArrayUtils;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hive.common.StatsSetupConst;
 import org.apache.hadoop.hive.conf.HiveConf;
+import org.apache.iceberg.AssertHelpers;
 import org.apache.iceberg.PartitionSpec;
 import org.apache.iceberg.Table;
 import org.apache.iceberg.TableProperties;
@@ -36,7 +39,9 @@ import org.apache.iceberg.types.Types.NestedField;
 import org.apache.thrift.TException;
 import org.junit.Assert;
 import org.junit.Assume;
+import org.junit.Before;
 import org.junit.Test;
+import org.junit.runners.Parameterized;
 
 
 /**
@@ -45,8 +50,32 @@ import org.junit.Test;
  */
 public class TestHiveIcebergStatistics extends HiveIcebergStorageHandlerWithEngineBase {
 
+  @Parameterized.Parameter(4)
+  public String statsSource;
+
+  @Parameterized.Parameters(name = "fileFormat={0}, catalog={1}, isVectorized={2}, formatVersion={3}, statsSource={4}")
+  public static Collection<Object[]> parameters() {
+    Collection<Object[]> baseParams = HiveIcebergStorageHandlerWithEngineBase.parameters();
+
+    Collection<Object[]> testParams = Lists.newArrayList();
+    for (String statsSource : new String[]{"iceberg", "metastore"}) {
+      for (Object[] params : baseParams) {
+        testParams.add(ArrayUtils.add(params, statsSource));
+      }
+    }
+    return testParams;
+  }
+
+  @Before
+  public void setStatsSource() {
+    HiveConf.setVar(shell.getHiveConf(), HiveConf.ConfVars.HIVE_ICEBERG_STATS_SOURCE, statsSource);
+  }
+
   @Test
   public void testAnalyzeTableComputeStatistics() throws IOException, TException, InterruptedException {
+    Assume.assumeTrue(statsSource.equals("iceberg") ||
+        testTableType == TestTables.TestTableType.HIVE_CATALOG);
+
     String dbName = "default";
     String tableName = "customers";
     Table table = testTables
@@ -80,7 +109,7 @@ public class TestHiveIcebergStatistics extends HiveIcebergStorageHandlerWithEngi
   public void testStatsWithInsert() {
     TableIdentifier identifier = TableIdentifier.of("default", "customers");
 
-    shell.setHiveSessionValue(HiveConf.ConfVars.HIVESTATSAUTOGATHER.varname, true);
+    shell.setHiveSessionValue(HiveConf.ConfVars.HIVE_STATS_AUTOGATHER.varname, true);
     testTables.createTable(shell, identifier.name(), HiveIcebergStorageHandlerTestUtils.CUSTOMER_SCHEMA,
         PartitionSpec.unpartitioned(), fileFormat, ImmutableList.of());
 
@@ -102,11 +131,45 @@ public class TestHiveIcebergStatistics extends HiveIcebergStorageHandlerWithEngi
     checkColStatMinMaxValue(identifier.name(), "customer_id", 0, 5);
   }
 
+  @Test
+  public void testStatsWithPessimisticLockInsert() {
+    Assume.assumeTrue(testTableType == TestTables.TestTableType.HIVE_CATALOG);
+    TableIdentifier identifier = getTableIdentifierWithPessimisticLock("false");
+    String insert = testTables.getInsertQuery(HiveIcebergStorageHandlerTestUtils.CUSTOMER_RECORDS, identifier, false);
+    shell.executeStatement(insert);
+
+    checkColStat(identifier.name(), "customer_id", true);
+    checkColStatMinMaxValue(identifier.name(), "customer_id", 0, 2);
+  }
+
+  @Test
+  public void testStatsWithPessimisticLockInsertWhenHiveLockEnabled() {
+    Assume.assumeTrue(testTableType == TestTables.TestTableType.HIVE_CATALOG);
+    TableIdentifier identifier = getTableIdentifierWithPessimisticLock("true");
+    String insert = testTables.getInsertQuery(HiveIcebergStorageHandlerTestUtils.CUSTOMER_RECORDS, identifier, false);
+    AssertHelpers.assertThrows(
+        "Should throw RuntimeException when Hive locking is on with 'engine.hive.lock-enabled=true'",
+        RuntimeException.class,
+        () -> shell.executeStatement(insert)
+    );
+  }
+
+  private TableIdentifier getTableIdentifierWithPessimisticLock(String hiveLockEnabled) {
+    TableIdentifier identifier = TableIdentifier.of("default", "customers");
+
+    shell.setHiveSessionValue(HiveConf.ConfVars.HIVE_STATS_AUTOGATHER.varname, true);
+    shell.setHiveSessionValue(HiveConf.ConfVars.HIVE_TXN_EXT_LOCKING_ENABLED.varname, true);
+    testTables.createTable(shell, identifier.name(), HiveIcebergStorageHandlerTestUtils.CUSTOMER_SCHEMA,
+        PartitionSpec.unpartitioned(), fileFormat, ImmutableList.of(), formatVersion,
+        ImmutableMap.of(TableProperties.HIVE_LOCK_ENABLED, hiveLockEnabled));
+    return identifier;
+  }
+
   @Test
   public void testStatsWithInsertOverwrite() {
     TableIdentifier identifier = TableIdentifier.of("default", "customers");
 
-    shell.setHiveSessionValue(HiveConf.ConfVars.HIVESTATSAUTOGATHER.varname, true);
+    shell.setHiveSessionValue(HiveConf.ConfVars.HIVE_STATS_AUTOGATHER.varname, true);
     testTables.createTable(shell, identifier.name(), HiveIcebergStorageHandlerTestUtils.CUSTOMER_SCHEMA,
         PartitionSpec.unpartitioned(), fileFormat, ImmutableList.of());
 
@@ -124,7 +187,7 @@ public class TestHiveIcebergStatistics extends HiveIcebergStorageHandlerWithEngi
     PartitionSpec spec = PartitionSpec.builderFor(HiveIcebergStorageHandlerTestUtils.CUSTOMER_SCHEMA)
         .identity("last_name").build();
 
-    shell.setHiveSessionValue(HiveConf.ConfVars.HIVESTATSAUTOGATHER.varname, true);
+    shell.setHiveSessionValue(HiveConf.ConfVars.HIVE_STATS_AUTOGATHER.varname, true);
     testTables.createTable(shell, identifier.name(), HiveIcebergStorageHandlerTestUtils.CUSTOMER_SCHEMA, spec,
         fileFormat, ImmutableList.of());
 
@@ -149,7 +212,7 @@ public class TestHiveIcebergStatistics extends HiveIcebergStorageHandlerWithEngi
     shell.executeStatement(testTables.getInsertQuery(
         HiveIcebergStorageHandlerTestUtils.CUSTOMER_RECORDS, TableIdentifier.of("default", "source"), false));
 
-    shell.setHiveSessionValue(HiveConf.ConfVars.HIVESTATSAUTOGATHER.varname, true);
+    shell.setHiveSessionValue(HiveConf.ConfVars.HIVE_STATS_AUTOGATHER.varname, true);
     shell.executeStatement(String.format(
         "CREATE TABLE target STORED BY ICEBERG %s %s AS SELECT * FROM source",
         testTables.locationForCreateTableSQL(TableIdentifier.of("default", "target")),
@@ -168,12 +231,17 @@ public class TestHiveIcebergStatistics extends HiveIcebergStorageHandlerWithEngi
     shell.executeStatement(testTables.getInsertQuery(
         HiveIcebergStorageHandlerTestUtils.CUSTOMER_RECORDS, TableIdentifier.of("default", "source"), false));
 
-    shell.setHiveSessionValue(HiveConf.ConfVars.HIVESTATSAUTOGATHER.varname, true);
+    shell.setHiveSessionValue(HiveConf.ConfVars.HIVE_STATS_AUTOGATHER.varname, true);
     shell.executeStatement(String.format(
         "CREATE TABLE target PARTITIONED BY (dept, name) STORED BY ICEBERG %s AS SELECT * FROM source s",
         testTables.propertiesForCreateTableSQL(
             ImmutableMap.of(TableProperties.DEFAULT_FILE_FORMAT, fileFormat.toString()))));
 
+    if (statsSource.equals("iceberg")) {
+      // TODO: Propagate partition spec from CREATE statement to the ColumnStatsSemanticAnalyzer
+      shell.executeStatement("ANALYZE TABLE target COMPUTE STATISTICS FOR COLUMNS");
+    }
+
     checkColStat("target", "id", true);
     checkColStat("target", "dept", true);
     checkColStatMinMaxValue("target", "id", 0, 2);
@@ -188,7 +256,7 @@ public class TestHiveIcebergStatistics extends HiveIcebergStorageHandlerWithEngi
 
     TableIdentifier identifier = TableIdentifier.of("default", "customers");
 
-    shell.setHiveSessionValue(HiveConf.ConfVars.HIVESTATSAUTOGATHER.varname, true);
+    shell.setHiveSessionValue(HiveConf.ConfVars.HIVE_STATS_AUTOGATHER.varname, true);
     testTables.createTable(shell, identifier.name(), HiveIcebergStorageHandlerTestUtils.CUSTOMER_SCHEMA,
         PartitionSpec.unpartitioned(), fileFormat, ImmutableList.of());
 
@@ -214,7 +282,7 @@ public class TestHiveIcebergStatistics extends HiveIcebergStorageHandlerWithEngi
   public void testColumnStatsAccurate() throws Exception {
     TableIdentifier identifier = TableIdentifier.of("default", "customers");
 
-    shell.setHiveSessionValue(HiveConf.ConfVars.HIVESTATSAUTOGATHER.varname, true);
+    shell.setHiveSessionValue(HiveConf.ConfVars.HIVE_STATS_AUTOGATHER.varname, true);
     testTables.createTable(shell, identifier.name(), HiveIcebergStorageHandlerTestUtils.CUSTOMER_SCHEMA,
             PartitionSpec.unpartitioned(), fileFormat, ImmutableList.of());
 
@@ -235,7 +303,7 @@ public class TestHiveIcebergStatistics extends HiveIcebergStorageHandlerWithEngi
   public void testMergeStatsWithInsert() {
     TableIdentifier identifier = TableIdentifier.of("default", "customers");
 
-    shell.setHiveSessionValue(HiveConf.ConfVars.HIVESTATSAUTOGATHER.varname, true);
+    shell.setHiveSessionValue(HiveConf.ConfVars.HIVE_STATS_AUTOGATHER.varname, true);
     testTables.createTable(shell, identifier.name(), HiveIcebergStorageHandlerTestUtils.CUSTOMER_SCHEMA,
         PartitionSpec.unpartitioned(), fileFormat, ImmutableList.of());
 
@@ -264,9 +332,11 @@ public class TestHiveIcebergStatistics extends HiveIcebergStorageHandlerWithEngi
 
   @Test
   public void testIcebergColStatsPath() throws IOException {
+    Assume.assumeTrue(statsSource.equals("iceberg"));
+
     TableIdentifier identifier = TableIdentifier.of("default", "customers");
 
-    shell.setHiveSessionValue(HiveConf.ConfVars.HIVESTATSAUTOGATHER.varname, true);
+    shell.setHiveSessionValue(HiveConf.ConfVars.HIVE_STATS_AUTOGATHER.varname, true);
     Table table = testTables.createTable(shell, identifier.name(), HiveIcebergStorageHandlerTestUtils.CUSTOMER_SCHEMA,
         PartitionSpec.unpartitioned(), fileFormat, ImmutableList.of());
 
@@ -332,8 +402,6 @@ public class TestHiveIcebergStatistics extends HiveIcebergStorageHandlerWithEngi
 
   private void checkColStatMinMaxDistinctValue(String tableName, String colName, int minValue, int maxValue,
       int distinct, int nulls) {
-
-    shell.executeStatement("set hive.iceberg.stats.source=metastore");
     List<Object[]> rows = shell.executeStatement("DESCRIBE FORMATTED " + tableName + " " + colName);
 
     // Check min
@@ -351,25 +419,5 @@ public class TestHiveIcebergStatistics extends HiveIcebergStorageHandlerWithEngi
     // Check distinct
     Assert.assertEquals("distinct_count", rows.get(5)[0]);
     Assert.assertEquals(String.valueOf(distinct), rows.get(5)[1]);
-
-    shell.executeStatement("set hive.iceberg.stats.source=iceberg");
-    rows = shell.executeStatement("DESCRIBE FORMATTED " + tableName + " " + colName);
-
-    // Check min
-    Assert.assertEquals("min", rows.get(2)[0]);
-    Assert.assertEquals(String.valueOf(minValue), rows.get(2)[1]);
-
-    // Check max
-    Assert.assertEquals("max", rows.get(3)[0]);
-    Assert.assertEquals(String.valueOf(maxValue), rows.get(3)[1]);
-
-    // Check num of nulls
-    Assert.assertEquals("num_nulls", rows.get(4)[0]);
-    Assert.assertEquals(String.valueOf(nulls), rows.get(4)[1]);
-
-    // Check distinct
-    Assert.assertEquals("distinct_count", rows.get(5)[0]);
-    Assert.assertEquals(String.valueOf(distinct), rows.get(5)[1]);
-
   }
 }
