@@ -25,6 +25,8 @@ import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
 import java.io.IOException;
+import java.lang.invoke.MethodHandles;
+import java.lang.invoke.VarHandle;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Field;
 import java.lang.reflect.Modifier;
@@ -67,6 +69,7 @@ import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.fs.permission.FsPermission;
 import org.apache.hadoop.hive.conf.HiveConf;
 import org.apache.hadoop.hive.conf.HiveConf.ConfVars;
+import org.apache.hadoop.hive.conf.HiveConfForTest;
 import org.apache.hadoop.hive.metastore.PersistenceManagerProvider;
 import org.apache.hadoop.hive.ql.QueryPlan;
 import org.apache.hadoop.hive.ql.exec.FunctionRegistry;
@@ -104,7 +107,7 @@ public class TestJdbcWithMiniHS2 {
   @BeforeClass
   public static void setupBeforeClass() throws Exception {
     MiniHS2.cleanupLocalDir();
-    HiveConf conf = new HiveConf();
+    HiveConf conf = getNewHiveConf();
     dataFileDir = conf.get("test.data.files").replace('\\', '/').replace("c:", "");
     kvDataFilePath = new Path(dataFileDir, "kv1.txt");
 
@@ -202,7 +205,7 @@ public class TestJdbcWithMiniHS2 {
       }
     }
     stopMiniHS2();
-    HiveConf conf = new HiveConf();
+    HiveConf conf = getNewHiveConf();
     startMiniHS2(conf);
     openDefaultConnections();
     openTestConnections();
@@ -215,7 +218,7 @@ public class TestJdbcWithMiniHS2 {
   private static void startMiniHS2(HiveConf conf, boolean httpMode) throws Exception {
     conf.setBoolVar(ConfVars.HIVE_SUPPORT_CONCURRENCY, false);
     conf.setBoolVar(ConfVars.HIVE_SERVER2_LOGGING_OPERATION_ENABLED, false);
-    conf.setBoolVar(ConfVars.HIVESTATSCOLAUTOGATHER, false);
+    conf.setBoolVar(ConfVars.HIVE_STATS_COL_AUTOGATHER, false);
     // store post-exec hooks calls so we can look at them later
     conf.setVar(ConfVars.POST_EXEC_HOOKS, ReadableHook.class.getName() + "," +
         LineageLogger.class.getName());
@@ -788,7 +791,7 @@ public class TestJdbcWithMiniHS2 {
   public void testSessionScratchDirs() throws Exception {
     // Stop HiveServer2
     stopMiniHS2();
-    HiveConf conf = new HiveConf();
+    HiveConf conf = getNewHiveConf();
     String userName;
     Path scratchDirPath;
     // Set a custom prefix for hdfs scratch dir path
@@ -856,7 +859,7 @@ public class TestJdbcWithMiniHS2 {
    */
   @Test
   public void testUdfWhiteBlackList() throws Exception {
-    HiveConf testConf = new HiveConf();
+    HiveConf testConf = getNewHiveConf();
     assertTrue(testConf.getVar(ConfVars.HIVE_SERVER2_BUILTIN_UDF_WHITELIST).isEmpty());
     // verify that udf in default whitelist can be executed
     Statement stmt = conDefault.createStatement();
@@ -892,6 +895,13 @@ public class TestJdbcWithMiniHS2 {
     restoreMiniHS2AndConnections();
   }
 
+  private static HiveConf getNewHiveConf() {
+    HiveConf conf = new HiveConfForTest(TestJdbcWithMiniHS2.class);
+    //TODO: HIVE-28284: TestJdbcWithMiniHS2/TestJdbcWithMiniHS2ErasureCoding to run on Tez
+    conf.setVar(HiveConf.ConfVars.HIVE_EXECUTION_ENGINE, "mr");
+    return conf;
+  }
+
   /** Test UDF blacklist
    *   - verify default value
    *   - verify udfs allowed with default blacklist
@@ -900,7 +910,7 @@ public class TestJdbcWithMiniHS2 {
    */
   @Test
   public void testUdfBlackList() throws Exception {
-    HiveConf testConf = new HiveConf();
+    HiveConf testConf = getNewHiveConf();
     assertTrue(testConf.getVar(ConfVars.HIVE_SERVER2_BUILTIN_UDF_BLACKLIST).isEmpty());
     Statement stmt = conDefault.createStatement();
     // verify that udf in default whitelist can be executed
@@ -931,7 +941,7 @@ public class TestJdbcWithMiniHS2 {
   public void testUdfBlackListOverride() throws Exception {
     stopMiniHS2();
     // setup whitelist
-    HiveConf testConf = new HiveConf();
+    HiveConf testConf = getNewHiveConf();
 
     Set<String> funcNames = FunctionRegistry.getFunctionNames();
     String funcNameStr = "";
@@ -967,7 +977,7 @@ public class TestJdbcWithMiniHS2 {
   public void testRootScratchDir() throws Exception {
     // Stop HiveServer2
     stopMiniHS2();
-    HiveConf conf = new HiveConf();
+    HiveConf conf = getNewHiveConf();
     String userName;
     Path scratchDirPath;
     conf.set("hive.exec.scratchdir", tmpDir + "/hs2");
@@ -1017,7 +1027,7 @@ public class TestJdbcWithMiniHS2 {
   public void testHttpHeaderSize() throws Exception {
     // Stop HiveServer2
     stopMiniHS2();
-    HiveConf conf = new HiveConf();
+    HiveConf conf = getNewHiveConf();
     conf.setIntVar(HiveConf.ConfVars.HIVE_SERVER2_THRIFT_HTTP_REQUEST_HEADER_SIZE, 1024);
     conf.setIntVar(HiveConf.ConfVars.HIVE_SERVER2_THRIFT_HTTP_RESPONSE_HEADER_SIZE, 1024);
     startMiniHS2(conf, true);
@@ -1087,7 +1097,7 @@ public class TestJdbcWithMiniHS2 {
   public void testHttpRetryOnServerIdleTimeout() throws Exception {
     // Stop HiveServer2
     stopMiniHS2();
-    HiveConf conf = new HiveConf();
+    HiveConf conf = getNewHiveConf();
     // Set server's idle timeout to a very low value
     conf.setVar(HiveConf.ConfVars.HIVE_SERVER2_THRIFT_HTTP_MAX_IDLE_TIME, "5000");
     startMiniHS2(conf, true);
@@ -1225,11 +1235,7 @@ public class TestJdbcWithMiniHS2 {
     try {
       constructorCacheField = ReflectionUtil.class.getDeclaredField("CONSTRUCTOR_CACHE");
       if (constructorCacheField != null) {
-        constructorCacheField.setAccessible(true);
-        Field modifiersField = Field.class.getDeclaredField("modifiers");
-        modifiersField.setAccessible(true);
-        modifiersField.setInt(constructorCacheField, constructorCacheField.getModifiers()
-            & ~Modifier.FINAL);
+        ReflectionUtil.setStaticFinalFieldsModifiable(constructorCacheField);
         tmp =
             CacheBuilder.newBuilder().expireAfterAccess(5, TimeUnit.SECONDS).concurrencyLevel(64)
                 .weakKeys().weakValues().build();
