@@ -33,9 +33,7 @@ import com.google.common.io.Resources;
 import java.io.File;
 import java.io.IOException;
 import java.net.InetAddress;
-import java.net.InetSocketAddress;
 import java.net.ServerSocket;
-import java.net.Socket;
 import java.nio.charset.StandardCharsets;
 import java.sql.ResultSet;
 import java.sql.SQLException;
@@ -53,16 +51,13 @@ import java.util.concurrent.Future;
 import org.apache.hadoop.hive.conf.HiveConf;
 import org.apache.hadoop.hive.conf.HiveConf.ConfVars;
 import org.apache.hadoop.hive.metastore.MetaStoreTestUtils;
-import org.apache.hadoop.hive.ql.metadata.TestHive;
 import org.apache.hive.jdbc.HiveConnection;
 import org.apache.hive.jdbc.Utils.JdbcConnectionParams;
 import org.apache.hive.jdbc.miniHS2.MiniHS2;
-import org.apache.hive.jdbc.saml.HiveJdbcBrowserClient;
 import org.apache.hive.jdbc.saml.IJdbcBrowserClient;
 import org.apache.hive.jdbc.saml.IJdbcBrowserClient.HiveJdbcBrowserException;
 import org.apache.hive.jdbc.saml.IJdbcBrowserClientFactory;
 import org.apache.hive.jdbc.saml.SimpleSAMLPhpTestBrowserClient;
-import org.apache.thrift.TException;
 import org.junit.After;
 import org.junit.AfterClass;
 import org.junit.BeforeClass;
@@ -84,6 +79,7 @@ import org.testcontainers.utility.DockerImageName;
  * It uses HTMLUnit to simulate the browser interaction (provide user/password) of the
  * end user.
  */
+@org.junit.Ignore("HIVE-29009")
 public class TestHttpSamlAuthentication {
 
   //user credentials. These much match with the authsources.php of the simpleSAMLPHP
@@ -108,7 +104,7 @@ public class TestHttpSamlAuthentication {
     HiveConf conf = new HiveConf();
     conf.setBoolVar(ConfVars.HIVE_SUPPORT_CONCURRENCY, false);
     conf.setBoolVar(ConfVars.HIVE_SERVER2_LOGGING_OPERATION_ENABLED, false);
-    conf.setBoolVar(ConfVars.HIVESTATSCOLAUTOGATHER, false);
+    conf.setBoolVar(ConfVars.HIVE_STATS_COL_AUTOGATHER, false);
     conf.setVar(ConfVars.HIVE_SERVER2_AUTHENTICATION, "SAML");
     conf.setVar(ConfVars.HIVE_SERVER2_SAML_IDP_METADATA,
         idpMetadataFile.getAbsolutePath());
