@@ -26,7 +26,6 @@ import java.util.Map;
 
 import org.apache.hadoop.fs.FSDataInputStream;
 import org.apache.hadoop.hive.common.ValidTxnList;
-import org.apache.hadoop.hive.common.ValidWriteIdList;
 import org.apache.hadoop.hive.common.metrics.common.Metrics;
 import org.apache.hadoop.hive.common.metrics.common.MetricsConstant;
 import org.apache.hadoop.hive.common.metrics.common.MetricsFactory;
@@ -54,6 +53,7 @@ import org.apache.hadoop.hive.ql.plan.mapper.PlanMapper;
 import org.apache.hadoop.hive.ql.plan.mapper.StatsSource;
 import org.apache.hadoop.hive.ql.processors.CommandProcessorException;
 import org.apache.hadoop.hive.ql.processors.CommandProcessorResponse;
+import org.apache.hadoop.hive.ql.queryhistory.QueryHistoryService;
 import org.apache.hadoop.hive.ql.session.LineageState;
 import org.apache.hadoop.hive.ql.session.SessionState;
 import org.apache.hadoop.hive.ql.session.SessionState.LogHelper;
@@ -70,7 +70,6 @@ import com.google.common.base.Strings;
  * Compiles and executes HQL commands.
  */
 public class Driver implements IDriver {
-
   private static final String CLASS_NAME = Driver.class.getName();
   private static final Logger LOG = LoggerFactory.getLogger(CLASS_NAME);
   private static final LogHelper CONSOLE = new LogHelper(LOG);
@@ -79,8 +78,8 @@ public class Driver implements IDriver {
       "snapshot was outdated when locks were acquired";
 
   private int maxRows = 100;
-
-  private final DriverContext driverContext;
+  @VisibleForTesting
+  final DriverContext driverContext;
   private final DriverState driverState = new DriverState();
   private final DriverTxnHandler driverTxnHandler;
 
@@ -94,7 +93,7 @@ public class Driver implements IDriver {
 
   // Pass lineageState when a driver instantiates another Driver to run or compile another query
   public Driver(HiveConf conf, Context ctx, LineageState lineageState) {
-    this(QueryState.getNewQueryState(conf, lineageState), null);
+    this(QueryState.getNewQueryState(conf, lineageState), QueryInfo.getFromConf(conf));
     context = ctx;
   }
 
@@ -106,17 +105,6 @@ public class Driver implements IDriver {
     this(queryState, queryInfo, null);
   }
 
-  public Driver(QueryState queryState, ValidWriteIdList compactionWriteIds, long compactorTxnId) {
-    this(queryState);
-    driverContext.setCompactionWriteIds(compactionWriteIds);
-    driverContext.setCompactorTxnId(compactorTxnId);
-  }
-
-  public Driver(QueryState queryState, long analyzeTableWriteId) {
-    this(queryState);
-    driverContext.setAnalyzeTableWriteId(analyzeTableWriteId);
-  }
-
   public Driver(QueryState queryState, QueryInfo queryInfo, HiveTxnManager txnManager) {
     driverContext = new DriverContext(queryState, queryInfo, new HookRunner(queryState.getConf(), CONSOLE),
         txnManager);
@@ -155,7 +143,8 @@ public class Driver implements IDriver {
       return new CommandProcessorResponse(getSchema(), null);
     } catch (CommandProcessorException cpe) {
       processRunException(cpe);
-      throw cpe;
+      saveErrorMessageAndRethrow(cpe);
+      return null;
     }
   }
 
@@ -184,6 +173,11 @@ public class Driver implements IDriver {
         driverContext.getPlan().setQueryStartTime(driverContext.getQueryDisplay().getQueryStartTime());
       }
 
+      DriverUtils.checkInterrupted(driverState, driverContext, "at acquiring the lock.", null, null);
+
+      lockAndRespond();
+      validateCurrentSnapshot();
+
       // Reset the PerfLogger so that it doesn't retain any previous values.
       // Any value from compilation phase can be obtained through the map set in queryDisplay during compilation.
       PerfLogger perfLogger = SessionState.getPerfLogger(true);
@@ -191,17 +185,7 @@ public class Driver implements IDriver {
       // the reason that we set the txn manager for the cxt here is because each query has its own ctx object.
       // The txn mgr is shared across the same instance of Driver, which can run multiple queries.
       context.setHiveTxnManager(driverContext.getTxnManager());
-
-      DriverUtils.checkInterrupted(driverState, driverContext, "at acquiring the lock.", null, null);
-
-      lockAndRespond();
-
-      if (validateTxnList()) {
-        // the reason that we set the txn manager for the cxt here is because each query has its own ctx object.
-        // The txn mgr is shared across the same instance of Driver, which can run multiple queries.
-        context.setHiveTxnManager(driverContext.getTxnManager());
-        perfLogger = SessionState.getPerfLogger(true);
-      }
+      
       execute();
 
       FetchTask fetchTask = driverContext.getPlan().getFetchTask();
@@ -235,20 +219,21 @@ public class Driver implements IDriver {
   }
 
   /**
-   * @return If the txn manager should be set.
+   * Checks if the recorded snapshot is up-to-date
    */
-  private boolean validateTxnList() throws CommandProcessorException {
+  private void validateCurrentSnapshot() throws CommandProcessorException {
     int retryShapshotCount = 0;
+    
     int maxRetrySnapshotCount = HiveConf.getIntVar(driverContext.getConf(),
         HiveConf.ConfVars.HIVE_TXN_MAX_RETRYSNAPSHOT_COUNT);
-    boolean shouldSet = false;
-
     try {
       do {
         driverContext.setOutdatedTxn(false);
         // Inserts will not invalidate the snapshot, that could cause duplicates.
+        
         if (!driverTxnHandler.isValidTxnListState()) {
           LOG.info("Re-compiling after acquiring locks, attempt #" + retryShapshotCount);
+          HiveTxnManager txnMgr = driverContext.getTxnManager();
           // Snapshot was outdated when locks were acquired, hence regenerate context, txn list and retry.
           // TODO: Lock acquisition should be moved before analyze, this is a bit hackish.
           // Currently, we acquire a snapshot, compile the query with that snapshot, and then - acquire locks.
@@ -257,10 +242,10 @@ public class Driver implements IDriver {
           if (driverContext.isOutdatedTxn()) {
             // Later transaction invalidated the snapshot, a new transaction is required
             LOG.info("Snapshot is outdated, re-initiating transaction ...");
-            driverContext.getTxnManager().rollbackTxn();
+            txnMgr.rollbackTxn();
 
             String userFromUGI = DriverUtils.getUserFromUGI(driverContext);
-            driverContext.getTxnManager().openTxn(context, userFromUGI, driverContext.getTxnType());
+            txnMgr.openTxn(context, userFromUGI, driverContext.getTxnType());
             lockAndRespond();
           } else {
             // We need to clear the possibly cached writeIds for the prior transaction, so new writeIds
@@ -276,43 +261,36 @@ public class Driver implements IDriver {
             // The scan basically does last writer wins for a given row which is determined by
             // max(committingWriteId) for a given ROW__ID(originalWriteId, bucketId, rowId). So the
             // data add ends up being > than the data delete.
-            driverContext.getTxnManager().clearCaches();
+            txnMgr.clearCaches();
           }
+          driverContext.getConf().unset(ValidTxnList.VALID_TXNS_KEY);
           driverContext.setRetrial(true);
-          driverContext.getConf().set(ValidTxnList.VALID_TXNS_KEY,
-              driverContext.getTxnManager().getValidTxns().toString());
+
+          compileInternal(context.getCmd(), true);
 
           if (driverContext.getPlan().hasAcidResourcesInQuery()) {
-            compileInternal(context.getCmd(), true);
             driverTxnHandler.recordValidWriteIds();
             driverTxnHandler.setWriteIdForAcidFileSinks();
           }
           // Since we're reusing the compiled plan, we need to update its start time for current run
-          driverContext.getPlan().setQueryStartTime(driverContext.getQueryDisplay().getQueryStartTime());
+          driverContext.getPlan().setQueryStartTime(
+              driverContext.getQueryDisplay().getQueryStartTime());
           driverContext.setRetrial(false);
         }
         // Re-check snapshot only in case we had to release locks and open a new transaction,
         // otherwise exclusive locks should protect output tables/partitions in snapshot from concurrent writes.
       } while (driverContext.isOutdatedTxn() && ++retryShapshotCount <= maxRetrySnapshotCount);
 
-      shouldSet = shouldSetTxnManager(retryShapshotCount, maxRetrySnapshotCount);
     } catch (LockException | SemanticException e) {
       DriverUtils.handleHiveException(driverContext, e, 13, null);
     }
 
-    return shouldSet;
-  }
-
-  private boolean shouldSetTxnManager(int retryShapshotCount, int maxRetrySnapshotCount)
-      throws CommandProcessorException {
     if (retryShapshotCount > maxRetrySnapshotCount) {
       // Throw exception
       HiveException e = new HiveException(
           "Operation could not be executed, " + SNAPSHOT_WAS_OUTDATED_WHEN_LOCKS_WERE_ACQUIRED + ".");
       DriverUtils.handleHiveException(driverContext, e, 14, null);
     }
-
-    return retryShapshotCount != 0;
   }
 
   private void setInitialStateForRun(boolean alreadyCompiled) throws CommandProcessorException {
@@ -356,7 +334,7 @@ public class Driver implements IDriver {
       driverTxnHandler.acquireLocksIfNeeded();
     } catch (CommandProcessorException cpe) {
       driverTxnHandler.rollback(cpe);
-      throw cpe;
+      saveErrorMessageAndRethrow(cpe);
     }
   }
 
@@ -367,7 +345,7 @@ public class Driver implements IDriver {
       executor.execute();
     } catch (CommandProcessorException cpe) {
       driverTxnHandler.rollback(cpe);
-      throw cpe;
+      saveErrorMessageAndRethrow(cpe);
     }
   }
 
@@ -436,7 +414,8 @@ public class Driver implements IDriver {
       compileInternal(command, false);
       return new CommandProcessorResponse(getSchema(), null);
     } catch (CommandProcessorException cpe) {
-      throw cpe;
+      saveErrorMessageAndRethrow(cpe);
+      return null;
     } finally {
       if (cleanupTxnList) {
         // Valid txn list might be generated for a query compiled using this command, thus we need to reset it
@@ -475,7 +454,7 @@ public class Driver implements IDriver {
         } catch (LockException e) {
           LOG.warn("Exception in releasing locks", e);
         }
-        throw cpe;
+        saveErrorMessageAndRethrow(cpe);
       }
     }
     //Save compile-time PerfLogging for WebUI.
@@ -574,15 +553,7 @@ public class Driver implements IDriver {
   }
 
   private void setTriggerContext(String queryId) {
-    long queryStartTime;
-    // query info is created by SQLOperation which will have start time of the operation. When JDBC Statement is not
-    // used queryInfo will be null, in which case we take creation of Driver instance as query start time (which is also
-    // the time when query display object is created)
-    if (driverContext.getQueryInfo() != null) {
-      queryStartTime = driverContext.getQueryInfo().getBeginTime();
-    } else {
-      queryStartTime = driverContext.getQueryDisplay().getQueryStartTime();
-    }
+    long queryStartTime = driverContext.getQueryStartTime();
     WmContext wmContext = new WmContext(queryStartTime, queryId);
     context.setWmContext(wmContext);
   }
@@ -758,7 +729,8 @@ public class Driver implements IDriver {
   // Close and release resources within a running query process. Since it runs under
   // driver state COMPILING, EXECUTING or INTERRUPT, it would not have race condition
   // with the releases probably running in the other closing thread.
-  private int closeInProcess(boolean destroyed) {
+  @VisibleForTesting
+  int closeInProcess(boolean destroyed) {
     releaseTaskQueue();
     releasePlan();
     releaseCachedResult();
@@ -774,6 +746,7 @@ public class Driver implements IDriver {
   // is called to stop the query if it is running, clean query results, and release resources.
   @Override
   public void close() {
+    logQueryHistory();
     driverState.lock();
     try {
       releaseTaskQueue();
@@ -793,6 +766,16 @@ public class Driver implements IDriver {
     destroy();
   }
 
+  private void logQueryHistory() {
+    if (driverContext.getConf().getBoolVar(HiveConf.ConfVars.HIVE_QUERY_HISTORY_ENABLED)) {
+      if (driverState.isClosed() || driverState.isDestroyed()) {
+        LOG.warn("Driver instance {} already closed or destroyed, prevent handling query history", this);
+      } else {
+        QueryHistoryService.getInstance().logQuery(driverContext);
+      }
+    }
+  }
+
   // TaskQueue could be released in the query and close processes at same
   // time, which needs to be thread protected.
   private void releaseTaskQueue() {
@@ -843,6 +826,8 @@ public class Driver implements IDriver {
           queryCache.clear();
         }
         queryState.disableHMSCache();
+        // Reset the resourceMap to ensure that previous execution's data is not reused during a retry.
+        queryState.clearResourceMap();
         // Remove any query state reference from the session state
         SessionState.get().removeQueryState(queryState.getQueryId());
       }
@@ -949,4 +934,9 @@ public class Driver implements IDriver {
   public StatsSource getStatsSource() {
     return driverContext.getStatsSource();
   }
+
+  private void saveErrorMessageAndRethrow(CommandProcessorException cpe) throws CommandProcessorException {
+    driverContext.setQueryErrorMessage(cpe.getMessage());
+    throw cpe;
+  }
 }
