@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -18,29 +18,33 @@
 
 package org.apache.hadoop.hive.ql.exec;
 
+import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
+import java.util.Comparator;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
+import java.util.stream.Collectors;
 
 import org.apache.commons.lang3.StringEscapeUtils;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.apache.commons.lang3.exception.ExceptionUtils;
 import org.apache.hadoop.conf.Configurable;
-import org.apache.hadoop.fs.FileStatus;
-import org.apache.hadoop.fs.FileSystem;
+import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.Path;
-import org.apache.hadoop.hive.common.FileUtils;
+import org.apache.hadoop.hive.common.ValidReaderWriteIdList;
+import org.apache.hadoop.hive.common.ValidWriteIdList;
 import org.apache.hadoop.hive.conf.HiveConf;
 import org.apache.hadoop.hive.ql.exec.mr.ExecMapperContext;
 import org.apache.hadoop.hive.ql.io.AcidUtils;
 import org.apache.hadoop.hive.ql.io.HiveContextAwareRecordReader;
 import org.apache.hadoop.hive.ql.io.HiveInputFormat;
+import org.apache.hadoop.hive.ql.io.HiveSequenceFileInputFormat;
 import org.apache.hadoop.hive.ql.io.HiveRecordReader;
 import org.apache.hadoop.hive.ql.metadata.HiveException;
 import org.apache.hadoop.hive.ql.metadata.VirtualColumn;
@@ -60,11 +64,11 @@ import org.apache.hadoop.hive.serde2.objectinspector.StructField;
 import org.apache.hadoop.hive.serde2.objectinspector.StructObjectInspector;
 import org.apache.hadoop.hive.serde2.objectinspector.primitive.PrimitiveObjectInspectorFactory;
 import org.apache.hadoop.hive.serde2.typeinfo.TypeInfoFactory;
-import org.apache.hadoop.hive.serde2.typeinfo.TypeInfoUtils;
 import org.apache.hadoop.io.Writable;
 import org.apache.hadoop.io.WritableComparable;
 import org.apache.hadoop.mapred.InputFormat;
 import org.apache.hadoop.mapred.InputSplit;
+import org.apache.hadoop.mapred.InvalidInputException;
 import org.apache.hadoop.mapred.JobConf;
 import org.apache.hadoop.mapred.JobConfigurable;
 import org.apache.hadoop.mapred.RecordReader;
@@ -72,15 +76,18 @@ import org.apache.hadoop.mapred.Reporter;
 import org.apache.hadoop.util.StringUtils;
 import org.apache.hive.common.util.AnnotationUtils;
 import org.apache.hive.common.util.ReflectionUtil;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import com.google.common.collect.Iterators;
+import com.google.common.collect.Lists;
 
 /**
  * FetchTask implementation.
  **/
 public class FetchOperator implements Serializable {
 
-  static final Log LOG = LogFactory.getLog(FetchOperator.class.getName());
+  static final Logger LOG = LoggerFactory.getLogger(FetchOperator.class.getName());
   static final LogHelper console = new LogHelper(LOG);
 
   public static final String FETCH_OPERATOR_DIRECTORY_LIST =
@@ -104,7 +111,7 @@ public class FetchOperator implements Serializable {
 
   private transient Iterator<Path> iterPath;
   private transient Iterator<PartitionDesc> iterPartDesc;
-  private transient Iterator<FetchInputFormatSplit> iterSplits = Iterators.emptyIterator();
+  private transient Iterator<FetchInputFormatSplit> iterSplits = Collections.emptyIterator();
 
   private transient Path currPath;
   private transient PartitionDesc currDesc;
@@ -133,6 +140,10 @@ public class FetchOperator implements Serializable {
     this.job = job;
     this.work = work;
     this.operator = operator;
+    if (operator instanceof TableScanOperator) {
+      Utilities.addTableSchemaToConf(job,
+          (TableScanOperator) operator);
+    }
     this.vcCols = vcCols;
     this.hasVC = vcCols != null && !vcCols.isEmpty();
     this.isStatReader = work.getTblDesc() == null;
@@ -141,7 +152,13 @@ public class FetchOperator implements Serializable {
     initialize();
   }
 
+  public void setValidWriteIdList(String writeIdStr) {
+    job.set(ValidWriteIdList.VALID_WRITEIDS_KEY, writeIdStr);
+    LOG.debug("FetchOperator set writeIdStr: " + writeIdStr);
+  }
   private void initialize() throws HiveException {
+    ensureCorrectSchemaEvolutionConfigs(job);
+
     if (isStatReader) {
       outputOI = work.getStatRowOI();
       return;
@@ -199,21 +216,25 @@ public class FetchOperator implements Serializable {
    */
   private static final Map<String, InputFormat> inputFormats = new HashMap<String, InputFormat>();
 
-  @SuppressWarnings("unchecked")
-  static InputFormat getInputFormatFromCache(Class<? extends InputFormat> inputFormatClass,
-       JobConf conf) throws IOException {
+  public static InputFormat getInputFormatFromCache(
+      Class<? extends InputFormat> inputFormatClass, Configuration conf) throws IOException {
     if (Configurable.class.isAssignableFrom(inputFormatClass) ||
         JobConfigurable.class.isAssignableFrom(inputFormatClass)) {
       return ReflectionUtil.newInstance(inputFormatClass, conf);
     }
+    // TODO: why is this copy-pasted from HiveInputFormat?
     InputFormat format = inputFormats.get(inputFormatClass.getName());
     if (format == null) {
       try {
         format = ReflectionUtil.newInstance(inputFormatClass, conf);
-        inputFormats.put(inputFormatClass.getName(), format);
+        // HBase input formats are not thread safe today. See HIVE-8808.
+        String inputFormatName = inputFormatClass.getName().toLowerCase();
+        if (!inputFormatName.contains("hbase")) {
+          inputFormats.put(inputFormatClass.getName(), format);
+        }
       } catch (Exception e) {
         throw new IOException("Cannot create an instance of InputFormat class "
-            + inputFormatClass.getName() + " as specified in mapredWork!", e);
+                                  + inputFormatClass.getName() + " as specified in mapredWork!", e);
       }
     }
     return format;
@@ -251,25 +272,6 @@ public class FetchOperator implements Serializable {
     return partValues;
   }
 
-  private boolean getNextPath() throws Exception {
-    while (iterPath.hasNext()) {
-      currPath = iterPath.next();
-      currDesc = iterPartDesc.next();
-      if (isNonNativeTable) {
-        return true;
-      }
-      FileSystem fs = currPath.getFileSystem(job);
-      if (fs.exists(currPath)) {
-        for (FileStatus fStat : listStatusUnderPath(fs, currPath)) {
-          if (fStat.getLen() > 0) {
-            return true;
-          }
-        }
-      }
-    }
-    return false;
-  }
-
   /**
    * Set context for this fetch operator in to the jobconf.
    * This helps InputFormats make decisions based on the scope of the complete
@@ -290,12 +292,26 @@ public class FetchOperator implements Serializable {
     }
   }
 
+  private void ensureCorrectSchemaEvolutionConfigs(JobConf conf) {
+    boolean isSchemaEvolution = HiveConf.getBoolVar(conf, HiveConf.ConfVars.HIVE_SCHEMA_EVOLUTION);
+    boolean isForcePositionalEvolution =
+        HiveConf.getBoolVar(conf, HiveConf.ConfVars.HIVE_ORC_FORCE_POSITIONAL_SCHEMA_EVOLUTION);
+
+    if (!isSchemaEvolution && isForcePositionalEvolution) {
+      LOG.warn(
+          "invalid schema evolution ({}) and force positional evolution ({}) pair, falling back to disabling both",
+          isSchemaEvolution, isForcePositionalEvolution);
+      HiveConf.setBoolVar(conf, HiveConf.ConfVars.HIVE_ORC_FORCE_POSITIONAL_SCHEMA_EVOLUTION, false);
+    }
+  }
+
   private RecordReader<WritableComparable, Writable> getRecordReader() throws Exception {
     if (!iterSplits.hasNext()) {
-      FetchInputFormatSplit[] splits = getNextSplits();
+      List<FetchInputFormatSplit> splits = getNextSplits();
       if (splits == null) {
         return null;
       }
+
       if (!isPartitioned || convertedOI == null) {
         currSerDe = tableSerDe;
         ObjectConverter = null;
@@ -307,7 +323,7 @@ public class FetchOperator implements Serializable {
       if (isPartitioned) {
         row[1] = createPartValue(currDesc, partKeyOI);
       }
-      iterSplits = Arrays.asList(splits).iterator();
+      iterSplits = splits.iterator();
 
       if (LOG.isDebugEnabled()) {
         LOG.debug("Creating fetchTask with deserializer typeinfo: "
@@ -320,7 +336,6 @@ public class FetchOperator implements Serializable {
 
     final FetchInputFormatSplit target = iterSplits.next();
 
-    @SuppressWarnings("unchecked")
     final RecordReader<WritableComparable, Writable> reader = target.getRecordReader(job);
     if (hasVC || work.getSplitSample() != null) {
       currRecReader = new HiveRecordReader<WritableComparable, Writable>(reader, job) {
@@ -346,8 +361,10 @@ public class FetchOperator implements Serializable {
     return currRecReader;
   }
 
-  protected FetchInputFormatSplit[] getNextSplits() throws Exception {
-    while (getNextPath()) {
+  private List<FetchInputFormatSplit> getNextSplits() throws Exception {
+    while (iterPath.hasNext()) {
+      currPath = iterPath.next();
+      currDesc = iterPartDesc.next();
       // not using FileInputFormat.setInputPaths() here because it forces a connection to the
       // default file system - which may or may not be online during pure metadata operations
       job.set("mapred.input.dir", StringUtils.escapeString(currPath.toString()));
@@ -359,34 +376,133 @@ public class FetchOperator implements Serializable {
       Class<? extends InputFormat> formatter = currDesc.getInputFileFormatClass();
       Utilities.copyTableJobPropertiesToConf(currDesc.getTableDesc(), job);
       InputFormat inputFormat = getInputFormatFromCache(formatter, job);
-
-      InputSplit[] splits = inputFormat.getSplits(job, 1);
-      FetchInputFormatSplit[] inputSplits = new FetchInputFormatSplit[splits.length];
-      for (int i = 0; i < splits.length; i++) {
-        inputSplits[i] = new FetchInputFormatSplit(splits[i], inputFormat);
+      if(inputFormat instanceof HiveSequenceFileInputFormat) {
+        // input format could be cached, in which case we need to reset the list of files to fetch
+        ((HiveSequenceFileInputFormat) inputFormat).setFiles(null);
       }
+
+      List<Path> dirs = new ArrayList<>(), dirsWithOriginals = new ArrayList<>();
+      processCurrPathForMmWriteIds(inputFormat, dirs, dirsWithOriginals);
+      if (dirs.isEmpty() && dirsWithOriginals.isEmpty()) {
+        LOG.debug("No valid directories for " + currPath);
+        continue;
+      }
+
+      List<FetchInputFormatSplit> inputSplits = new ArrayList<>();
+      if(inputFormat instanceof HiveSequenceFileInputFormat && this.getWork().getFilesToFetch() != null
+          && !this.getWork().getFilesToFetch().isEmpty() && !this.getWork().isSourceTable()) {
+        HiveSequenceFileInputFormat fileFormat = (HiveSequenceFileInputFormat)inputFormat;
+        fileFormat.setFiles(this.getWork().getFilesToFetch());
+        InputSplit[] splits = inputFormat.getSplits(job, 1);
+        for (int i = 0; i < splits.length; i++) {
+          inputSplits.add(new FetchInputFormatSplit(splits[i], inputFormat));
+        }
+      } else {
+        if (!dirs.isEmpty()) {
+          String inputs = makeInputString(dirs);
+          Utilities.FILE_OP_LOGGER.trace("Setting fetch inputs to {}", inputs);
+          job.set("mapred.input.dir", inputs);
+
+          generateWrappedSplits(inputFormat, inputSplits, job);
+        }
+      }
+
+      if (!dirsWithOriginals.isEmpty()) {
+        String inputs = makeInputString(dirsWithOriginals);
+        Utilities.FILE_OP_LOGGER.trace("Setting originals fetch inputs to {}", inputs);
+        JobConf jobNoRec = HiveInputFormat.createConfForMmOriginalsSplit(job, dirsWithOriginals);
+        jobNoRec.set("mapred.input.dir", inputs);
+        generateWrappedSplits(inputFormat, inputSplits, jobNoRec);
+      }
+
       if (work.getSplitSample() != null) {
         inputSplits = splitSampling(work.getSplitSample(), inputSplits);
       }
-      if (inputSplits.length > 0) {
-        return inputSplits;
+
+      if (inputSplits.isEmpty()) {
+        LOG.debug("No splits for " + currPath);
+        continue;
       }
+      if (HiveConf.getBoolVar(job, HiveConf.ConfVars.HIVE_IN_TEST)) {
+        Collections.sort(inputSplits, new FetchInputFormatSplitComparator());
+      }
+      return inputSplits;
     }
+
     return null;
   }
 
-  private FetchInputFormatSplit[] splitSampling(SplitSample splitSample,
-      FetchInputFormatSplit[] splits) {
+  private void generateWrappedSplits(InputFormat inputFormat, List<FetchInputFormatSplit> inputSplits, JobConf job)
+      throws IOException {
+    InputSplit[] splits = new InputSplit[0];
+    try {
+      splits = inputFormat.getSplits(job, 1);
+    } catch (InvalidInputException iie) {
+      LOG.warn("Input path " + currPath + " is empty", iie);
+    } catch (Exception ex) {
+      Throwable t = ExceptionUtils.getRootCause(ex);
+      if (t instanceof FileNotFoundException || t instanceof InvalidInputException) {
+        LOG.warn("Input path " + currPath + " is empty", t);
+      } else {
+        throw ex;
+      }
+    }
+    for (int i = 0; i < splits.length; i++) {
+      inputSplits.add(new FetchInputFormatSplit(splits[i], inputFormat));
+    }
+  }
+
+  private void processCurrPathForMmWriteIds(InputFormat inputFormat,
+      List<Path> dirs, List<Path> dirsWithOriginals) throws IOException {
+    if (inputFormat instanceof HiveInputFormat) {
+      dirs.add(currPath); // No need to process here.
+    }
+    ValidWriteIdList validWriteIdList;
+    if (AcidUtils.isInsertOnlyTable(currDesc.getTableDesc().getProperties())) {
+      validWriteIdList = extractValidWriteIdList();
+    } else {
+      validWriteIdList = null;  // non-MM case
+    }
+    if (validWriteIdList != null) {
+      Utilities.FILE_OP_LOGGER.info("Processing " + currDesc.getTableName() + " for MM paths");
+    }
+
+    HiveInputFormat.processPathsForMmRead(
+        Lists.newArrayList(currPath), job, validWriteIdList, dirs, dirsWithOriginals);
+  }
+
+  private String makeInputString(List<Path> dirs) {
+    if (dirs == null || dirs.isEmpty()) {
+      return "";
+    }
+    // remove duplicates
+    return dirs.stream()
+        .map(dir -> StringUtils.escapeString(dir.toString()))
+        .distinct()
+        .collect(Collectors.joining(","));
+
+  }
+  private ValidWriteIdList extractValidWriteIdList() {
+    if (currDesc.getTableName() == null || !org.apache.commons.lang3.StringUtils.isBlank(currDesc.getTableName())) {
+      String txnString = job.get(ValidWriteIdList.VALID_WRITEIDS_KEY);
+      LOG.debug("FetchOperator get writeIdStr: " + txnString);
+      return txnString == null ? new ValidReaderWriteIdList() : ValidReaderWriteIdList.fromValue(txnString);
+    }
+    return null;  // not fetching from a table directly but from a temp location
+  }
+
+  private List<FetchInputFormatSplit> splitSampling(SplitSample splitSample,
+      List<FetchInputFormatSplit> splits) {
     long totalSize = 0;
     for (FetchInputFormatSplit split: splits) {
         totalSize += split.getLength();
     }
-    List<FetchInputFormatSplit> result = new ArrayList<FetchInputFormatSplit>(splits.length);
+    List<FetchInputFormatSplit> result = new ArrayList<FetchInputFormatSplit>(splits.size());
     long targetSize = splitSample.getTargetSize(totalSize);
-    int startIndex = splitSample.getSeedNum() % splits.length;
+    int startIndex = splitSample.getSeedNum() % splits.size();
     long size = 0;
-    for (int i = 0; i < splits.length; i++) {
-      FetchInputFormatSplit split = splits[(startIndex + i) % splits.length];
+    for (int i = 0; i < splits.size(); i++) {
+      FetchInputFormatSplit split = splits.get((startIndex + i) % splits.size());
       result.add(split);
       long splitgLength = split.getLength();
       if (size + splitgLength >= targetSize) {
@@ -397,7 +513,7 @@ public class FetchOperator implements Serializable {
       }
       size += splitgLength;
     }
-    return result.toArray(new FetchInputFormatSplit[result.size()]);
+    return result;
   }
 
   /**
@@ -405,6 +521,10 @@ public class FetchOperator implements Serializable {
    * Currently only used by FetchTask.
    **/
   public boolean pushRow() throws IOException, HiveException {
+    if (operator == null) {
+      return false;
+    }
+
     if (work.getRowsComputedUsingStats() != null) {
       for (List<Object> row : work.getRowsComputedUsingStats()) {
         operator.process(row, 0);
@@ -519,10 +639,7 @@ public class FetchOperator implements Serializable {
         currRecReader.close();
         currRecReader = null;
       }
-      if (operator != null) {
-        operator.close(false);
-        operator = null;
-      }
+      closeOperator();
       if (context != null) {
         context.clear();
         context = null;
@@ -530,19 +647,25 @@ public class FetchOperator implements Serializable {
       this.currPath = null;
       this.iterPath = null;
       this.iterPartDesc = null;
-      this.iterSplits = Iterators.emptyIterator();
+      this.iterSplits = Collections.emptyIterator();
     } catch (Exception e) {
       throw new HiveException("Failed with exception " + e.getMessage()
           + StringUtils.stringifyException(e));
     }
   }
 
+  public void closeOperator() throws HiveException {
+    if (operator != null) {
+      operator.close(false);
+      operator = null;
+    }
+  }
+
   /**
    * used for bucket map join
    */
   public void setupContext(List<Path> paths) {
     this.iterPath = paths.iterator();
-    List<PartitionDesc> partitionDescs;
     if (!isPartitioned) {
       this.iterPartDesc = Iterators.cycle(new PartitionDesc(work.getTblDesc(), null));
     } else {
@@ -561,14 +684,13 @@ public class FetchOperator implements Serializable {
   private StructObjectInspector setupOutputObjectInspector() throws HiveException {
     TableDesc tableDesc = work.getTblDesc();
     try {
-      tableSerDe = tableDesc.getDeserializer(job, true);
+      tableSerDe = tableDesc.getSerDe(job, true);
       tableOI = (StructObjectInspector) tableSerDe.getObjectInspector();
       if (!isPartitioned) {
         return getTableRowOI(tableOI);
       }
       partKeyOI = getPartitionKeyOI(tableDesc);
 
-      PartitionDesc partDesc = new PartitionDesc(tableDesc, null);
       List<PartitionDesc> listParts = work.getPartDesc();
       // Chose the table descriptor if none of the partitions is present.
       // For eg: consider the query:
@@ -599,12 +721,16 @@ public class FetchOperator implements Serializable {
   }
 
   private boolean needConversion(PartitionDesc partitionDesc) {
+    boolean isAcid = AcidUtils.isTablePropertyTransactional(partitionDesc.getTableDesc().getProperties());
+    if (Utilities.isSchemaEvolutionEnabled(job, isAcid) && Utilities.isInputFileFormatSelfDescribing(partitionDesc)) {
+      return false;
+    }
     return needConversion(partitionDesc.getTableDesc(), Arrays.asList(partitionDesc));
   }
 
   // if table and all partitions have the same schema and serde, no need to convert
   private boolean needConversion(TableDesc tableDesc, List<PartitionDesc> partDescs) {
-    Class<?> tableSerDe = tableDesc.getDeserializerClass();
+    Class<?> tableSerDe = tableDesc.getSerDeClass();
     SerDeSpec spec = AnnotationUtils.getAnnotation(tableSerDe, SerDeSpec.class);
     if (null == spec) {
       // Serde may not have this optional annotation defined in which case be conservative
@@ -628,31 +754,6 @@ public class FetchOperator implements Serializable {
     return false;
   }
 
-  /**
-   * Lists status for all files under a given path. Whether or not this is recursive depends on the
-   * setting of job configuration parameter mapred.input.dir.recursive.
-   *
-   * @param fs
-   *          file system
-   *
-   * @param p
-   *          path in file system
-   *
-   * @return list of file status entries
-   */
-  private FileStatus[] listStatusUnderPath(FileSystem fs, Path p) throws IOException {
-    boolean recursive = HiveConf.getBoolVar(job, HiveConf.ConfVars.HADOOPMAPREDINPUTDIRRECURSIVE);
-    // If this is in acid format always read it recursively regardless of what the jobconf says.
-    if (!recursive && !AcidUtils.isAcid(p, job)) {
-      return fs.listStatus(p, FileUtils.HIDDEN_FILES_PATH_FILTER);
-    }
-    List<FileStatus> results = new ArrayList<FileStatus>();
-    for (FileStatus stat : fs.listStatus(p, FileUtils.HIDDEN_FILES_PATH_FILTER)) {
-      FileUtils.listStatusRecursively(fs, stat, results);
-    }
-    return results.toArray(new FileStatus[results.size()]);
-  }
-
   // for split sampling. shrinkedLength is checked against IOContext.getCurrentBlockStart,
   // which is from RecordReader.getPos(). So some inputformats which does not support getPos()
   // like HiveHBaseTableInputFormat cannot be used with this (todo)
@@ -674,4 +775,20 @@ public class FetchOperator implements Serializable {
       return inputFormat.getRecordReader(getInputSplit(), job, Reporter.NULL);
     }
   }
+
+  private static class FetchInputFormatSplitComparator implements Comparator<FetchInputFormatSplit> {
+    @Override
+    public int compare(FetchInputFormatSplit a, FetchInputFormatSplit b) {
+      final Path ap = a.getPath();
+      final Path bp = b.getPath();
+      if (ap != null) {
+        return (ap.compareTo(bp));
+      }
+      return Long.signum(a.getLength() - b.getLength());
+    }
+  }
+
+  public Configuration getJobConf() {
+    return job;
+  }
 }
