@@ -24,7 +24,6 @@ import org.apache.hadoop.hive.llap.DebugUtils;
 
 import java.util.Arrays;
 
-import org.apache.hadoop.hive.serde2.io.TimestampWritableV2;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -33,6 +32,7 @@ import java.util.ArrayList;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.function.Consumer;
 
 import org.apache.hadoop.hive.ql.exec.vector.BytesColumnVector;
 import org.apache.hadoop.hive.ql.exec.vector.ColumnVector;
@@ -47,6 +47,7 @@ import org.apache.hadoop.hive.ql.exec.vector.TimestampColumnVector;
 import org.apache.hadoop.hive.ql.exec.vector.UnionColumnVector;
 import org.apache.hadoop.hive.ql.exec.vector.VectorizedRowBatch;
 import org.apache.hadoop.hive.ql.exec.vector.VectorizedRowBatchCtx;
+import org.apache.hadoop.hive.ql.metadata.VirtualColumn;
 import org.apache.hadoop.hive.serde2.io.ByteWritable;
 import org.apache.hadoop.hive.serde2.io.DateWritableV2;
 import org.apache.hadoop.hive.serde2.io.DoubleWritable;
@@ -54,6 +55,7 @@ import org.apache.hadoop.hive.serde2.io.HiveCharWritable;
 import org.apache.hadoop.hive.serde2.io.HiveDecimalWritable;
 import org.apache.hadoop.hive.serde2.io.HiveVarcharWritable;
 import org.apache.hadoop.hive.serde2.io.ShortWritable;
+import org.apache.hadoop.hive.serde2.io.TimestampWritableV2;
 import org.apache.hadoop.hive.serde2.typeinfo.CharTypeInfo;
 import org.apache.hadoop.hive.serde2.typeinfo.ListTypeInfo;
 import org.apache.hadoop.hive.serde2.typeinfo.MapTypeInfo;
@@ -89,6 +91,8 @@ public abstract class BatchToRowReader<StructType, UnionType>
   private final boolean[] included;
   private int rowInBatch = 0;
 
+  protected List<VirtualColumnHandler> virtualColumnHandlers;
+
   public BatchToRowReader(RecordReader<NullWritable, VectorizedRowBatch> vrbReader,
       VectorizedRowBatchCtx vrbCtx, List<Integer> includedCols) {
     this.vrbReader = vrbReader;
@@ -104,15 +108,45 @@ public abstract class BatchToRowReader<StructType, UnionType>
     } else {
       Arrays.fill(included, true);
     }
+
+    virtualColumnHandlers = requestedVirtualColumns();
+    for (VirtualColumnHandler handler : virtualColumnHandlers) {
+      int idx = vrbCtx.findVirtualColumnNum(handler.virtualColumn);
+      if (idx >= 0) {
+        included[idx] = true;
+        handler.indexInSchema = idx;
+        batch.cols[idx].noNulls = false;
+        Arrays.fill(batch.cols[idx].isNull, true);
+      }
+    }
+
     if (LOG.isDebugEnabled()) {
       LOG.debug("Including the columns " + DebugUtils.toString(included));
     }
     this.included = included;
   }
 
+  /**
+   * Wrapper class to map a virtual column to a handler defined by subclasses of {@link BatchToRowReader}.
+   * The handler should be a set operation which sets the value of the virtual column value
+   * in the current row.
+   */
+  public static class VirtualColumnHandler {
+    private final VirtualColumn virtualColumn;
+    private final Consumer<Object> handler;
+    private int indexInSchema = -1;
+
+    public VirtualColumnHandler(VirtualColumn virtualColumn, Consumer<Object> handler) {
+      this.virtualColumn = virtualColumn;
+      this.handler = handler;
+    }
+  }
+
+  protected abstract List<VirtualColumnHandler> requestedVirtualColumns();
   protected abstract StructType createStructObject(Object previous, List<TypeInfo> childrenTypes);
   protected abstract void setStructCol(StructType structObj, int i, Object value);
   protected abstract Object getStructCol(StructType structObj, int i);
+  protected abstract int getStructLength(StructType structObj);
   protected abstract UnionType createUnionObject(List<TypeInfo> childrenTypes, Object previous);
   protected abstract void setUnion(UnionType unionObj, byte tag, Object object);
   protected abstract Object getUnionField(UnionType unionObj);
@@ -138,27 +172,47 @@ public abstract class BatchToRowReader<StructType, UnionType>
   }
 
   @Override
+  @SuppressWarnings("unchecked")
   public boolean next(NullWritable key, Object previous) throws IOException {
     if (!ensureBatch()) {
       return false;
     }
-    @SuppressWarnings("unchecked")
-    StructType value = (StructType)previous;
+
+    virtualColumnHandlers.forEach(handler -> handler.handler.accept(null));
+
+    StructType value = (StructType) previous;
     for (int i = 0; i < schema.size(); ++i) {
-      if (!included[i]) continue; // TODO: shortcut for last col below length?
+      if (!included[i] || i >= getStructLength(value)) continue;
       try {
         setStructCol(value, i,
             nextValue(batch.cols[i], rowInBatch, schema.get(i), getStructCol(value, i)));
       } catch (Throwable t) {
-        LOG.error("Error at row " + rowInBatch + "/" + batch.size + ", column " + i
-            + "/" + schema.size() + " " + batch.cols[i], t);
-        throw (t instanceof IOException) ? (IOException)t : new IOException(t);
+        throwIOException(i, t);
       }
     }
+
+    for (VirtualColumnHandler handler : virtualColumnHandlers) {
+      if (handler.indexInSchema < 0 || !included[handler.indexInSchema] ||
+              handler.indexInSchema >= getStructLength(value)) {
+        continue;
+      }
+      try {
+        handler.handler.accept(getStructCol(value, handler.indexInSchema));
+      } catch (Throwable t) {
+        throwIOException(handler.indexInSchema, t);
+      }
+    }
+
     ++rowInBatch;
     return true;
   }
 
+  private void throwIOException(int i, Throwable t) throws IOException {
+    LOG.error("Error at row " + rowInBatch + "/" + batch.size + ", column " + i
+        + "/" + schema.size() + " " + batch.cols[i], t);
+    throw (t instanceof IOException) ? (IOException) t : new IOException(t);
+  }
+
   /**
    * If the current batch is empty, get a new one.
    * @return true if we have rows available.
