@@ -20,15 +20,21 @@ package org.apache.hadoop.hive.ql.metadata;
 
 import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableList;
+import java.io.IOException;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.Collections;
+import java.util.concurrent.ExecutorService;
+
+import com.google.common.collect.Maps;
 import org.apache.hadoop.conf.Configurable;
+import org.apache.hadoop.fs.FileStatus;
 import org.apache.hadoop.hive.common.classification.InterfaceAudience;
 import org.apache.hadoop.hive.common.classification.InterfaceStability;
 import org.apache.hadoop.hive.common.type.SnapshotContext;
 import org.apache.hadoop.hive.conf.HiveConf;
 import org.apache.hadoop.hive.metastore.HiveMetaHook;
+import org.apache.hadoop.hive.metastore.api.AggrStats;
 import org.apache.hadoop.hive.metastore.api.ColumnStatistics;
 import org.apache.hadoop.hive.metastore.api.ColumnStatisticsObj;
 import org.apache.hadoop.hive.metastore.api.EnvironmentContext;
@@ -55,9 +61,11 @@ import org.apache.hadoop.hive.ql.parse.StorageFormat.StorageHandlerTypes;
 import org.apache.hadoop.hive.ql.parse.TransformSpec;
 import org.apache.hadoop.hive.ql.parse.SemanticException;
 import org.apache.hadoop.hive.ql.parse.UpdateSemanticAnalyzer;
+import org.apache.hadoop.hive.ql.plan.PartitionAwareOptimizationCtx;
 import org.apache.hadoop.hive.ql.plan.DynamicPartitionCtx;
 import org.apache.hadoop.hive.ql.plan.ExprNodeDesc;
 import org.apache.hadoop.hive.ql.plan.FileSinkDesc;
+import org.apache.hadoop.hive.ql.plan.MergeTaskProperties;
 import org.apache.hadoop.hive.ql.plan.OperatorDesc;
 import org.apache.hadoop.hive.ql.plan.TableDesc;
 import org.apache.hadoop.hive.ql.security.authorization.HiveAuthorizationProvider;
@@ -66,7 +74,9 @@ import org.apache.hadoop.hive.ql.stats.Partish;
 import org.apache.hadoop.hive.serde2.AbstractSerDe;
 import org.apache.hadoop.mapred.InputFormat;
 import org.apache.hadoop.mapred.JobConf;
+import org.apache.hadoop.mapred.OutputCommitter;
 import org.apache.hadoop.mapred.OutputFormat;
+import org.apache.hadoop.mapred.TaskAttemptContext;
 
 import java.util.HashMap;
 import java.util.List;
@@ -101,23 +111,23 @@ public interface HiveStorageHandler extends Configurable {
   /**
    * @return Class providing an implementation of {@link InputFormat}
    */
-  public Class<? extends InputFormat> getInputFormatClass();
+  Class<? extends InputFormat> getInputFormatClass();
 
   /**
    * @return Class providing an implementation of {@link OutputFormat}
    */
-  public Class<? extends OutputFormat> getOutputFormatClass();
+  Class<? extends OutputFormat> getOutputFormatClass();
 
   /**
    * @return Class providing an implementation of {@link AbstractSerDe}
    */
-  public Class<? extends AbstractSerDe> getSerDeClass();
+  Class<? extends AbstractSerDe> getSerDeClass();
 
   /**
    * @return metadata hook implementation, or null if this
    * storage handler does not need any metadata notifications
    */
-  public HiveMetaHook getMetaHook();
+  HiveMetaHook getMetaHook();
 
   /**
    * Returns the implementation specific authorization provider
@@ -125,8 +135,7 @@ public interface HiveStorageHandler extends Configurable {
    * @return authorization provider
    * @throws HiveException
    */
-  public HiveAuthorizationProvider getAuthorizationProvider()
-    throws HiveException;
+  HiveAuthorizationProvider getAuthorizationProvider() throws HiveException;
 
   /**
    * This method is called to allow the StorageHandlers the chance
@@ -145,14 +154,13 @@ public interface HiveStorageHandler extends Configurable {
    * @param jobProperties receives properties copied or transformed
    * from the table properties
    */
-  public abstract void configureInputJobProperties(TableDesc tableDesc,
-    Map<String, String> jobProperties);
+  void configureInputJobProperties(TableDesc tableDesc, Map<String, String> jobProperties);
 
   /**
    * This method is called to allow the StorageHandlers the chance to
    * populate secret keys into the job's credentials.
    */
-  public abstract void configureInputJobCredentials(TableDesc tableDesc, Map<String, String> secrets);
+  void configureInputJobCredentials(TableDesc tableDesc, Map<String, String> secrets);
 
   /**
    * This method is called to allow the StorageHandlers the chance
@@ -171,8 +179,7 @@ public interface HiveStorageHandler extends Configurable {
    * @param jobProperties receives properties copied or transformed
    * from the table properties
    */
-  public abstract void configureOutputJobProperties(TableDesc tableDesc,
-    Map<String, String> jobProperties);
+  void configureOutputJobProperties(TableDesc tableDesc, Map<String, String> jobProperties);
 
   /**
    * Deprecated use configureInputJobProperties/configureOutputJobProperties
@@ -187,9 +194,7 @@ public interface HiveStorageHandler extends Configurable {
    * from the table properties
    */
   @Deprecated
-  public void configureTableJobProperties(
-    TableDesc tableDesc,
-    Map<String, String> jobProperties);
+  void configureTableJobProperties(TableDesc tableDesc, Map<String, String> jobProperties);
 
   /**
    * Called just before submitting MapReduce job.
@@ -197,7 +202,7 @@ public interface HiveStorageHandler extends Configurable {
    * @param tableDesc descriptor for the table being accessed
    * @param jobConf jobConf for MapReduce job
    */
-  public void configureJobConf(TableDesc tableDesc, JobConf jobConf);
+  void configureJobConf(TableDesc tableDesc, JobConf jobConf);
 
   /**
    * Used to fetch runtime information about storage handler during DESCRIBE EXTENDED statement
@@ -206,8 +211,7 @@ public interface HiveStorageHandler extends Configurable {
    * @return StorageHandlerInfo containing runtime information about storage handler
    * OR `null` if the storage handler choose to not provide any runtime information.
    */
-  public default StorageHandlerInfo getStorageHandlerInfo(Table table) throws MetaException
-  {
+  default StorageHandlerInfo getStorageHandlerInfo(Table table) throws MetaException {
     return null;
   }
   
@@ -245,15 +249,24 @@ public interface HiveStorageHandler extends Configurable {
   }
 
   /**
-   * Return some basic statistics (numRows, numFiles, totalSize) calculated by the underlying storage handler
+   * Returns basic statistics (numRows, numFiles, totalSize) calculated by the underlying storage handler
    * implementation.
-   * @param partish a partish wrapper class
-   * @return map of basic statistics, can be null
+   * @param partish table/partition wrapper object
+   * @return map of basic statistics
    */
   default Map<String, String> getBasicStatistics(Partish partish) {
     return null;
   }
 
+  /**
+   * Compute basic statistics (numRows, numFiles, totalSize) for the given table/partition.
+   * @param partish table/partition wrapper object
+   * @return map of basic statistics
+   */
+  default Map<String, String> computeBasicStatistics(Partish partish) {
+    return null;
+  }
+
   /**
    * Check if the storage handler can provide basic statistics.
    * @return true if the storage handler can supply the basic statistics
@@ -261,52 +274,78 @@ public interface HiveStorageHandler extends Configurable {
   default boolean canProvideBasicStatistics() {
     return false;
   }
+  
+  /**
+   * Check if the storage handler can provide partition statistics.
+   * @return true if the storage handler can supply the partition statistics
+   */
+  default boolean canProvidePartitionStatistics(org.apache.hadoop.hive.ql.metadata.Table table) {
+    return false;
+  }
 
   /**
-   * Return some col statistics (Lower bounds, Upper bounds, Null value counts, NaN, total counts) calculated by
-   * the underlying storage handler implementation.
-   * @param table
-   * @return A List of Column Statistics Objects, can be null
+   * Returns column statistics (upper/lower bounds, number of Null/NaN values, NDVs, histogram).
+   * @param table table object
+   * @return list of ColumnStatisticsObj objects
    */
-  default List<ColumnStatisticsObj>getColStatistics(org.apache.hadoop.hive.ql.metadata.Table table) {
+  default List<ColumnStatisticsObj> getColStatistics(org.apache.hadoop.hive.ql.metadata.Table table) {
+    return null;
+  }
+
+  /**
+   * Returns an aggregated column statistics for the supplied partition list
+   * @param table table object
+   * @param colNames list of column names
+   * @param partNames list of partition names
+   * @return AggrStats object
+  */ 
+  default AggrStats getAggrColStatsFor(org.apache.hadoop.hive.ql.metadata.Table table, List<String> colNames, 
+        List<String> partNames) throws MetaException {
     return null;
   }
 
   /**
    * Set column stats for non-native tables
-   * @param table
-   * @param colStats
-   * @return boolean
+   * @param table table object
+   * @param colStats list of ColumnStatistics objects
+   * @return true if operation is successful                 
    */
   default boolean setColStatistics(org.apache.hadoop.hive.ql.metadata.Table table, List<ColumnStatistics> colStats) {
     return false;
   }
 
   /**
-   * Check if the storage handler can provide col statistics.
-   * @param tbl
-   * @return true if the storage handler can supply the col statistics
+   * Check if the storage handler can provide column statistics.
+   * @param table table object
+   * @return true if the storage handler can supply the column statistics
    */
-  default boolean canProvideColStatistics(org.apache.hadoop.hive.ql.metadata.Table tbl) {
+  default boolean canProvideColStatistics(org.apache.hadoop.hive.ql.metadata.Table table) {
     return false;
   }
 
   /**
    * Check if the storage handler can set col statistics.
+   * @param table table object
    * @return true if the storage handler can set the col statistics
    */
-  default boolean canSetColStatistics(org.apache.hadoop.hive.ql.metadata.Table tbl) {
+  default boolean canSetColStatistics(org.apache.hadoop.hive.ql.metadata.Table table) {
     return false;
   }
 
   /**
-   * Check if the storage handler answer a few queries like count(1) purely using stats.
+   * Check if the storage handler can answer a few queries like count(1) purely using statistics.
+   * @param partish table/partition wrapper object
    * @return true if the storage handler can answer query using statistics
    */
-  default boolean canComputeQueryUsingStats(org.apache.hadoop.hive.ql.metadata.Table tbl) {
+  default boolean canComputeQueryUsingStats(Partish partish) {
     return false;
   }
-
+  
+  @Deprecated
+  default boolean canComputeQueryUsingStats(org.apache.hadoop.hive.ql.metadata.Table table) {
+    return canComputeQueryUsingStats(Partish.buildFor(table));
+  }
+  
   /**
    *
    * Gets the storage format descriptor to be used for temp table for LOAD data.
@@ -362,9 +401,9 @@ public interface HiveStorageHandler extends Configurable {
    * Any partitioning scheme provided via the standard HiveQL syntax will be honored but stored in someplace
    * other than HMS, depending on the storage handler implementation.
    *
-   * @return whether table should always be unpartitioned from the perspective of HMS
+   * @return whether the table has built-in partitioning support
    */
-  default boolean alwaysUnpartitioned() {
+  default boolean supportsPartitioning() {
     return false;
   }
 
@@ -534,6 +573,10 @@ public interface HiveStorageHandler extends Configurable {
     return null;
   }
 
+  default Map<Integer,List<TransformSpec>> getPartitionTransformSpecs(org.apache.hadoop.hive.ql.metadata.Table table) {
+    return null;
+  }
+
   /**
    * Creates a DynamicPartitionCtx instance that will be set up by the storage handler itself. Useful for non-native
    * tables where partitions are not handled by Hive, and sorting is required in a custom way before writing the table.
@@ -545,8 +588,31 @@ public interface HiveStorageHandler extends Configurable {
   default DynamicPartitionCtx createDPContext(
           HiveConf conf, org.apache.hadoop.hive.ql.metadata.Table table, Operation writeOperation)
       throws SemanticException {
-    Preconditions.checkState(alwaysUnpartitioned(), "Should only be called for table formats where partitioning " +
-        "is not handled by Hive but the table format itself. See alwaysUnpartitioned() method.");
+    Preconditions.checkState(supportsPartitioning(), "Should only be called for table formats where partitioning " +
+        "is not handled by Hive but the table format itself. See supportsPartitioning() method.");
+    return null;
+  }
+
+  /**
+   * Checks whether the table supports Partition-Aware Optimization.
+   * @param table the table
+   * @return true if the table has partition definitions which potentially optimize query plans
+   * @throws SemanticException in case of any error.
+   */
+  default boolean supportsPartitionAwareOptimization(org.apache.hadoop.hive.ql.metadata.Table table)
+      throws SemanticException {
+    return false;
+  }
+
+  /**
+   * Creates a PartitionAwareOptimizationCtx that retains partition definitions applicable to query plan optimization.
+   * Hive can apply special optimization such as Bucket Map Join based on the storage layouts provided by this context.
+   * @param table the table
+   * @return the created PartitionAwareOptimizationCtx, null if this table doesn't support Partition-Aware Optimization
+   * @throws SemanticException in case of any error.
+   */
+  default PartitionAwareOptimizationCtx createPartitionAwareOptimizationContext(
+      org.apache.hadoop.hive.ql.metadata.Table table) throws SemanticException {
     return null;
   }
 
@@ -567,14 +633,27 @@ public interface HiveStorageHandler extends Configurable {
   default boolean commitInMoveTask() {
     return false;
   }
-  
+
   /**
    * Commits the inserts for the non-native tables. Used in the {@link org.apache.hadoop.hive.ql.exec.MoveTask}.
    * @param commitProperties Commit properties which are needed for the handler based commit
    * @param operation the operation type
    * @throws HiveException If there is an error during commit
    */
-  default void storageHandlerCommit(Properties commitProperties, Operation operation) throws HiveException {
+  default void storageHandlerCommit(Properties commitProperties, Operation operation)
+      throws HiveException {
+    storageHandlerCommit(commitProperties, operation, null);
+  }
+
+  /**
+   * Commits the inserts for the non-native tables. Used in the {@link org.apache.hadoop.hive.ql.exec.MoveTask}.
+   * @param commitProperties Commit properties which are needed for the handler based commit
+   * @param operation the operation type
+   * @param executorService an ExecutorService to be used by the StorageHandler (optional)
+   * @throws HiveException If there is an error during commit
+   */
+  default void storageHandlerCommit(Properties commitProperties, Operation operation, ExecutorService executorService)
+      throws HiveException {
     throw new UnsupportedOperationException();
   }
 
@@ -593,6 +672,13 @@ public interface HiveStorageHandler extends Configurable {
     return DEFAULT_ALLOWED_ALTER_OPS.contains(opType);
   }
 
+  /**
+   * Returns an OutputCommitter that belongs to this storage handler if any.
+   */
+  default OutputCommitter getOutputCommitter() {
+    return null;
+  }
+
   /**
    * Check if the underlying storage handler implementation supports truncate operation
    * for non native tables.
@@ -685,6 +771,23 @@ public interface HiveStorageHandler extends Configurable {
   default SnapshotContext getCurrentSnapshotContext(org.apache.hadoop.hive.ql.metadata.Table table) {
     return null;
   }
+  
+  default void validateCurrentSnapshot(TableDesc tableDesc) {
+  }
+
+  /**
+   * Return snapshot metadata of table snapshots which are newer than the specified.
+   * The specified snapshot is excluded.
+   * @param hmsTable table metadata stored in Hive Metastore
+   * @param since the snapshot preceding the oldest snapshot which should be checked.
+   *              The value null means all should be checked.
+   * @return Iterable of {@link SnapshotContext}.
+   */
+  default Iterable<SnapshotContext> getSnapshotContexts(
+      org.apache.hadoop.hive.ql.metadata.Table hmsTable, SnapshotContext since) {
+    return Collections.emptyList();
+  }
+
 
   /**
    * Alter table operations can rely on this to customize the EnvironmentContext to be used during the alter table
@@ -696,23 +799,46 @@ public interface HiveStorageHandler extends Configurable {
       EnvironmentContext environmentContext) {
   }
 
+  /**
+   * Check the operation type of all snapshots which are newer than the specified. The specified snapshot is excluded.
+   * @deprecated
+   * <br>Use {@link HiveStorageHandler#getSnapshotContexts(org.apache.hadoop.hive.ql.metadata.Table hmsTable, SnapshotContext since)}
+   * and check {@link SnapshotContext.WriteOperationType#APPEND}.equals({@link SnapshotContext#getOperation()}).
+   *
+   * @param hmsTable table metadata stored in Hive Metastore
+   * @param since the snapshot preceding the oldest snapshot which should be checked.
+   *              The value null means all should be checked.
+   * @return null if table is empty, true if all snapshots are {@link SnapshotContext.WriteOperationType#APPEND}s, false otherwise.
+   */
+  @Deprecated
   default Boolean hasAppendsOnly(org.apache.hadoop.hive.ql.metadata.Table hmsTable, SnapshotContext since) {
     return null;
   }
 
-  /**
-   * Checks if storage handler supports Show Partitions and returns a list of partitions
-   * @return List of partitions
-   * @throws UnsupportedOperationException
-   * @throws HiveException
-   */
+  @Deprecated
   default List<String> showPartitions(DDLOperationContext context,
       org.apache.hadoop.hive.ql.metadata.Table tbl) throws UnsupportedOperationException, HiveException {
-    throw new UnsupportedOperationException("Storage handler does not support show partitions command");
+    return getPartitionNames(tbl);
   }
 
+  /**
+   * Validates that the provided partitionSpec is a valid according to the current table partitioning.
+   * @param hmsTable {@link org.apache.hadoop.hive.ql.metadata.Table} table metadata stored in Hive Metastore
+   * @param partitionSpec Map of Strings {@link java.util.Map} partition specification
+   */
   default void validatePartSpec(org.apache.hadoop.hive.ql.metadata.Table hmsTable, Map<String, String> partitionSpec)
       throws SemanticException {
+    validatePartSpec(hmsTable, partitionSpec, Context.RewritePolicy.DEFAULT);
+  }
+
+  /**
+   * Validates that the provided partitionSpec is a valid according to the current table partitioning.
+   * @param hmsTable {@link org.apache.hadoop.hive.ql.metadata.Table} table metadata stored in Hive Metastore
+   * @param partitionSpec Map of Strings {@link java.util.Map} partition specification
+   * @param policy {@link org.apache.hadoop.hive.ql.Context.RewritePolicy} compaction rewrite policy
+   */
+  default void validatePartSpec(org.apache.hadoop.hive.ql.metadata.Table hmsTable, Map<String, String> partitionSpec,
+      Context.RewritePolicy policy) throws SemanticException {
     throw new UnsupportedOperationException("Storage handler does not support validation of partition values");
   }
 
@@ -721,12 +847,21 @@ public interface HiveStorageHandler extends Configurable {
     return true;
   }
 
-  default List<String> getPartitionNames(org.apache.hadoop.hive.ql.metadata.Table hmsTable,
+  /**
+   * Returns partitions names for the current table spec that correspond to the provided partition spec.
+   * @param table {@link org.apache.hadoop.hive.ql.metadata.Table} table metadata stored in Hive Metastore
+   * @param partitionSpec Map of Strings {@link java.util.Map} partition specification
+   * @return List of partition names
+   */
+  default List<String> getPartitionNames(org.apache.hadoop.hive.ql.metadata.Table table,
       Map<String, String> partitionSpec) throws SemanticException {
-    throw new UnsupportedOperationException("Storage handler does not support getting partitions " +
-            "by a partition specification.");
+    throw new UnsupportedOperationException("Storage handler does not support getting partition names");
   }
 
+  default List<String> getPartitionNames(org.apache.hadoop.hive.ql.metadata.Table table) throws SemanticException {
+    return getPartitionNames(table, Maps.newHashMap());
+  }
+  
   default ColumnInfo getColumnInfo(org.apache.hadoop.hive.ql.metadata.Table hmsTable, String colName)
       throws SemanticException {
     throw new UnsupportedOperationException("Storage handler does not support getting column type " +
@@ -737,14 +872,112 @@ public interface HiveStorageHandler extends Configurable {
     SearchArgument searchArgument) {
     return false;
   }
+
   default List<FieldSchema> getPartitionKeys(org.apache.hadoop.hive.ql.metadata.Table hmsTable) {
-    throw new UnsupportedOperationException("Storage handler does not support getting partition keys " +
+    throw new UnsupportedOperationException("Storage handler does not support getting partition keys for a table.");
+  }
+
+  /**
+   * Returns a list of partitions which contain any files whose content falls under the provided filter condition.
+   * @param table {@link org.apache.hadoop.hive.ql.metadata.Table} table metadata stored in Hive Metastore
+   * @param filter Iceberg filter expression
+   * @param latestSpecOnly when True, returns partitions with the current spec only; 
+   *                       False - older specs only; 
+   *                       Null - any spec
+   * @return List of Partitions {@link org.apache.hadoop.hive.ql.metadata.Partition}
+   */
+  default List<Partition> getPartitionsByExpr(org.apache.hadoop.hive.ql.metadata.Table table,
+        ExprNodeDesc filter, Boolean latestSpecOnly) throws SemanticException {
+    throw new UnsupportedOperationException("Storage handler does not support getting partitions " +
+        "by generic expressions");
+  }
+
+  default List<Partition> getPartitionsByExpr(org.apache.hadoop.hive.ql.metadata.Table table, ExprNodeDesc filter)
+        throws SemanticException {
+    return getPartitionsByExpr(table, filter, null);
+  }
+
+  /**
+   * Returns partition based on table and partition specification.
+   * @param table {@link org.apache.hadoop.hive.ql.metadata.Table} table metadata stored in Hive Metastore
+   * @param partitionSpec Map of Strings {@link java.util.Map} partition specification
+   * @return Partition {@link org.apache.hadoop.hive.ql.metadata.Partition} 
+   * @throws SemanticException {@link org.apache.hadoop.hive.ql.parse.SemanticException} 
+   */
+  default Partition getPartition(org.apache.hadoop.hive.ql.metadata.Table table, Map<String, String> partitionSpec)
+        throws SemanticException {
+    return getPartition(table, partitionSpec, Context.RewritePolicy.DEFAULT);
+  }
+
+  /**
+   * Returns partition based on table and partition specification.
+   * @param table {@link org.apache.hadoop.hive.ql.metadata.Table} table metadata stored in Hive Metastore
+   * @param partitionSpec Map of Strings {@link java.util.Map} partition specification
+   * @param policy {@link org.apache.hadoop.hive.ql.Context.RewritePolicy} compaction rewrite policy
+   * @return Partition {@link org.apache.hadoop.hive.ql.metadata.Partition}
+   * @throws SemanticException {@link org.apache.hadoop.hive.ql.parse.SemanticException}
+   */
+  default Partition getPartition(org.apache.hadoop.hive.ql.metadata.Table table, Map<String, String> partitionSpec,
+        Context.RewritePolicy policy) throws SemanticException {
+    throw new UnsupportedOperationException("Storage handler does not support getting partition for a table.");
+  }
+
+  /**
+   * Returns a list of partitions based on table and partial partition specification.
+   * @param table {@link org.apache.hadoop.hive.ql.metadata.Table} table metadata stored in Hive Metastore
+   * @param partitionSpec Map of Strings {@link java.util.Map} partition specification
+   * @param latestSpecOnly when True, returns partitions with the current spec only, else - any spec
+   * @return List of Partitions {@link org.apache.hadoop.hive.ql.metadata.Partition}
+   * @throws SemanticException {@link org.apache.hadoop.hive.ql.parse.SemanticException}
+   */
+  default List<Partition> getPartitions(org.apache.hadoop.hive.ql.metadata.Table table,
+        Map<String, String> partitionSpec, boolean latestSpecOnly) throws SemanticException {
+    throw new UnsupportedOperationException("Storage handler does not support getting partitions for a table.");
+  }
+
+  default List<Partition> getPartitions(org.apache.hadoop.hive.ql.metadata.Table table,
+        Map<String, String> partitionSpec) throws SemanticException {
+    return getPartitions(table, partitionSpec, false);
+  }
+
+  default List<Partition> getPartitions(org.apache.hadoop.hive.ql.metadata.Table table) throws SemanticException {
+    return getPartitions(table, Collections.emptyMap());
+  }
+
+  default boolean isPartitioned(org.apache.hadoop.hive.ql.metadata.Table table) {
+    return false;
+  }
+
+  default boolean hasUndergonePartitionEvolution(org.apache.hadoop.hive.ql.metadata.Table table) {
+    throw new UnsupportedOperationException("Storage handler does not support checking if table " +
+        "undergone partition evolution.");
+  }
+
+  /**
+   * Returns true if the table contain any records matching the filter expression, false otherwise
+   * @param hmsTable {@link org.apache.hadoop.hive.ql.metadata.Table} table metadata stored in Hive Metastore
+   * @param filter Iceberg filter expression
+   * @return boolean 
+   */
+  default boolean hasDataMatchingFilterExpr(org.apache.hadoop.hive.ql.metadata.Table hmsTable, ExprNodeDesc filter) {
+    throw new UnsupportedOperationException("Storage handler does not support checking if filter has any match");
+  }
+
+  default boolean supportsMergeFiles() {
+    return false;
+  }
+
+  default List<FileStatus> getMergeTaskInputFiles(Properties properties) throws IOException {
+    throw new UnsupportedOperationException("Storage handler does not support getting merge input files " +
             "for a table.");
   }
 
-  default List<Partition> getPartitionsByExpr(org.apache.hadoop.hive.ql.metadata.Table hmsTable, ExprNodeDesc desc)
-          throws SemanticException {
-    throw new UnsupportedOperationException("Storage handler does not support getting partitions by expression " +
+  default MergeTaskProperties getMergeTaskProperties(Properties properties) {
+    throw new UnsupportedOperationException("Storage handler does not support getting merge input files " +
             "for a table.");
   }
+
+  default void setMergeTaskDeleteProperties(TableDesc tableDesc) {
+    throw new UnsupportedOperationException("Storage handler does not support getting custom delete merge schema.");
+  }
 }
