@@ -22,27 +22,34 @@ import java.io.IOException;
 import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Properties;
 import java.util.Set;
+import java.util.stream.Collectors;
 
-import com.google.common.base.Preconditions;
+import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileStatus;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hive.common.FileUtils;
+import org.apache.hadoop.hive.common.StatsSetupConst;
+import org.apache.hadoop.hive.common.TableName;
 import org.apache.hadoop.hive.conf.HiveConf;
 import org.apache.hadoop.hive.conf.HiveConf.ConfVars;
 import org.apache.hadoop.hive.metastore.HiveMetaStoreUtils;
+import org.apache.hadoop.hive.metastore.api.SourceTable;
+import org.apache.hadoop.hive.metastore.conf.MetastoreConf;
 import org.apache.hadoop.hive.metastore.TableType;
 import org.apache.hadoop.hive.metastore.Warehouse;
-import org.apache.hadoop.hive.metastore.api.CreationMetadata;
+import org.apache.hadoop.hive.metastore.api.ColumnStatistics;
 import org.apache.hadoop.hive.metastore.api.FieldSchema;
 import org.apache.hadoop.hive.metastore.api.MetaException;
 import org.apache.hadoop.hive.metastore.api.Order;
@@ -51,8 +58,10 @@ import org.apache.hadoop.hive.metastore.api.SerDeInfo;
 import org.apache.hadoop.hive.metastore.api.SkewedInfo;
 import org.apache.hadoop.hive.metastore.api.StorageDescriptor;
 import org.apache.hadoop.hive.metastore.api.hive_metastoreConstants;
+import org.apache.hadoop.hive.metastore.utils.MetaStoreServerUtils;
 import org.apache.hadoop.hive.metastore.utils.MetaStoreUtils;
 import org.apache.hadoop.hive.ql.exec.Utilities;
+import org.apache.hadoop.hive.ql.io.AcidUtils;
 import org.apache.hadoop.hive.ql.io.HiveFileFormatUtils;
 import org.apache.hadoop.hive.ql.io.HiveSequenceFileOutputFormat;
 import org.apache.hadoop.hive.ql.parse.BaseSemanticAnalyzer.TableSpec;
@@ -75,6 +84,10 @@ import org.apache.hive.common.util.ReflectionUtil;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import com.google.common.base.Preconditions;
+
+import static org.apache.commons.lang3.StringUtils.isBlank;
+
 /**
  * A Hive Table: is a fundamental unit of data in Hive that shares a common schema/DDL.
  *
@@ -93,16 +106,46 @@ public class Table implements Serializable {
   /**
    * These fields are all cached fields.  The information comes from tTable.
    */
-  private Deserializer deserializer;
+  private transient Deserializer deserializer;
   private Class<? extends OutputFormat> outputFormatClass;
   private Class<? extends InputFormat> inputFormatClass;
   private Path path;
 
   private transient HiveStorageHandler storageHandler;
+  private transient StorageHandlerInfo storageHandlerInfo;
+  private transient MaterializedViewMetadata materializedViewMetadata;
 
-  private transient TableSpec tableSpec;
+  private TableSpec tableSpec;
 
-  private transient boolean materializedTable;
+  private boolean materializedTable;
+
+  /** Note: This is set only for describe table purposes, it cannot be used to verify whether
+   * a materialization is up-to-date or not. */
+  private Boolean outdatedForRewritingMaterializedView;
+
+  /** Constraint related objects */
+  private TableConstraintsInfo tableConstraintsInfo;
+
+  /** Constraint related flags
+   *  This is to track if constraints are retrieved from metastore or not
+   */
+  private boolean isTableConstraintsFetched=false;
+
+  private String metaTable;
+
+  /**
+   * The version of the table. For Iceberg tables this is the snapshotId.
+   */
+  private String asOfVersion = null;
+  private String versionIntervalFrom = null;
+
+  /**
+   * The version of the table at the given timestamp. The format will be parsed with
+   * TimestampTZUtil.parse.
+   */
+  private String asOfTimestamp = null;
+
+  private String snapshotRef;
 
   /**
    * Used only for serialization.
@@ -130,6 +173,26 @@ public class Table implements Serializable {
     this(getEmptyTable(databaseName, tableName));
   }
 
+  /** This api is used by getMetaData which require deep copy of metastore.api.table
+   * and constraints copy
+   */
+  public Table makeCopy() {
+
+    // make deep copy of metastore.api.table
+    Table newTab = new Table(this.getTTable().deepCopy());
+
+    // copy constraints
+    newTab.copyConstraints(this);
+
+    newTab.setAsOfTimestamp(this.asOfTimestamp);
+    newTab.setAsOfVersion(this.asOfVersion);
+    newTab.setVersionIntervalFrom(this.versionIntervalFrom);
+
+    newTab.setMetaTable(this.getMetaTable());
+    newTab.setSnapshotRef(this.getSnapshotRef());
+    return newTab;
+  }
+
   public boolean isDummyTable() {
     return tTable.getTableName().equals(SemanticAnalyzer.DUMMY_TABLE);
   }
@@ -154,7 +217,7 @@ public class Table implements Serializable {
    * Initialize an empty table.
    */
   public static org.apache.hadoop.hive.metastore.api.Table
-  getEmptyTable(String databaseName, String tableName) {
+    getEmptyTable(String databaseName, String tableName) {
     StorageDescriptor sd = new StorageDescriptor();
     {
       sd.setSerdeInfo(new SerDeInfo());
@@ -167,6 +230,8 @@ public class Table implements Serializable {
       // We have to use MetadataTypedColumnsetSerDe because LazySimpleSerDe does
       // not support a table with no columns.
       sd.getSerdeInfo().setSerializationLib(MetadataTypedColumnsetSerDe.class.getName());
+      //TODO setting serializaton format here is hacky. Only lazy simple serde needs it
+      // so should be set by serde only. Setting it here sets it unconditionally.
       sd.getSerdeInfo().getParameters().put(serdeConstants.SERIALIZATION_FORMAT, "1");
       sd.setInputFormat(SequenceFileInputFormat.class.getName());
       sd.setOutputFormat(HiveSequenceFileOutputFormat.class.getName());
@@ -189,7 +254,7 @@ public class Table implements Serializable {
       // set create time
       t.setCreateTime((int) (System.currentTimeMillis() / 1000));
     }
-    // Explictly set the bucketing version
+    // Explicitly set the bucketing version
     t.getParameters().put(hive_metastoreConstants.TABLE_BUCKETING_VERSION,
         "2");
     return t;
@@ -197,17 +262,13 @@ public class Table implements Serializable {
 
   public void checkValidity(Configuration conf) throws HiveException {
     // check for validity
-    String name = tTable.getTableName();
-    if (null == name || name.length() == 0
-        || !MetaStoreUtils.validateName(name, conf)) {
-      throw new HiveException("[" + name + "]: is not a valid table name");
-    }
-    if (0 == getCols().size()) {
+    validateName(conf);
+    if (getCols().isEmpty()) {
       throw new HiveException(
           "at least one column must be specified for the table");
     }
     if (!isView()) {
-      if (null == getDeserializerFromMetaStore(false)) {
+      if (null == getDeserializer(false)) {
         throw new HiveException("must specify a non-null serDe");
       }
       if (null == getInputFormatClass()) {
@@ -229,6 +290,13 @@ public class Table implements Serializable {
     validateColumns(getCols(), getPartCols());
   }
 
+  public void validateName(Configuration conf) throws HiveException {
+    String name = tTable.getTableName();
+    if (StringUtils.isBlank(name) || !MetaStoreUtils.validateName(name, conf)) {
+      throw new HiveException("[" + name + "]: is not a valid table name");
+    }
+  }
+
   public StorageDescriptor getSd() {
     return tTable.getSd();
   }
@@ -259,6 +327,10 @@ public class Table implements Serializable {
     return tTable.getTableName();
   }
 
+  public TableName getFullTableName() {
+    return new TableName(getCatName(), getDbName(), getTableName(), getSnapshotRef());
+  }
+
   final public Path getDataLocation() {
     if (path == null) {
       path = getPath();
@@ -273,10 +345,6 @@ public class Table implements Serializable {
     return deserializer;
   }
 
-  final public Class<? extends Deserializer> getDeserializerClass() throws Exception {
-    return HiveMetaStoreUtils.getDeserializerClass(SessionState.getSessionConf(), tTable);
-  }
-
   final public Deserializer getDeserializer(boolean skipConfError) {
     if (deserializer == null) {
       deserializer = getDeserializerFromMetaStore(skipConfError);
@@ -286,7 +354,7 @@ public class Table implements Serializable {
 
   final public Deserializer getDeserializerFromMetaStore(boolean skipConfError) {
     try {
-      return HiveMetaStoreUtils.getDeserializer(SessionState.getSessionConf(), tTable, skipConfError);
+      return HiveMetaStoreUtils.getDeserializer(SessionState.getSessionConf(), tTable, metaTable, skipConfError);
     } catch (MetaException e) {
       throw new RuntimeException(e);
     }
@@ -299,14 +367,38 @@ public class Table implements Serializable {
     try {
       storageHandler = HiveUtils.getStorageHandler(
           SessionState.getSessionConf(),
-        getProperty(
-          org.apache.hadoop.hive.metastore.api.hive_metastoreConstants.META_TABLE_STORAGE));
+          getProperty(
+              org.apache.hadoop.hive.metastore.api.hive_metastoreConstants.META_TABLE_STORAGE));
     } catch (Exception e) {
       throw new RuntimeException(e);
     }
     return storageHandler;
   }
 
+  public HiveStorageHandler getStorageHandlerWithoutCaching() {
+    if (storageHandler != null || !isNonNative()) {
+      return storageHandler;
+    }
+    try {
+      return HiveUtils.getStorageHandler(SessionState.getSessionConf(),
+          getProperty(org.apache.hadoop.hive.metastore.api.hive_metastoreConstants.META_TABLE_STORAGE));
+    } catch (Exception e) {
+      throw new RuntimeException(e);
+    }
+  }
+
+  public void setStorageHandler(HiveStorageHandler sh){
+    storageHandler = sh;
+  }
+
+  public StorageHandlerInfo getStorageHandlerInfo() {
+    return storageHandlerInfo;
+  }
+
+  public void setStorageHandlerInfo(StorageHandlerInfo storageHandlerInfo) {
+    this.storageHandlerInfo = storageHandlerInfo;
+  }
+
   final public Class<? extends InputFormat> getInputFormatClass() {
     if (inputFormatClass == null) {
       try {
@@ -318,7 +410,7 @@ public class Table implements Serializable {
           inputFormatClass = getStorageHandler().getInputFormatClass();
         } else {
           inputFormatClass = (Class<? extends InputFormat>)
-            Class.forName(className, true, Utilities.getSessionSpecifiedClassLoader());
+              Class.forName(className, true, Utilities.getSessionSpecifiedClassLoader());
         }
       } catch (ClassNotFoundException e) {
         throw new RuntimeException(e);
@@ -365,13 +457,13 @@ public class Table implements Serializable {
     public ValidationFailureSemanticException(String s) {
       super(s);
     }
-  };
+  }
 
   final public void validatePartColumnNames(
       Map<String, String> spec, boolean shouldBeFull) throws SemanticException {
     List<FieldSchema> partCols = tTable.getPartitionKeys();
     final String tableName = Warehouse.getQualifiedName(tTable);
-    if (partCols == null || (partCols.size() == 0)) {
+    if (CollectionUtils.isEmpty(partCols)) {
       if (spec != null) {
         throw new ValidationFailureSemanticException(tableName +
             " table is not partitioned but partition spec exists: " + spec);
@@ -408,14 +500,14 @@ public class Table implements Serializable {
   }
 
   // Please note : Be very careful in using this function. If not used carefully,
-  // you may end up overwriting all the existing properties. If the usecase is to
+  // you may end up overwriting all the existing properties. If the use case is to
   // add or update certain properties use setProperty() instead.
   public void setParameters(Map<String, String> params) {
     tTable.setParameters(params);
   }
 
   public String getProperty(String name) {
-    return tTable.getParameters().get(name);
+    return tTable.getParameters() != null ? tTable.getParameters().get(name) : null;
   }
 
   public boolean isImmutable(){
@@ -424,12 +516,12 @@ public class Table implements Serializable {
   }
 
   public void setTableType(TableType tableType) {
-     tTable.setTableType(tableType.toString());
-   }
+    tTable.setTableType(tableType.toString());
+  }
 
   public TableType getTableType() {
-     return Enum.valueOf(TableType.class, tTable.getTableType());
-   }
+    return Enum.valueOf(TableType.class, tTable.getTableType());
+  }
 
   public ArrayList<StructField> getFields() {
 
@@ -442,9 +534,7 @@ public class Table implements Serializable {
           .getObjectInspector();
       List<? extends StructField> fld_lst = structObjectInspector
           .getAllStructFieldRefs();
-      for (StructField field : fld_lst) {
-        fields.add(field);
-      }
+      fields.addAll(fld_lst);
     } catch (SerDeException e) {
       throw new RuntimeException(e);
     }
@@ -466,72 +556,80 @@ public class Table implements Serializable {
         getProperty(hive_metastoreConstants.TABLE_BUCKETING_VERSION));
   }
 
-   @Override
+  @Override
   public String toString() {
     return tTable.getTableName();
   }
 
-   /* (non-Javadoc)
-    * @see java.lang.Object#hashCode()
-    */
-   @Override
-   public int hashCode() {
-     final int prime = 31;
-     int result = 1;
-     result = prime * result + ((tTable == null) ? 0 : tTable.hashCode());
-     return result;
-   }
-
-   /* (non-Javadoc)
-    * @see java.lang.Object#equals(java.lang.Object)
-    */
-   @Override
-   public boolean equals(Object obj) {
-     if (this == obj) {
-       return true;
-     }
-     if (obj == null) {
-       return false;
-     }
-     if (getClass() != obj.getClass()) {
-       return false;
-     }
-     Table other = (Table) obj;
-     if (tTable == null) {
-       if (other.tTable != null) {
-         return false;
-       }
-     } else if (!tTable.equals(other.tTable)) {
-       return false;
-     }
-     return true;
-   }
+  /* (non-Javadoc)
+   * @see java.lang.Object#hashCode()
+   */
+  @Override
+  public int hashCode() {
+    final int prime = 31;
+    int result = 1;
+    result = prime * result + ((tTable == null) ? 0 : tTable.hashCode());
+    return result;
+  }
 
+  /* (non-Javadoc)
+   * @see java.lang.Object#equals(java.lang.Object)
+   */
+  @Override
+  public boolean equals(Object obj) {
+    if (this == obj) {
+      return true;
+    }
+    if (obj == null) {
+      return false;
+    }
+    if (getClass() != obj.getClass()) {
+      return false;
+    }
+    Table other = (Table) obj;
+    if (tTable == null) {
+      if (other.tTable != null) {
+        return false;
+      }
+    } else if (!tTable.equals(other.tTable)) {
+      return false;
+    }
+    if (!Objects.equals(asOfTimestamp, other.asOfTimestamp)) {
+      return false;
+    }
+    if (!Objects.equals(asOfVersion, other.asOfVersion)) {
+      return false;
+    }
+    if (!Objects.equals(versionIntervalFrom, other.versionIntervalFrom)) {
+      return false;
+    }
+    return true;
+  }
 
   public List<FieldSchema> getPartCols() {
     List<FieldSchema> partKeys = tTable.getPartitionKeys();
     if (partKeys == null) {
-      partKeys = new ArrayList<FieldSchema>();
+      partKeys = new ArrayList<>();
       tTable.setPartitionKeys(partKeys);
     }
     return partKeys;
   }
 
   public FieldSchema getPartColByName(String colName) {
-    for (FieldSchema key : getPartCols()) {
-      if (key.getName().toLowerCase().equals(colName)) {
-        return key;
-      }
-    }
-    return null;
+    return getPartCols().stream()
+      .filter(key -> key.getName().toLowerCase().equals(colName))
+      .findFirst().orElse(null);
   }
 
   public List<String> getPartColNames() {
-    List<String> partColNames = new ArrayList<String>();
-    for (FieldSchema key : getPartCols()) {
-      partColNames.add(key.getName());
-    }
-    return partColNames;
+    List<FieldSchema> partCols = hasNonNativePartitionSupport() ?
+        getStorageHandler().getPartitionKeys(this) : getPartCols();
+    return partCols.stream().map(FieldSchema::getName)
+      .collect(Collectors.toList());
+  }
+
+  public boolean hasNonNativePartitionSupport() {
+    return getStorageHandler() != null && getStorageHandler().supportsPartitioning();
   }
 
   public boolean isPartitionKey(String colName) {
@@ -541,7 +639,7 @@ public class Table implements Serializable {
   // TODO merge this with getBucketCols function
   public String getBucketingDimensionId() {
     List<String> bcols = tTable.getSd().getBucketCols();
-    if (bcols == null || bcols.size() == 0) {
+    if (CollectionUtils.isEmpty(bcols)) {
       return null;
     }
 
@@ -555,7 +653,7 @@ public class Table implements Serializable {
 
   public void setDataLocation(Path path) {
     this.path = path;
-    tTable.getSd().setLocation(path.toString());
+    tTable.getSd().setLocation(path == null ? null : path.toString());
   }
 
   public void unsetDataLocation() {
@@ -571,7 +669,7 @@ public class Table implements Serializable {
     for (String col : bucketCols) {
       if (!isField(col)) {
         throw new HiveException("Bucket columns " + col
-            + " is not part of the table columns (" + getCols() );
+            + " is not part of the table columns (" + getCols());
       }
     }
     tTable.getSd().setBucketCols(bucketCols);
@@ -581,8 +679,7 @@ public class Table implements Serializable {
     tTable.getSd().setSortCols(sortOrder);
   }
 
-  public void setSkewedValueLocationMap(List<String> valList, String dirName)
-      throws HiveException {
+  public void setSkewedValueLocationMap(List<String> valList, String dirName) {
     Map<List<String>, String> mappings = tTable.getSd().getSkewedInfo()
         .getSkewedColValueLocationMaps();
     if (null == mappings) {
@@ -594,12 +691,12 @@ public class Table implements Serializable {
     mappings.put(valList, dirName);
   }
 
-  public Map<List<String>,String> getSkewedColValueLocationMaps() {
+  public Map<List<String>, String> getSkewedColValueLocationMaps() {
     return (tTable.getSd().getSkewedInfo() != null) ? tTable.getSd().getSkewedInfo()
         .getSkewedColValueLocationMaps() : new HashMap<List<String>, String>();
   }
 
-  public void setSkewedColValues(List<List<String>> skewedValues) throws HiveException {
+  public void setSkewedColValues(List<List<String>> skewedValues) {
     tTable.getSd().getSkewedInfo().setSkewedColValues(skewedValues);
   }
 
@@ -608,7 +705,7 @@ public class Table implements Serializable {
         .getSkewedColValues() : new ArrayList<List<String>>();
   }
 
-  public void setSkewedColNames(List<String> skewedColNames) throws HiveException {
+  public void setSkewedColNames(List<String> skewedColNames) {
     tTable.getSd().getSkewedInfo().setSkewedColNames(skewedColNames);
   }
 
@@ -621,7 +718,7 @@ public class Table implements Serializable {
     return tTable.getSd().getSkewedInfo();
   }
 
-  public void setSkewedInfo(SkewedInfo skewedInfo) throws HiveException {
+  public void setSkewedInfo(SkewedInfo skewedInfo) {
     tTable.getSd().setSkewedInfo(skewedInfo);
   }
 
@@ -658,21 +755,22 @@ public class Table implements Serializable {
         return tTable.getSd().getCols();
       } else if (forMs && !shouldStoreFieldsInMetastore(
           SessionState.getSessionConf(), serializationLib, tTable.getParameters())) {
-        return Hive.getFieldsFromDeserializerForMsStorage(this, getDeserializer());
+        return Hive.getFieldsFromDeserializerForMsStorage(this, getDeserializer(), SessionState.getSessionConf());
       } else {
-        return HiveMetaStoreUtils.getFieldsFromDeserializer(getTableName(), getDeserializer());
+        return HiveMetaStoreUtils.getFieldsFromDeserializer(getTableName(), getDeserializer(),
+            SessionState.getSessionConf());
       }
     } catch (Exception e) {
       LOG.error("Unable to get field from serde: " + serializationLib, e);
     }
-    return new ArrayList<FieldSchema>();
+    return Collections.emptyList();
   }
 
   /**
    * Returns a list of all the columns of the table (data columns + partition
    * columns in that order.
    *
-   * @return List<FieldSchema>
+   * @return List&lt;FieldSchema&gt;
    */
   public List<FieldSchema> getAllCols() {
     ArrayList<FieldSchema> f_list = new ArrayList<FieldSchema>();
@@ -724,12 +822,10 @@ public class Table implements Serializable {
       throw new HiveException("Class not found: " + name, e);
     }
   }
-
+  
   public boolean isPartitioned() {
-    if (getPartCols() == null) {
-      return false;
-    }
-    return (getPartCols().size() != 0);
+    return hasNonNativePartitionSupport() ? getStorageHandler().isPartitioned(this) : 
+        CollectionUtils.isNotEmpty(getPartCols());
   }
 
   public void setFields(List<FieldSchema> fields) {
@@ -882,16 +978,24 @@ public class Table implements Serializable {
   /**
    * @return the creation metadata (only for materialized views)
    */
-  public CreationMetadata getCreationMetadata() {
-    return tTable.getCreationMetadata();
+  public MaterializedViewMetadata getMVMetadata() {
+    if (tTable.getCreationMetadata() == null) {
+      return null;
+    }
+    if (materializedViewMetadata == null) {
+      materializedViewMetadata = new MaterializedViewMetadata(tTable.getCreationMetadata());
+    }
+
+    return materializedViewMetadata;
   }
 
   /**
-   * @param creationMetadata
+   * @param materializedViewMetadata
    *          the creation metadata (only for materialized views)
    */
-  public void setCreationMetadata(CreationMetadata creationMetadata) {
-    tTable.setCreationMetadata(creationMetadata);
+  public void setMaterializedViewMetadata(MaterializedViewMetadata materializedViewMetadata) {
+    this.materializedViewMetadata = materializedViewMetadata;
+    tTable.setCreationMetadata(materializedViewMetadata.creationMetadata);
   }
 
   public void clearSerDeInfo() {
@@ -910,7 +1014,7 @@ public class Table implements Serializable {
   }
 
   /**
-   * Creates a partition name -> value spec map object
+   * Creates a partition name -&gt; value spec map object
    *
    * @param tp
    *          Use the information from this partition.
@@ -952,8 +1056,8 @@ public class Table implements Serializable {
 
   public boolean isNonNative() {
     return getProperty(
-      org.apache.hadoop.hive.metastore.api.hive_metastoreConstants.META_TABLE_STORAGE)
-      != null;
+        org.apache.hadoop.hive.metastore.api.hive_metastoreConstants.META_TABLE_STORAGE)
+        != null;
   }
 
   public String getFullyQualifiedName() {
@@ -1010,9 +1114,14 @@ public class Table implements Serializable {
     return tTable.isTemporary();
   }
 
+  public void setTemporary(boolean isTemporary) {
+    tTable.setTemporary(isTemporary);
+  }
+
   public static boolean hasMetastoreBasedSchema(HiveConf conf, String serdeLib) {
     return StringUtils.isEmpty(serdeLib) ||
-        conf.getStringCollection(ConfVars.SERDESUSINGMETASTOREFORSCHEMA.varname).contains(serdeLib);
+        MetastoreConf.getStringCollection(conf,
+            MetastoreConf.ConfVars.SERDES_USING_METASTORE_FOR_SCHEMA).contains(serdeLib);
   }
 
   public static boolean shouldStoreFieldsInMetastore(
@@ -1028,7 +1137,7 @@ public class Table implements Serializable {
     try {
       Class<?> clazz = conf.getClassByName(serdeLib);
       if (!AbstractSerDe.class.isAssignableFrom(clazz))
-       {
+      {
         return true; // The default.
       }
       deserializer = ReflectionUtil.newInstance(
@@ -1064,7 +1173,7 @@ public class Table implements Serializable {
   }
 
   private static String normalize(String colName) throws HiveException {
-    if (!MetaStoreUtils.validateColumnName(colName)) {
+    if (!MetaStoreServerUtils.validateColumnName(colName)) {
       throw new HiveException("Invalid column name '" + colName
           + "' in the table definition");
     }
@@ -1079,7 +1188,199 @@ public class Table implements Serializable {
     this.tableSpec = tableSpec;
   }
 
-  public boolean hasDeserializer() {
-    return deserializer != null;
+  public String getCatalogName() {
+    return this.tTable.getCatName();
   }
-};
+
+  public void setOutdatedForRewriting(Boolean validForRewritingMaterializedView) {
+    this.outdatedForRewritingMaterializedView = validForRewritingMaterializedView;
+  }
+
+  /** Note: This is set only for describe table purposes, it cannot be used to verify whether
+   * a materialization is up-to-date or not. */
+  public Boolean isOutdatedForRewriting() {
+    return outdatedForRewritingMaterializedView;
+  }
+
+  public ColumnStatistics getColStats() {
+    return tTable.isSetColStats() ? tTable.getColStats() : null;
+  }
+
+  /**
+   * Setup the table level stats as if the table is new. Used when setting up Table for a new
+   * table or during replication.
+   */
+  public void setStatsStateLikeNewTable() {
+    if (isPartitioned()) {
+      StatsSetupConst.setStatsStateForCreateTable(getParameters(), null,
+          StatsSetupConst.FALSE);
+    } else {
+      StatsSetupConst.setStatsStateForCreateTable(getParameters(),
+          MetaStoreUtils.getColumnNames(getCols()), StatsSetupConst.TRUE);
+    }
+  }
+
+  /** Constraints related methods
+   *  Note that set apis are used by DESCRIBE only, although get apis return RELY or ENABLE
+   *  constraints DESCRIBE could set all type of constraints
+   * */
+
+  public TableConstraintsInfo getTableConstraintsInfo() {
+
+    if (!isTableConstraintsFetched) {
+      try {
+        tableConstraintsInfo = Hive.get().getTableConstraints(this.getDbName(), this.getTableName(), true, true,
+            this.getTTable() != null ? this.getTTable().getId() : -1);
+        this.isTableConstraintsFetched = true;
+      } catch (HiveException e) {
+        LOG.warn("Cannot retrieve Table Constraints info for table : " + this.getTableName() + " ignoring exception: " + e);
+        tableConstraintsInfo = new TableConstraintsInfo();
+      }
+    }
+    return tableConstraintsInfo;
+  }
+
+  public void setTableConstraintsInfo(TableConstraintsInfo tableConstraintsInfo) {
+    this.tableConstraintsInfo = tableConstraintsInfo;
+    this.isTableConstraintsFetched = true;
+  }
+
+  /* This only return PK which are created with RELY */
+  public PrimaryKeyInfo getPrimaryKeyInfo() {
+    if (!isTableConstraintsFetched) {
+      getTableConstraintsInfo();
+    }
+    return tableConstraintsInfo.getPrimaryKeyInfo();
+  }
+
+  /* This only return FK constraints which are created with RELY */
+  public ForeignKeyInfo getForeignKeyInfo() {
+    if (!isTableConstraintsFetched) {
+      getTableConstraintsInfo();
+    }
+    return tableConstraintsInfo.getForeignKeyInfo();
+  }
+
+  /* This only return UNIQUE constraint defined with RELY */
+  public UniqueConstraint getUniqueKeyInfo() {
+    if (!isTableConstraintsFetched) {
+      getTableConstraintsInfo();
+    }
+    return tableConstraintsInfo.getUniqueConstraint();
+  }
+
+  /* This only return NOT NULL constraint defined with RELY */
+  public NotNullConstraint getNotNullConstraint() {
+    if (!isTableConstraintsFetched) {
+      getTableConstraintsInfo();
+    }
+    return tableConstraintsInfo.getNotNullConstraint();
+  }
+
+  /* This only return DEFAULT constraint defined with ENABLE */
+  public DefaultConstraint getDefaultConstraint() {
+    if (!isTableConstraintsFetched) {
+      getTableConstraintsInfo();
+    }
+    return tableConstraintsInfo.getDefaultConstraint();
+  }
+
+  /* This only return CHECK constraint defined with ENABLE */
+  public CheckConstraint getCheckConstraint() {
+    if (!isTableConstraintsFetched) {
+      getTableConstraintsInfo();
+    }
+    return tableConstraintsInfo.getCheckConstraint();
+  }
+
+  /** This shouldn't use get apis because those api call metastore
+   * to fetch constraints.
+   * getMetaData only need to make a reference copy of existing constraints, even if those are not fetched
+   */
+  public void copyConstraints(final Table tbl) {
+    this.tableConstraintsInfo = tbl.tableConstraintsInfo;
+    this.isTableConstraintsFetched = tbl.isTableConstraintsFetched;
+  }
+
+  /**
+   * This method ignores the write Id, while comparing two tables.
+   *
+   * @param tbl table to compare with
+   * @return
+   */
+  public boolean equalsWithIgnoreWriteId(Table tbl ) {
+    long targetWriteId = getTTable().getWriteId();
+    long entityWriteId = tbl.getTTable().getWriteId();
+    getTTable().setWriteId(0L);
+    tbl.getTTable().setWriteId(0L);
+    boolean result = equals(tbl);
+    getTTable().setWriteId(targetWriteId);
+    tbl.getTTable().setWriteId(entityWriteId);
+    return result;
+  }
+
+  public String getAsOfVersion() {
+    return asOfVersion;
+  }
+
+  public void setAsOfVersion(String asOfVersion) {
+    this.asOfVersion = asOfVersion;
+  }
+
+  public String getVersionIntervalFrom() {
+    return versionIntervalFrom;
+  }
+
+  public void setVersionIntervalFrom(String versionIntervalFrom) {
+    this.versionIntervalFrom = versionIntervalFrom;
+  }
+
+  public String getAsOfTimestamp() {
+    return asOfTimestamp;
+  }
+
+  public void setAsOfTimestamp(String asOfTimestamp) {
+    this.asOfTimestamp = asOfTimestamp;
+  }
+
+  public String getMetaTable() {
+    return metaTable;
+  }
+
+  public void setMetaTable(String metaTable) {
+    this.metaTable = metaTable;
+  }
+
+  public String getSnapshotRef() {
+    return snapshotRef;
+  }
+
+  public void setSnapshotRef(String snapshotRef) {
+    this.snapshotRef = snapshotRef;
+  }
+
+  public SourceTable createSourceTable() {
+    SourceTable sourceTable = new SourceTable();
+    sourceTable.setTable(this.tTable);
+    sourceTable.setInsertedCount(0L);
+    sourceTable.setUpdatedCount(0L);
+    sourceTable.setDeletedCount(0L);
+    return sourceTable;
+  }
+
+  public List<VirtualColumn> getVirtualColumns() {
+    List<VirtualColumn> virtualColumns = new ArrayList<>();
+    if (!isNonNative()) {
+      virtualColumns.addAll(VirtualColumn.getRegistry());
+    }
+    if (isNonNative() && AcidUtils.isNonNativeAcidTable(this)) {
+      virtualColumns.addAll(getStorageHandler().acidVirtualColumns());
+    }
+    if (isNonNative() && getStorageHandler().areSnapshotsSupported() &&
+        isBlank(getMetaTable())) {
+      virtualColumns.add(VirtualColumn.SNAPSHOT_ID);
+    }
+    
+    return virtualColumns;
+  }
+}
