@@ -28,7 +28,7 @@ import java.util.Map.Entry;
 import java.util.Set;
 import java.util.Stack;
 
-import org.apache.commons.lang.StringUtils;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.hadoop.hive.conf.HiveConf;
 import org.apache.hadoop.hive.ql.exec.ColumnInfo;
 import org.apache.hadoop.hive.ql.exec.FileSinkOperator;
@@ -45,7 +45,7 @@ import org.apache.hadoop.hive.ql.exec.UDF;
 import org.apache.hadoop.hive.ql.exec.UDFArgumentException;
 import org.apache.hadoop.hive.ql.exec.Utilities;
 import org.apache.hadoop.hive.ql.lib.Node;
-import org.apache.hadoop.hive.ql.lib.NodeProcessor;
+import org.apache.hadoop.hive.ql.lib.SemanticNodeProcessor;
 import org.apache.hadoop.hive.ql.lib.NodeProcessorCtx;
 import org.apache.hadoop.hive.ql.metadata.HiveException;
 import org.apache.hadoop.hive.ql.metadata.VirtualColumn;
@@ -63,13 +63,11 @@ import org.apache.hadoop.hive.ql.plan.JoinCondDesc;
 import org.apache.hadoop.hive.ql.plan.JoinDesc;
 import org.apache.hadoop.hive.ql.plan.ReduceSinkDesc;
 import org.apache.hadoop.hive.ql.plan.TableScanDesc;
-import org.apache.hadoop.hive.ql.udf.UDFType;
 import org.apache.hadoop.hive.ql.udf.generic.GenericUDF;
 import org.apache.hadoop.hive.ql.udf.generic.GenericUDF.DeferredJavaObject;
 import org.apache.hadoop.hive.ql.udf.generic.GenericUDFBaseCompare;
 import org.apache.hadoop.hive.ql.udf.generic.GenericUDFBridge;
-import org.apache.hadoop.hive.ql.udf.generic.GenericUDFCase;
-import org.apache.hadoop.hive.ql.udf.generic.GenericUDFNvl;
+import org.apache.hadoop.hive.ql.udf.generic.GenericUDFCoalesce;
 import org.apache.hadoop.hive.ql.udf.generic.GenericUDFOPAnd;
 import org.apache.hadoop.hive.ql.udf.generic.GenericUDFOPEqual;
 import org.apache.hadoop.hive.ql.udf.generic.GenericUDFOPNot;
@@ -82,6 +80,7 @@ import org.apache.hadoop.hive.ql.udf.generic.GenericUDFToUnixTimeStamp;
 import org.apache.hadoop.hive.ql.udf.generic.GenericUDFUnixTimeStamp;
 import org.apache.hadoop.hive.ql.udf.generic.GenericUDFWhen;
 import org.apache.hadoop.hive.serde.serdeConstants;
+import org.apache.hadoop.hive.serde2.avro.AvroSerDe;
 import org.apache.hadoop.hive.serde2.objectinspector.ConstantObjectInspector;
 import org.apache.hadoop.hive.serde2.objectinspector.ObjectInspector;
 import org.apache.hadoop.hive.serde2.objectinspector.ObjectInspector.Category;
@@ -121,7 +120,7 @@ public final class ConstantPropagateProcFactory {
   /**
    * Get ColumnInfo from column expression.
    *
-   * @param rr
+   * @param rs
    * @param desc
    * @return
    */
@@ -183,28 +182,22 @@ public final class ConstantPropagateProcFactory {
     if (unSupportedTypes.contains(priti.getPrimitiveCategory())
         || unSupportedTypes.contains(descti.getPrimitiveCategory())) {
       // FIXME: support template types. It currently has conflict with ExprNodeConstantDesc
-      if (LOG.isDebugEnabled()) {
-        LOG.debug("Unsupported types " + priti + "; " + descti);
-      }
+      LOG.debug("Unsupported types {}; {}", priti, descti);
       return null;
     }
 
     // We shouldn't cast strings to other types because that can break original data in cases of
     // leading zeros or zeros trailing after decimal point.
     // Example: "000126" => 126 => "126"
-    boolean brokenDataTypesCombination = unsafeConversionTypes.contains(
-        priti.getPrimitiveCategory()) && !unsafeConversionTypes.contains(
-            descti.getPrimitiveCategory());
+    boolean brokenDataTypesCombination = unsafeConversionTypes.contains(priti.getPrimitiveCategory())
+        && !unsafeConversionTypes.contains(descti.getPrimitiveCategory())
+        || priti.getPrimitiveCategory() == PrimitiveCategory.TIMESTAMP && descti.getPrimitiveCategory() == PrimitiveCategory.DATE;
     if (performSafeTypeCast && brokenDataTypesCombination) {
-      if (LOG.isDebugEnabled()) {
-        LOG.debug("Unsupported cast " + priti + "; " + descti);
-      }
+      LOG.debug("Unsupported cast {}; {}", priti, descti);
       return null;
     }
 
-    if (LOG.isDebugEnabled()) {
-      LOG.debug("Casting " + desc + " to type " + ti);
-    }
+    LOG.debug("Casting {} to type {}", desc, ti);
     ExprNodeConstantDesc c = (ExprNodeConstantDesc) desc;
     if (null != c.getFoldedFromVal() && priti.getTypeName().equals(serdeConstants.STRING_TYPE_NAME)) {
       // avoid double casting to preserve original string representation of constant.
@@ -220,22 +213,21 @@ public final class ConstantPropagateProcFactory {
     // Convert integer related types because converters are not sufficient
     if (convObj instanceof Integer) {
       switch (priti.getPrimitiveCategory()) {
-        case BYTE:
-          convObj = new Byte((byte) (((Integer) convObj).intValue()));
-          break;
-        case SHORT:
-          convObj = new Short((short) ((Integer) convObj).intValue());
-          break;
-        case LONG:
-          convObj = new Long(((Integer) convObj).intValue());
-        default:
+      case BYTE:
+        convObj = Byte.valueOf((((Integer) convObj).byteValue()));
+        break;
+      case SHORT:
+        convObj = Short.valueOf(((Integer) convObj).shortValue());
+        break;
+      case LONG:
+        convObj = Long.valueOf(((Integer) convObj).intValue());
+      default:
       }
     }
     return new ExprNodeConstantDesc(ti, convObj);
   }
 
   public static ExprNodeDesc foldExpr(ExprNodeGenericFuncDesc funcDesc) {
-
     GenericUDF udf = funcDesc.getGenericUDF();
     if (!isConstantFoldableUdf(udf, funcDesc.getChildren())) {
       return funcDesc;
@@ -357,7 +349,7 @@ public final class ConstantPropagateProcFactory {
       // Don't evaluate nondeterministic function since the value can only calculate during runtime.
       if (!isConstantFoldableUdf(udf, newExprs)) {
         if (LOG.isDebugEnabled()) {
-          LOG.debug("Function " + udf.getClass() + " is undeterministic. Don't evaluate immediately.");
+          LOG.debug("Function " + udf.getClass() + " is not deterministic. Don't evaluate immediately.");
         }
         ((ExprNodeGenericFuncDesc) desc).setChildren(newExprs);
         return desc;
@@ -366,9 +358,7 @@ public final class ConstantPropagateProcFactory {
       // Check if the function can be short cut.
       ExprNodeDesc shortcut = shortcutFunction(udf, newExprs, op);
       if (shortcut != null) {
-        if (LOG.isDebugEnabled()) {
-          LOG.debug("Folding expression:" + desc + " -> " + shortcut);
-        }
+        LOG.debug("Folding expression:{} -> {}", desc, shortcut);
         return shortcut;
       }
       ((ExprNodeGenericFuncDesc) desc).setChildren(newExprs);
@@ -416,7 +406,7 @@ public final class ConstantPropagateProcFactory {
       // Don't evaluate nondeterministic function since the value can only calculate during runtime.
       if (!isConstantFoldableUdf(udf, newExprs)) {
         if (LOG.isDebugEnabled()) {
-          LOG.debug("Function " + udf.getClass() + " is undeterministic. Don't evaluate immediately.");
+          LOG.debug("Function " + udf.getClass() + " is not deterministic. Don't evaluate immediately.");
         }
         ((ExprNodeGenericFuncDesc) desc).setChildren(newExprs);
         return desc;
@@ -424,17 +414,13 @@ public final class ConstantPropagateProcFactory {
         // If all child expressions of deterministic function are constants, evaluate such UDF immediately
         ExprNodeDesc constant = evaluateFunction(udf, newExprs, desc.getChildren());
         if (constant != null) {
-          if (LOG.isDebugEnabled()) {
-            LOG.debug("Folding expression:" + desc + " -> " + constant);
-          }
+          LOG.debug("Folding expression: {} -> {}", desc, constant);
           return constant;
         } else {
           // Check if the function can be short cut.
           ExprNodeDesc shortcut = shortcutFunction(udf, newExprs, op);
           if (shortcut != null) {
-            if (LOG.isDebugEnabled()) {
-              LOG.debug("Folding expression:" + desc + " -> " + shortcut);
-            }
+            LOG.debug("Folding expression: {} -> {}", desc, shortcut);
             return shortcut;
           }
           ((ExprNodeGenericFuncDesc) desc).setChildren(newExprs);
@@ -456,9 +442,7 @@ public final class ConstantPropagateProcFactory {
       Operator<? extends Serializable> parent = op.getParentOperators().get(tag);
       ExprNodeDesc col = evaluateColumn((ExprNodeColumnDesc) desc, cppCtx, parent);
       if (col != null) {
-        if (LOG.isDebugEnabled()) {
-          LOG.debug("Folding expression:" + desc + " -> " + col);
-        }
+        LOG.debug("Folding expression: {} -> {}", desc, col);
         return col;
       }
     }
@@ -471,7 +455,7 @@ public final class ConstantPropagateProcFactory {
   private static boolean isConstantFoldableUdf(GenericUDF udf,  List<ExprNodeDesc> children) {
     // Runtime constants + deterministic functions can be folded.
     if (!FunctionRegistry.isConsistentWithinQuery(udf)) {
-      if (udf.getClass().equals(GenericUDFUnixTimeStamp.class) 
+      if (udf.getClass().equals(GenericUDFUnixTimeStamp.class)
           && children != null && children.size() > 0) {
         // unix_timestamp is polymorphic (ignore class annotations)
         return true;
@@ -531,18 +515,22 @@ public final class ConstantPropagateProcFactory {
         return;
       }
       // If both sides are constants, there is nothing to propagate
-      ExprNodeColumnDesc c = ExprNodeDescUtils.getColumnExpr(lOperand);
-      if (null == c) {
-        c = ExprNodeDescUtils.getColumnExpr(rOperand);
-      }
-      if (null == c) {
-        // we need a column expression on other side.
+      ExprNodeColumnDesc c;
+      if (lOperand instanceof ExprNodeColumnDesc) {
+        c = (ExprNodeColumnDesc)lOperand;
+      } else if (rOperand instanceof ExprNodeColumnDesc) {
+        c = (ExprNodeColumnDesc)rOperand;
+      } else {
+        // we need a column expression on other side
+        // NOTE: we also cannot rely on column expressions wrapped inside casts as casting might
+        // truncate information
         return;
       }
       ColumnInfo ci = resolveColumn(rs, c);
       if (ci != null) {
         if (LOG.isDebugEnabled()) {
-          LOG.debug("Filter " + udf + " is identified as a value assignment, propagate it.");
+          LOG.debug("Filter {} is identified as a value assignment, propagate it.",
+              udf.getDisplayString(new String[]{lOperand.getExprString(), rOperand.getExprString()}));
         }
         if (!v.getTypeInfo().equals(ci.getType())) {
           v = typeCast(v, ci.getType(), true);
@@ -555,7 +543,8 @@ public final class ConstantPropagateProcFactory {
       ExprNodeDesc operand = newExprs.get(0);
       if (operand instanceof ExprNodeColumnDesc) {
         if (LOG.isDebugEnabled()) {
-          LOG.debug("Filter " + udf + " is identified as a value assignment, propagate it.");
+          LOG.debug("Filter {} is identified as a value assignment, propagate it.",
+              udf.getDisplayString(new String[]{operand.getExprString()}));
         }
         ExprNodeColumnDesc c = (ExprNodeColumnDesc) operand;
         ColumnInfo ci = resolveColumn(rs, c);
@@ -574,13 +563,13 @@ public final class ConstantPropagateProcFactory {
      ExprNodeGenericFuncDesc caseOrWhenexpr = null;
      if (newExprs.get(0) instanceof ExprNodeGenericFuncDesc) {
        caseOrWhenexpr = (ExprNodeGenericFuncDesc) newExprs.get(0);
-       if (caseOrWhenexpr.getGenericUDF() instanceof GenericUDFWhen || caseOrWhenexpr.getGenericUDF() instanceof GenericUDFCase) {
+       if (caseOrWhenexpr.getGenericUDF() instanceof GenericUDFWhen) {
          foundUDFInFirst = true;
        }
      }
      if (!foundUDFInFirst && newExprs.get(1) instanceof ExprNodeGenericFuncDesc) {
        caseOrWhenexpr = (ExprNodeGenericFuncDesc) newExprs.get(1);
-       if (!(caseOrWhenexpr.getGenericUDF() instanceof GenericUDFWhen || caseOrWhenexpr.getGenericUDF() instanceof GenericUDFCase)) {
+       if (!(caseOrWhenexpr.getGenericUDF() instanceof GenericUDFWhen)) {
          return null;
        }
      }
@@ -589,7 +578,7 @@ public final class ConstantPropagateProcFactory {
        return null;
      }
      GenericUDF childUDF = caseOrWhenexpr.getGenericUDF();
-     List<ExprNodeDesc> children = caseOrWhenexpr.getChildren();
+      List<ExprNodeDesc> children = new ArrayList(caseOrWhenexpr.getChildren());
      int i;
      if (childUDF instanceof GenericUDFWhen) {
        for (i = 1; i < children.size(); i+=2) {
@@ -606,21 +595,6 @@ public final class ConstantPropagateProcFactory {
        ExprNodeGenericFuncDesc newCaseOrWhenExpr = ExprNodeGenericFuncDesc.newInstance(childUDF,
            caseOrWhenexpr.getFuncText(), children);
        return newCaseOrWhenExpr;
-     } else if (childUDF instanceof GenericUDFCase) {
-       for (i = 2; i < children.size(); i+=2) {
-         children.set(i, ExprNodeGenericFuncDesc.newInstance(new GenericUDFOPEqual(),
-             Lists.newArrayList(children.get(i),newExprs.get(foundUDFInFirst ? 1 : 0))));
-       }
-        if(children.size() % 2 == 0) {
-          i = children.size()-1;
-          children.set(i, ExprNodeGenericFuncDesc.newInstance(new GenericUDFOPEqual(),
-              Lists.newArrayList(children.get(i),newExprs.get(foundUDFInFirst ? 1 : 0))));
-        }
-       // after constant folding of child expression the return type of UDFCase might have changed,
-       // so recreate the expression
-       ExprNodeGenericFuncDesc newCaseOrWhenExpr = ExprNodeGenericFuncDesc.newInstance(childUDF,
-           caseOrWhenexpr.getFuncText(), children);
-       return newCaseOrWhenExpr;
      } else {
        // cant happen
        return null;
@@ -640,8 +614,10 @@ public final class ConstantPropagateProcFactory {
             // if true, prune it
             positionsToRemove.set(i);
           } else {
-            // if false, return false
-            return childExpr;
+            if (Boolean.FALSE.equals(c.getValue())) {
+              // if false, return false
+              return childExpr;
+            }
           }
         } else if (childExpr instanceof ExprNodeGenericFuncDesc &&
                 ((ExprNodeGenericFuncDesc)childExpr).getGenericUDF() instanceof GenericUDFOPNotNull &&
@@ -762,60 +738,7 @@ public final class ConstantPropagateProcFactory {
           List<ExprNodeDesc> children = new ArrayList<>();
           children.add(whenExpr);
           children.add(new ExprNodeConstantDesc(false));
-          ExprNodeGenericFuncDesc func = ExprNodeGenericFuncDesc.newInstance(new GenericUDFNvl(),
-              children);
-          if (Boolean.TRUE.equals(thenVal)) {
-            return func;
-          } else {
-            List<ExprNodeDesc> exprs = new ArrayList<>();
-            exprs.add(func);
-            return ExprNodeGenericFuncDesc.newInstance(new GenericUDFOPNot(), exprs);
-          }
-        } else {
-          return null;
-        }
-      }
-    }
-
-    if (udf instanceof GenericUDFCase) {
-      // HIVE-9644 Attempt to fold expression like :
-      // where (case ss_sold_date when '1998-01-01' then 1=1 else null=1 end);
-      // where ss_sold_date= '1998-01-01' ;
-      if (!(newExprs.size() == 3 || newExprs.size() == 4)) {
-        // In general case can have unlimited # of branches,
-        // we currently only handle either 1 or 2 branch.
-        return null;
-      }
-      ExprNodeDesc thenExpr = newExprs.get(2);
-      ExprNodeDesc elseExpr = newExprs.size() == 4 ? newExprs.get(3) :
-        new ExprNodeConstantDesc(newExprs.get(2).getTypeInfo(),null);
-
-      if (thenExpr instanceof ExprNodeConstantDesc && elseExpr instanceof ExprNodeConstantDesc) {
-        ExprNodeConstantDesc constThen = (ExprNodeConstantDesc) thenExpr;
-        ExprNodeConstantDesc constElse = (ExprNodeConstantDesc) elseExpr;
-        Object thenVal = constThen.getValue();
-        Object elseVal = constElse.getValue();
-        if (thenVal == null) {
-          if (null == elseVal) {
-            return thenExpr;
-          } else if (op instanceof FilterOperator) {
-            return Boolean.TRUE.equals(elseVal) ? ExprNodeGenericFuncDesc.newInstance(new GenericUDFOPNotEqual(), newExprs.subList(0, 2)) :
-              Boolean.FALSE.equals(elseVal) ? elseExpr : null;
-          } else {
-            return null;
-          }
-        } else if (null == elseVal && op instanceof FilterOperator) {
-            return Boolean.TRUE.equals(thenVal) ? ExprNodeGenericFuncDesc.newInstance(new GenericUDFOPEqual(), newExprs.subList(0, 2)) :
-              Boolean.FALSE.equals(thenVal) ? thenExpr : null;
-        } else if(thenVal.equals(elseVal)){
-          return thenExpr;
-        } else if (thenVal instanceof Boolean && elseVal instanceof Boolean) {
-          ExprNodeGenericFuncDesc equal = ExprNodeGenericFuncDesc.newInstance(
-              new GenericUDFOPEqual(), newExprs.subList(0, 2));
-          List<ExprNodeDesc> children = new ArrayList<>();
-          children.add(equal);
-          children.add(new ExprNodeConstantDesc(false));
-          ExprNodeGenericFuncDesc func = ExprNodeGenericFuncDesc.newInstance(new GenericUDFNvl(),
+          ExprNodeGenericFuncDesc func = ExprNodeGenericFuncDesc.newInstance(new GenericUDFCoalesce(),
               children);
           if (Boolean.TRUE.equals(thenVal)) {
             return func;
@@ -975,8 +898,8 @@ public final class ConstantPropagateProcFactory {
             ObjectInspectorUtils.copyToStandardJavaObject(o, coi));
       } else if (!PrimitiveObjectInspectorUtils.isPrimitiveJavaClass(clz)) {
         if (LOG.isErrorEnabled()) {
-          LOG.error("Unable to evaluate " + udf
-              + ". Return value unrecoginizable.");
+          LOG.error("Unable to evaluate {}({}). Return value unrecognizable.",
+              udf.getClass().getName(), exprs);
         }
         return null;
       } else {
@@ -989,8 +912,8 @@ public final class ConstantPropagateProcFactory {
       }
       return new ExprNodeConstantDesc(o).setFoldedFromVal(constStr);
     } catch (HiveException e) {
-      LOG.error("Evaluation function " + udf.getClass()
-          + " failed in Constant Propagation Optimizer.");
+      LOG.error("Evaluation function {}({}) failed in Constant Propagation Optimizer.",
+          udf.getClass().getName(), exprs);
       throw new RuntimeException(e);
     }
   }
@@ -1038,7 +961,7 @@ public final class ConstantPropagateProcFactory {
    * Node Processor for Constant Propagation on Filter Operators. The processor is to fold
    * conditional expressions and extract assignment expressions and propagate them.
    */
-  public static class ConstantPropagateFilterProc implements NodeProcessor {
+  public static class ConstantPropagateFilterProc implements SemanticNodeProcessor {
     @Override
     public Object process(Node nd, Stack<Node> stack, NodeProcessorCtx ctx, Object... nodeOutputs)
         throws SemanticException {
@@ -1056,13 +979,9 @@ public final class ConstantPropagateProcFactory {
         ExprNodeConstantDesc c = (ExprNodeConstantDesc) newCondn;
         if (Boolean.TRUE.equals(c.getValue())) {
           cppCtx.addOpToDelete(op);
-          if (LOG.isDebugEnabled()) {
-            LOG.debug("Filter expression " + condn + " holds true. Will delete it.");
-          }
+          LOG.debug("Filter expression {} holds true. Will delete it", condn);
         } else if (Boolean.FALSE.equals(c.getValue())) {
-          if (LOG.isWarnEnabled()) {
-            LOG.warn("Filter expression " + condn + " holds false!");
-          }
+          LOG.warn("Filter expression {} holds false!", condn);
         }
       }
       if (newCondn instanceof ExprNodeConstantDesc && ((ExprNodeConstantDesc)newCondn).getValue() == null) {
@@ -1093,7 +1012,7 @@ public final class ConstantPropagateProcFactory {
   /**
    * Node Processor for Constant Propagate for Group By Operators.
    */
-  public static class ConstantPropagateGroupByProc implements NodeProcessor {
+  public static class ConstantPropagateGroupByProc implements SemanticNodeProcessor {
     @Override
     public Object process(Node nd, Stack<Node> stack, NodeProcessorCtx ctx, Object... nodeOutputs)
         throws SemanticException {
@@ -1119,7 +1038,7 @@ public final class ConstantPropagateProcFactory {
       }
 
       GroupByDesc conf = op.getConf();
-      ArrayList<ExprNodeDesc> keys = conf.getKeys();
+      List<ExprNodeDesc> keys = conf.getKeys();
       for (int i = 0; i < keys.size(); i++) {
         ExprNodeDesc key = keys.get(i);
         ExprNodeDesc newkey = foldExpr(key, colToConstants, cppCtx, op, 0, false);
@@ -1142,7 +1061,7 @@ public final class ConstantPropagateProcFactory {
   /**
    * The Default Node Processor for Constant Propagation.
    */
-  public static class ConstantPropagateDefaultProc implements NodeProcessor {
+  public static class ConstantPropagateDefaultProc implements SemanticNodeProcessor {
     @Override
     @SuppressWarnings("unchecked")
     public Object process(Node nd, Stack<Node> stack, NodeProcessorCtx ctx, Object... nodeOutputs)
@@ -1182,7 +1101,7 @@ public final class ConstantPropagateProcFactory {
   /**
    * The Node Processor for Constant Propagation for Select Operators.
    */
-  public static class ConstantPropagateSelectProc implements NodeProcessor {
+  public static class ConstantPropagateSelectProc implements SemanticNodeProcessor {
     @Override
     public Object process(Node nd, Stack<Node> stack, NodeProcessorCtx ctx, Object... nodeOutputs)
         throws SemanticException {
@@ -1204,19 +1123,28 @@ public final class ConstantPropagateProcFactory {
             if (HiveConf.getPositionFromInternalName(colName) == -1) {
               // if its not an internal name, this is what we want.
               ((ExprNodeConstantDesc)newCol).setFoldedFromCol(colName);
+              // See if we can set the tabAlias this was folded from as well.
+              ExprNodeDesc colExpr = colList.get(i);
+              if (colExpr instanceof ExprNodeColumnDesc) {
+                ((ExprNodeConstantDesc)newCol).setFoldedFromTab(((ExprNodeColumnDesc) colExpr).getTabAlias());
+              }
             } else {
               // If it was internal column, lets try to get name from columnExprMap
               ExprNodeDesc desc = columnExprMap.get(colName);
               if (desc instanceof ExprNodeConstantDesc) {
                 ((ExprNodeConstantDesc)newCol).setFoldedFromCol(((ExprNodeConstantDesc)desc).getFoldedFromCol());
+                ((ExprNodeConstantDesc)newCol).setFoldedFromTab(((ExprNodeConstantDesc)desc).getFoldedFromTab());
               }
             }
           }
           colList.set(i, newCol);
-          if (newCol instanceof ExprNodeConstantDesc && op.getSchema() != null) {
+          if (op.getSchema() != null) {
             ColumnInfo colInfo = op.getSchema().getSignature().get(i);
-            if (!VirtualColumn.isVirtualColumnBasedOnAlias(colInfo)) {
-              constants.put(colInfo, newCol);
+            colInfo.setInternalName(columnNames.get(i));
+            if (newCol instanceof ExprNodeConstantDesc ) {
+              if (!VirtualColumn.isVirtualColumnBasedOnAlias(colInfo)) {
+                constants.put(colInfo, newCol);
+              }
             }
           }
           if (columnExprMap != null) {
@@ -1244,7 +1172,7 @@ public final class ConstantPropagateProcFactory {
    * The Node Processor for constant propagation for FileSink Operators. In addition to constant
    * propagation, this processor also prunes dynamic partitions to static partitions if possible.
    */
-  public static class ConstantPropagateFileSinkProc implements NodeProcessor {
+  public static class ConstantPropagateFileSinkProc implements SemanticNodeProcessor {
     @Override
     public Object process(Node nd, Stack<Node> stack, NodeProcessorCtx ctx, Object... nodeOutputs)
         throws SemanticException {
@@ -1257,8 +1185,11 @@ public final class ConstantPropagateProcFactory {
       }
       FileSinkDesc fsdesc = op.getConf();
       DynamicPartitionCtx dpCtx = fsdesc.getDynPartCtx();
-      if (dpCtx != null) {
 
+      // HIVE-22595: For Avro tables with external schema URL, Utilities.getDPColOffset() gives
+      // wrong results unless AvroSerDe.initialize() is called to update the table properties.
+      // To be safe just disable this check for Avro tables.
+      if (dpCtx != null && fsdesc.getTableInfo().getSerdeClassName().equals(AvroSerDe.class.getName())) {
         // Assume only 1 parent for FS operator
         Operator<? extends Serializable> parent = op.getParentOperators().get(0);
         Map<ColumnInfo, ExprNodeDesc> parentConstants = cppCtx.getPropagatedConstants(parent);
@@ -1287,7 +1218,7 @@ public final class ConstantPropagateProcFactory {
     }
   }
 
-  public static NodeProcessor getFileSinkProc() {
+  public static SemanticNodeProcessor getFileSinkProc() {
     return new ConstantPropagateFileSinkProc();
   }
 
@@ -1295,7 +1226,7 @@ public final class ConstantPropagateProcFactory {
    * The Node Processor for Constant Propagation for Operators which is designed to stop propagate.
    * Currently these kinds of Operators include UnionOperator and ScriptOperator.
    */
-  public static class ConstantPropagateStopProc implements NodeProcessor {
+  public static class ConstantPropagateStopProc implements SemanticNodeProcessor {
     @Override
     public Object process(Node nd, Stack<Node> stack, NodeProcessorCtx ctx, Object... nodeOutputs)
         throws SemanticException {
@@ -1309,7 +1240,7 @@ public final class ConstantPropagateProcFactory {
     }
   }
 
-  public static NodeProcessor getStopProc() {
+  public static SemanticNodeProcessor getStopProc() {
     return new ConstantPropagateStopProc();
   }
 
@@ -1318,7 +1249,7 @@ public final class ConstantPropagateProcFactory {
    * a join, then only those constants from inner join tables, or from the 'inner side' of a outer
    * join (left table for left outer join and vice versa) can be propagated.
    */
-  public static class ConstantPropagateReduceSinkProc implements NodeProcessor {
+  public static class ConstantPropagateReduceSinkProc implements SemanticNodeProcessor {
     @Override
     public Object process(Node nd, Stack<Node> stack, NodeProcessorCtx ctx, Object... nodeOutputs)
         throws SemanticException {
@@ -1347,9 +1278,7 @@ public final class ConstantPropagateProcFactory {
           && op.getChildOperators().get(0) instanceof JoinOperator) {
         JoinOperator joinOp = (JoinOperator) op.getChildOperators().get(0);
         if (skipFolding(joinOp.getConf())) {
-          if (LOG.isDebugEnabled()) {
-            LOG.debug("Skip folding in outer join " + op);
-          }
+          LOG.debug("Skip folding in outer join {}", op);
           cppCtx.getOpToConstantExprs().put(op, new HashMap<ColumnInfo, ExprNodeDesc>());
           return null;
         }
@@ -1357,9 +1286,7 @@ public final class ConstantPropagateProcFactory {
 
       if (rsDesc.getDistinctColumnIndices() != null
           && !rsDesc.getDistinctColumnIndices().isEmpty()) {
-        if (LOG.isDebugEnabled()) {
-          LOG.debug("Skip folding in distinct subqueries " + op);
-        }
+        LOG.debug("Skip folding in distinct subqueries {}", op);
         cppCtx.getOpToConstantExprs().put(op, new HashMap<ColumnInfo, ExprNodeDesc>());
         return null;
       }
@@ -1369,7 +1296,7 @@ public final class ConstantPropagateProcFactory {
       for (ExprNodeDesc desc : rsDesc.getKeyCols()) {
         ExprNodeDesc newDesc = foldExpr(desc, constants, cppCtx, op, 0, false);
         if (newDesc != desc && desc instanceof ExprNodeColumnDesc && newDesc instanceof ExprNodeConstantDesc) {
-          ((ExprNodeConstantDesc)newDesc).setFoldedFromCol(((ExprNodeColumnDesc)desc).getColumn());
+          ((ExprNodeConstantDesc)newDesc).setFoldedTabCol((ExprNodeColumnDesc)desc);
         }
         newKeyEpxrs.add(newDesc);
       }
@@ -1381,7 +1308,7 @@ public final class ConstantPropagateProcFactory {
         ExprNodeDesc expr = foldExpr(desc, constants, cppCtx, op, 0, false);
         if (expr != desc && desc instanceof ExprNodeColumnDesc
             && expr instanceof ExprNodeConstantDesc) {
-          ((ExprNodeConstantDesc) expr).setFoldedFromCol(((ExprNodeColumnDesc) desc).getColumn());
+          ((ExprNodeConstantDesc) expr).setFoldedTabCol((ExprNodeColumnDesc) desc);
         }
         newPartExprs.add(expr);
       }
@@ -1405,7 +1332,7 @@ public final class ConstantPropagateProcFactory {
     private boolean skipFolding(JoinDesc joinDesc) {
       for (JoinCondDesc cond : joinDesc.getConds()) {
         if (cond.getType() == JoinDesc.INNER_JOIN || cond.getType() == JoinDesc.UNIQUE_JOIN
-            || cond.getType() == JoinDesc.LEFT_SEMI_JOIN) {
+            || cond.getType() == JoinDesc.LEFT_SEMI_JOIN || cond.getType() == JoinDesc.ANTI_JOIN) {
           continue;
         }
         return true;
@@ -1415,14 +1342,14 @@ public final class ConstantPropagateProcFactory {
 
   }
 
-  public static NodeProcessor getReduceSinkProc() {
+  public static SemanticNodeProcessor getReduceSinkProc() {
     return new ConstantPropagateReduceSinkProc();
   }
 
   /**
    * The Node Processor for Constant Propagation for Join Operators.
    */
-  public static class ConstantPropagateJoinProc implements NodeProcessor {
+  public static class ConstantPropagateJoinProc implements SemanticNodeProcessor {
     @Override
     public Object process(Node nd, Stack<Node> stack, NodeProcessorCtx ctx, Object... nodeOutputs)
         throws SemanticException {
@@ -1438,15 +1365,18 @@ public final class ConstantPropagateProcFactory {
       // Note: the following code (removing folded constants in exprs) is deeply coupled with
       //    ColumnPruner optimizer.
       // Assuming ColumnPrunner will remove constant columns so we don't deal with output columns.
-      //    Except one case that the join operator is followed by a redistribution (RS operator).
-      if (op.getChildOperators().size() == 1
-          && op.getChildOperators().get(0) instanceof ReduceSinkOperator) {
-        LOG.debug("Skip JOIN-RS structure.");
-        return null;
-      }
-      if (LOG.isInfoEnabled()) {
-        LOG.info("Old exprs " + conf.getExprs());
+      //    Except one case that the join operator is followed by a redistribution (RS operator) -- skipping filter ops
+      if (op.getChildOperators().size() == 1) {
+        Node ndRecursive = op;
+        while (ndRecursive.getChildren().size() == 1 && ndRecursive.getChildren().get(0) instanceof FilterOperator) {
+          ndRecursive = ndRecursive.getChildren().get(0);
+        }
+        if (ndRecursive.getChildren().get(0) instanceof ReduceSinkOperator) {
+          LOG.debug("Skip JOIN-FIL(*)-RS structure.");
+          return null;
+        }
       }
+      LOG.info("Old exprs " + conf.getExprs());
       Iterator<Entry<Byte, List<ExprNodeDesc>>> itr = conf.getExprs().entrySet().iterator();
       while (itr.hasNext()) {
         Entry<Byte, List<ExprNodeDesc>> e = itr.next();
@@ -1459,18 +1389,14 @@ public final class ConstantPropagateProcFactory {
         for (ExprNodeDesc expr : exprs) {
           ExprNodeDesc newExpr = foldExpr(expr, constants, cppCtx, op, tag, false);
           if (newExpr instanceof ExprNodeConstantDesc) {
-            if (LOG.isInfoEnabled()) {
-              LOG.info("expr " + newExpr + " fold from " + expr + " is removed.");
-            }
+            LOG.info("expr " + newExpr + " fold from " + expr + " is removed.");
             continue;
           }
           newExprs.add(newExpr);
         }
         e.setValue(newExprs);
       }
-      if (LOG.isInfoEnabled()) {
-        LOG.info("New exprs " + conf.getExprs());
-      }
+      LOG.info("New exprs " + conf.getExprs());
 
       for (List<ExprNodeDesc> v : conf.getFilters().values()) {
         for (int i = 0; i < v.size(); i++) {
@@ -1484,14 +1410,14 @@ public final class ConstantPropagateProcFactory {
 
   }
 
-  public static NodeProcessor getJoinProc() {
+  public static SemanticNodeProcessor getJoinProc() {
     return new ConstantPropagateJoinProc();
   }
 
   /**
    * The Node Processor for Constant Propagation for Table Scan Operators.
    */
-  public static class ConstantPropagateTableScanProc implements NodeProcessor {
+  public static class ConstantPropagateTableScanProc implements SemanticNodeProcessor {
     @Override
     public Object process(Node nd, Stack<Node> stack, NodeProcessorCtx ctx, Object... nodeOutputs)
         throws SemanticException {
@@ -1515,7 +1441,7 @@ public final class ConstantPropagateProcFactory {
     }
   }
 
-  public static NodeProcessor getTableScanProc() {
+  public static SemanticNodeProcessor getTableScanProc() {
     return new ConstantPropagateTableScanProc();
   }
 }
