@@ -23,6 +23,9 @@ import java.util.List;
 import java.util.stream.Collectors;
 
 import org.apache.calcite.adapter.druid.DruidQuery;
+import org.apache.calcite.adapter.jdbc.JdbcConvention;
+import org.apache.calcite.adapter.jdbc.JdbcRel;
+import org.apache.calcite.adapter.jdbc.JdbcRules;
 import org.apache.calcite.plan.RelOptUtil;
 import org.apache.calcite.plan.hep.HepRelVertex;
 import org.apache.calcite.plan.volcano.RelSubset;
@@ -36,6 +39,7 @@ import org.apache.calcite.rel.core.Join;
 import org.apache.calcite.rel.core.Project;
 import org.apache.calcite.rel.core.SetOp;
 import org.apache.calcite.rel.core.Sort;
+import org.apache.calcite.rel.core.Spool;
 import org.apache.calcite.rel.core.Window.RexWinAggCall;
 import org.apache.calcite.rel.rules.MultiJoin;
 import org.apache.calcite.rel.type.RelDataType;
@@ -43,6 +47,7 @@ import org.apache.calcite.rel.type.RelDataTypeFactory;
 import org.apache.calcite.rex.RexNode;
 import org.apache.calcite.rex.RexOver;
 import org.apache.calcite.sql.SqlAggFunction;
+import org.apache.calcite.tools.RelBuilder;
 import org.apache.calcite.util.Pair;
 import org.apache.hadoop.hive.metastore.api.FieldSchema;
 import org.apache.hadoop.hive.ql.optimizer.calcite.CalciteSemanticException;
@@ -208,6 +213,12 @@ public class PlanModifierForASTConv {
           // called recursively, it will have the correct parent.
           rel = newParent.getInputs().get(0);
         }
+      } else if (rel instanceof Spool) {
+        Spool spool = (Spool) rel;
+        RelBuilder b = HiveRelFactories.HIVE_BUILDER.create(spool.getCluster(),null);
+        b.push(spool.getInput());
+        b.project(b.fields(), spool.getTable().getRowType().getFieldNames(), true);
+        spool.replaceInput(0, b.build());
       }
     }
 
@@ -266,8 +277,12 @@ public class PlanModifierForASTConv {
   private static RelNode introduceDerivedTable(final RelNode rel) {
     List<RexNode> projectList = HiveCalciteUtil.getProjsFromBelowAsInputRef(rel);
 
-    HiveProject select = HiveProject.create(rel.getCluster(), rel, projectList,
+    RelNode select = HiveProject.create(rel.getCluster(), rel, projectList,
         rel.getRowType(), Collections.emptyList());
+    
+    if (rel instanceof JdbcRel) {
+      select = JdbcRules.JdbcProjectRule.create((JdbcConvention) rel.getConvention()).convert(select);
+    }
 
     return select;
   }
