@@ -21,7 +21,6 @@ package org.apache.hadoop.hive.ql.stats;
 
 import java.io.IOException;
 import java.io.Serializable;
-import java.util.ArrayList;
 import java.util.Collection;
 import java.util.List;
 import java.util.Map;
@@ -82,7 +81,7 @@ import static org.apache.hadoop.hive.common.StatsSetupConst.UPDATE_COUNT;
 public class BasicStatsTask implements Serializable, IStatsProcessor {
 
   private static final long serialVersionUID = 1L;
-  private static transient final Logger LOG = LoggerFactory.getLogger(BasicStatsTask.class);
+  private static final Logger LOG = LoggerFactory.getLogger(BasicStatsTask.class);
 
   private Table table;
   private Collection<Partition> dpPartSpecs;
@@ -102,7 +101,6 @@ public class BasicStatsTask implements Serializable, IStatsProcessor {
 
   @Override
   public int process(Hive db, Table tbl) throws Exception {
-
     LOG.info("Executing stats task");
     table = tbl;
     return aggregateStats(db, tbl);
@@ -128,14 +126,18 @@ public class BasicStatsTask implements Serializable, IStatsProcessor {
     private BasicStatsWork work;
     private boolean followedColStats1;
     private Map<String, String> providedBasicStats;
+    private boolean skipStatsUpdate = false;
 
-    public BasicStatsProcessor(Partish partish, BasicStatsWork work, HiveConf conf, boolean followedColStats2) {
+    public BasicStatsProcessor(Partish partish, BasicStatsWork work, boolean followedColStats2) {
       this.partish = partish;
       this.work = work;
-      followedColStats1 = followedColStats2;
+      this.followedColStats1 = followedColStats2;
+      
       Table table = partish.getTable();
       if (table.isNonNative() && table.getStorageHandler().canProvideBasicStatistics()) {
-        providedBasicStats = table.getStorageHandler().getBasicStatistics(partish);
+        this.providedBasicStats = table.getStorageHandler().computeBasicStatistics(partish);
+        this.skipStatsUpdate = StatsSetupConst.STATS_REQUIRE_COMPUTE.stream()
+            .anyMatch(providedBasicStats::containsKey);
       }
     }
 
@@ -176,19 +178,19 @@ public class BasicStatsTask implements Serializable, IStatsProcessor {
 
       if (providedBasicStats == null) {
         MetaStoreServerUtils.populateQuickStats(partfileStatus, parameters);
-
-        if (statsAggregator != null) {
-          // Update stats for transactional tables (MM, or full ACID with overwrite), even
-          // though we are marking stats as not being accurate.
-          if (StatsSetupConst.areBasicStatsUptoDate(parameters) || p.isTransactionalTable()) {
-            String prefix = getAggregationPrefix(p.getTable(), p.getPartition());
-            updateStats(statsAggregator, parameters, prefix);
-          }
-        }
       } else {
         parameters.putAll(providedBasicStats);
       }
 
+      if (statsAggregator != null && !skipStatsUpdate) {
+        // Update stats for transactional tables (MM, or full ACID with overwrite), even
+        // though we are marking stats as not being accurate.
+        if (StatsSetupConst.areBasicStatsUptoDate(parameters) || p.isTransactionalTable()) {
+          String prefix = getAggregationPrefix(p.getTable(), p.getPartition());
+          updateStats(statsAggregator, parameters, prefix);
+        }
+      }
+
       return p.getOutput();
     }
 
@@ -205,7 +207,7 @@ public class BasicStatsTask implements Serializable, IStatsProcessor {
     }
 
     private void updateStats(StatsAggregator statsAggregator, Map<String, String> parameters,
-        String aggKey) throws HiveException {
+        String aggKey) {
       for (String statType : StatsSetupConst.STATS_REQUIRE_COMPUTE) {
         String value = statsAggregator.aggregateStats(aggKey, statType);
         if (value != null && !value.isEmpty()) {
@@ -292,13 +294,10 @@ public class BasicStatsTask implements Serializable, IStatsProcessor {
 
       String tableFullName = table.getDbName() + "." + table.getTableName();
 
-      List<Partish> partishes = new ArrayList<>();
-
       if (partitions == null) {
-        Partish p;
-        partishes.add(p = new Partish.PTable(table));
+        Partish p = new Partish.PTable(table);
 
-        BasicStatsProcessor basicStatsProcessor = new BasicStatsProcessor(p, work, conf, followedColStats);
+        BasicStatsProcessor basicStatsProcessor = new BasicStatsProcessor(p, work, followedColStats);
         basicStatsProcessor.collectFileStatus(wh, conf);
         Table res = (Table) basicStatsProcessor.process(statsAggregator);
         if (res == null) {
@@ -325,7 +324,7 @@ public class BasicStatsTask implements Serializable, IStatsProcessor {
         // Need to get the old stats of the partition
         // and update the table stats based on the old and new stats.
 
-        List<Partition> updates = new ArrayList<Partition>();
+        List<Partition> updates = Lists.newArrayList();
 
         final ExecutorService pool = buildBasicStatsExecutor();
 
@@ -334,9 +333,10 @@ public class BasicStatsTask implements Serializable, IStatsProcessor {
         List<TransactionalStatsProcessor> transactionalStatsProcessors = Lists.newLinkedList();
 
         try {
-          for(final Partition partn : partitions) {
-            Partish p;
-            BasicStatsProcessor bsp = new BasicStatsProcessor(p = new Partish.PPart(table, partn), work, conf, followedColStats);
+          for (final Partition partn : partitions) {
+            Partish p = new Partish.PPart(table, partn);
+            
+            BasicStatsProcessor bsp = new BasicStatsProcessor(p, work, followedColStats);
             processors.add(bsp);
             transactionalStatsProcessors.add(new TransactionalStatsProcessor(db, p));
 
@@ -363,9 +363,7 @@ public class BasicStatsTask implements Serializable, IStatsProcessor {
             ret = 1;
           }
         } finally {
-          if (pool != null) {
-            pool.shutdownNow();
-          }
+          pool.shutdownNow();
           LOG.debug("Finished getting file stats of all partitions!");
         }
 
@@ -431,7 +429,7 @@ public class BasicStatsTask implements Serializable, IStatsProcessor {
   }
 
   private StatsAggregator createStatsAggregator(StatsCollectionContext scc, HiveConf conf) throws HiveException {
-    String statsImpl = HiveConf.getVar(conf, HiveConf.ConfVars.HIVESTATSDBCLASS);
+    String statsImpl = HiveConf.getVar(conf, HiveConf.ConfVars.HIVE_STATS_DBCLASS);
     StatsFactory factory = StatsFactory.newFactory(statsImpl, conf);
     if (factory == null) {
       throw new HiveException(ErrorMsg.STATSPUBLISHER_NOT_OBTAINED.getErrorCodedMsg());
@@ -506,7 +504,7 @@ public class BasicStatsTask implements Serializable, IStatsProcessor {
       // INSERT OVERWRITE command
       LoadTableDesc tbd = work.getLoadTableDesc();
       table = db.getTable(tbd.getTable().getTableName());
-      if (!table.isPartitioned()) {
+      if (!table.isPartitioned() || table.hasNonNativePartitionSupport()) {
         return null;
       }
       DynamicPartitionCtx dpCtx = tbd.getDPCtx();
@@ -518,16 +516,12 @@ public class BasicStatsTask implements Serializable, IStatsProcessor {
           return db.getPartitionsByNames(table, partNames);
         }
       } else { // static partition
-        return singletonList(db.getPartition(table, tbd.getPartitionSpec(), false));
+        return singletonList(db.getPartition(table, tbd.getPartitionSpec()));
       }
     }
     return emptyList();
   }
 
-  public Collection<Partition> getDpPartSpecs() {
-    return dpPartSpecs;
-  }
-
   @Override
   public void setDpPartSpecs(Collection<Partition> dpPartSpecs) {
     this.dpPartSpecs = dpPartSpecs;
@@ -535,8 +529,7 @@ public class BasicStatsTask implements Serializable, IStatsProcessor {
 
   public static String getAggregationPrefix(Table table, Partition partition) throws MetaException {
     String prefix = getAggregationPrefix0(table, partition);
-    String aggKey = prefix.endsWith(Path.SEPARATOR) ? prefix : prefix + Path.SEPARATOR;
-    return aggKey;
+    return prefix.endsWith(Path.SEPARATOR) ? prefix : prefix + Path.SEPARATOR;
   }
 
   private static String getAggregationPrefix0(Table table, Partition partition) throws MetaException {
