@@ -63,15 +63,15 @@ public class TestTxnAddPartition extends TxnCommandsBaseForTests {
 
 
   @Test
-  public void addPartition() throws Exception {
-
-    addPartition(false);
+  public void testAddPartition() throws Exception {
+    hiveConf.setBoolVar(HiveConf.ConfVars.HIVE_VECTORIZATION_ENABLED, false);
+    addPartition();
   }
 
   @Test
-  public void addPartitionVectorized() throws Exception {
+  public void testAddPartitionVectorized() throws Exception {
     hiveConf.setBoolVar(HiveConf.ConfVars.HIVE_VECTORIZATION_ENABLED, true);
-    addPartition(true);
+    addPartition();
   }
 
   /**
@@ -80,7 +80,7 @@ public class TestTxnAddPartition extends TxnCommandsBaseForTests {
    * adding partition when it already exists
    * adding partition when it already exists with "if not exists"
    */
-  private void addPartition(boolean isVectorized) throws Exception {
+  private void addPartition() throws Exception {
     runStatementOnDriver("drop table if exists T");
     runStatementOnDriver("drop table if exists Tstage");
     runStatementOnDriver("create table T (a int, b int) partitioned by (p int) stored as orc" +
@@ -97,8 +97,7 @@ public class TestTxnAddPartition extends TxnCommandsBaseForTests {
         " PARTITION (p=1) location '" + getWarehouseDir() + "/2/data'" +
         " PARTITION (p=2)");
 
-    String testQuery = isVectorized ? "select ROW__ID, p, a, b from T order by p, ROW__ID" :
-        "select ROW__ID, p, a, b, INPUT__FILE__NAME from T order by p, ROW__ID";
+    String testQuery = "select ROW__ID, p, a, b, INPUT__FILE__NAME from T order by p, ROW__ID";
     String[][] expected = new String[][]{
         {"{\"writeid\":1,\"bucketid\":536870912,\"rowid\":0}\t0\t0\t2",
             "warehouse/t/p=0/delta_0000001_0000001_0000/000000_0"},
@@ -108,7 +107,7 @@ public class TestTxnAddPartition extends TxnCommandsBaseForTests {
             "warehouse/t/p=1/delta_0000001_0000001_0000/000000_0"},
         {"{\"writeid\":1,\"bucketid\":536870912,\"rowid\":1}\t1\t0\t4",
             "warehouse/t/p=1/delta_0000001_0000001_0000/000000_0"}};
-    checkResult(expected, testQuery, isVectorized, "add 2 parts w/data and 1 empty", LOG);
+    checkResultAndVectorization(expected, testQuery, "add 2 parts w/data and 1 empty", LOG);
 
     runStatementOnDriver("export table Tstage to '" + getWarehouseDir() + "/3'");
     //should be an error since p=3 exists
@@ -136,18 +135,19 @@ public class TestTxnAddPartition extends TxnCommandsBaseForTests {
             "warehouse/t/p=3/delta_0000003_0000003_0000/000000_0"},
         {"{\"writeid\":3,\"bucketid\":536870912,\"rowid\":1}\t3\t0\t4",
             "warehouse/t/p=3/delta_0000003_0000003_0000/000000_0"}};
-    checkResult(expected2, testQuery, isVectorized, "add 2 existing parts and 1 empty", LOG);
+    checkResultAndVectorization(expected2, testQuery, "add 2 existing parts and 1 empty", LOG);
   }
 
   @Test
-  public void addPartitionMM() throws Exception {
-    addPartitionMM(false);
+  public void testAddPartitionMM() throws Exception {
+    hiveConf.setBoolVar(HiveConf.ConfVars.HIVE_VECTORIZATION_ENABLED, false);
+    addPartitionMM();
   }
 
   @Test
-  public void addPartitionMMVectorized() throws Exception {
+  public void testAddPartitionMMVectorized() throws Exception {
     hiveConf.setBoolVar(HiveConf.ConfVars.HIVE_VECTORIZATION_ENABLED, true);
-    addPartitionMM(true);
+    addPartitionMM();
   }
 
   /**
@@ -157,7 +157,7 @@ public class TestTxnAddPartition extends TxnCommandsBaseForTests {
    * adding partition when it already exists
    * adding partition when it already exists with "if not exists"
    */
-  private void addPartitionMM(boolean isVectorized) throws Exception {
+  private void addPartitionMM() throws Exception {
     runStatementOnDriver("drop table if exists T");
     runStatementOnDriver("drop table if exists Tstage");
 
@@ -175,14 +175,13 @@ public class TestTxnAddPartition extends TxnCommandsBaseForTests {
         " PARTITION (p=1) location '" + getWarehouseDir() + "/2/data'" +
         " PARTITION (p=2)");
 
-    String testQuery = isVectorized ? "select p, a, b from T order by p, a, b" :
-        "select p, a, b, INPUT__FILE__NAME from T order by p, a, b";
+    String testQuery = "select p, a, b, INPUT__FILE__NAME from T order by p, a, b";
     String[][] expected = new String[][]{
         {"0\t0\t2", "warehouse/t/p=0/delta_0000001_0000001_0000/000000_0"},
         {"0\t0\t4", "warehouse/t/p=0/delta_0000001_0000001_0000/000000_0"},
         {"1\t0\t2", "warehouse/t/p=1/delta_0000001_0000001_0000/000000_0"},
         {"1\t0\t4", "warehouse/t/p=1/delta_0000001_0000001_0000/000000_0"}};
-    checkResult(expected, testQuery, isVectorized, "add 2 parts w/data and 1 empty", LOG);
+    checkResultAndVectorization(expected, testQuery, "add 2 parts w/data and 1 empty", LOG);
 
     runStatementOnDriver("export table Tstage to '" + getWarehouseDir() + "/3'");
     //should be an error since p=3 exists
@@ -203,7 +202,7 @@ public class TestTxnAddPartition extends TxnCommandsBaseForTests {
         {"1\t0\t4", "warehouse/t/p=1/delta_0000001_0000001_0000/000000_0"},
         {"3\t0\t2", "warehouse/t/p=3/delta_0000003_0000003_0000/000000_0"},
         {"3\t0\t4", "warehouse/t/p=3/delta_0000003_0000003_0000/000000_0"}};
-    checkResult(expected2, testQuery, isVectorized, "add 2 existing parts and 1 empty", LOG);
+    checkResultAndVectorization(expected2, testQuery, "add 2 existing parts and 1 empty", LOG);
   }
 
   @Test
@@ -232,7 +231,7 @@ public class TestTxnAddPartition extends TxnCommandsBaseForTests {
   }
 
   private void checkExpected(List<String> rs, String[][] expected, String msg) {
-    super.checkExpected(rs, expected, msg, LOG, true);
+    super.checkExpected(rs, expected, msg, LOG);
   }
 
   /**
@@ -249,7 +248,7 @@ public class TestTxnAddPartition extends TxnCommandsBaseForTests {
     runStatementOnDriver("create table Tstage (a int, b int)  clustered by (a) into 2 " +
         "buckets stored as orc tblproperties('transactional'='false')");
 
-    runStatementOnDriver("insert into Tstage values(0,2),(1,4)");
+    runStatementOnDriver("insert into Tstage values(0,2),(2,4)");
     runStatementOnDriver("export table Tstage to '" + getWarehouseDir() + "/1'");
     FileSystem fs = FileSystem.get(hiveConf);
     fs.rename(new Path(getWarehouseDir() + "/1/data/000000_0"), new Path(getWarehouseDir() + "/1/data/part-m000"));
@@ -261,9 +260,9 @@ public class TestTxnAddPartition extends TxnCommandsBaseForTests {
     List<String> rs = runStatementOnDriver(
         "select ROW__ID, p, a, b, INPUT__FILE__NAME from T order by p, ROW__ID");
     String[][] expected = new String[][]{
+            {"{\"writeid\":1,\"bucketid\":536870912,\"rowid\":0}\t0\t2\t4",
+                    "warehouse/t/p=0/delta_0000001_0000001_0000/000000_0"},
         {"{\"writeid\":1,\"bucketid\":536936448,\"rowid\":0}\t0\t0\t2",
-            "warehouse/t/p=0/delta_0000001_0000001_0000/000001_0"},
-        {"{\"writeid\":1,\"bucketid\":536936448,\"rowid\":1}\t0\t1\t4",
             "warehouse/t/p=0/delta_0000001_0000001_0000/000001_0"}};
     checkExpected(rs, expected, "add partition (p=0)");
   }
