@@ -62,7 +62,7 @@ public class TestTxnConcatenate extends TxnCommandsBaseForTests {
             "acidtbl/delta_0000003_0000003_0000/bucket_00001_0"},
         {"{\"writeid\":3,\"bucketid\":536936448,\"rowid\":1}\t8\t8",
             "acidtbl/delta_0000003_0000003_0000/bucket_00001_0"}};
-    checkResult(expected, testQuery, false, "check data", LOG);
+    checkResultAndVectorization(expected, testQuery, "check data", LOG);
 
     /*in UTs, there is no standalone HMS running to kick off compaction so it's done via runWorker()
      but in normal usage 'concatenate' is blocking, */
@@ -79,14 +79,14 @@ public class TestTxnConcatenate extends TxnCommandsBaseForTests {
     Assert.assertEquals(TxnStore.CLEANING_RESPONSE, rsp.getCompacts().get(0).getState());
     String[][] expected2 = new String[][] {
         {"{\"writeid\":2,\"bucketid\":536936449,\"rowid\":0}\t1\t4",
-            "acidtbl/base_0000003_v0000021/bucket_00001"},
+            "acidtbl/base_0000003_v0000011/bucket_00001"},
         {"{\"writeid\":2,\"bucketid\":536936449,\"rowid\":1}\t4\t4",
-            "acidtbl/base_0000003_v0000021/bucket_00001"},
+            "acidtbl/base_0000003_v0000011/bucket_00001"},
         {"{\"writeid\":3,\"bucketid\":536936448,\"rowid\":0}\t5\t6",
-            "acidtbl/base_0000003_v0000021/bucket_00001"},
+            "acidtbl/base_0000003_v0000011/bucket_00001"},
         {"{\"writeid\":3,\"bucketid\":536936448,\"rowid\":1}\t8\t8",
-            "acidtbl/base_0000003_v0000021/bucket_00001"}};
-    checkResult(expected2, testQuery, false, "check data after concatenate", LOG);
+            "acidtbl/base_0000003_v0000011/bucket_00001"}};
+    checkResultAndVectorization(expected2, testQuery, "check data after concatenate", LOG);
   }
   @Test
   public void testConcatenatePart() throws Exception {
@@ -103,7 +103,7 @@ public class TestTxnConcatenate extends TxnCommandsBaseForTests {
             "acidtblpart/p=p1/delta_0000003_0000003_0000/bucket_00001_0"},
         {"{\"writeid\":3,\"bucketid\":536936448,\"rowid\":0}\t8\t8",
             "acidtblpart/p=p2/delta_0000003_0000003_0000/bucket_00001_0"}};
-    checkResult(expected, testQuery, false, "check data", LOG);
+    checkResultAndVectorization(expected, testQuery, "check data", LOG);
 
     /*in UTs, there is no standalone HMS running to kick off compaction so it's done via runWorker()
      but in normal usage 'concatenate' is blocking, */
@@ -120,21 +120,25 @@ public class TestTxnConcatenate extends TxnCommandsBaseForTests {
     Assert.assertEquals(TxnStore.CLEANING_RESPONSE, rsp.getCompacts().get(0).getState());
     String[][] expected2 = new String[][] {
         {"{\"writeid\":2,\"bucketid\":536936449,\"rowid\":0}\t1\t4",
-            "acidtblpart/p=p1/base_0000003_v0000021/bucket_00001"},
+            "acidtblpart/p=p1/base_0000003_v0000011/bucket_00001"},
         {"{\"writeid\":1,\"bucketid\":536936448,\"rowid\":0}\t4\t5",
             "acidtblpart/p=p2/delta_0000001_0000001_0000/bucket_00001_0"},
         {"{\"writeid\":3,\"bucketid\":536936448,\"rowid\":0}\t5\t6",
-            "acidtblpart/p=p1/base_0000003_v0000021/bucket_00001"},
+            "acidtblpart/p=p1/base_0000003_v0000011/bucket_00001"},
         {"{\"writeid\":3,\"bucketid\":536936448,\"rowid\":0}\t8\t8",
             "acidtblpart/p=p2/delta_0000003_0000003_0000/bucket_00001_0"}};
 
-    checkResult(expected2, testQuery, false, "check data after concatenate", LOG);
+    checkResultAndVectorization(expected2, testQuery, "check data after concatenate", LOG);
   }
 
   @Test
   public void testConcatenateMM() throws Exception {
+    // Only one bucket is expected in this test
+    hiveConf.set("tez.grouping.max-size", "1024");
+    hiveConf.set("tez.grouping.min-size", "1");
+
     HiveConf.setBoolVar(hiveConf, HiveConf.ConfVars.HIVE_CREATE_TABLES_AS_INSERT_ONLY, true);
-    runStatementOnDriver("drop table if exists T");
+    dropTables("T");
     runStatementOnDriver("create table T(a int, b int)");
     runStatementOnDriver("insert into T values(1,2),(4,5)");
     runStatementOnDriver("insert into T values(5,6),(8,8)");
@@ -144,7 +148,7 @@ public class TestTxnConcatenate extends TxnCommandsBaseForTests {
         {"4\t5", "t/delta_0000001_0000001_0000/000000_0"},
         {"5\t6", "t/delta_0000002_0000002_0000/000000_0"},
         {"8\t8", "t/delta_0000002_0000002_0000/000000_0"}};
-    checkResult(expected, testQuery, false, "check data", LOG);
+    checkResultAndVectorization(expected, testQuery, "check data", LOG);
 
     /*in UTs, there is no standalone HMS running to kick off compaction so it's done via runWorker()
       but in normal usage 'concatenate' is blocking, */
@@ -160,10 +164,10 @@ public class TestTxnConcatenate extends TxnCommandsBaseForTests {
     Assert.assertEquals(1, rsp.getCompactsSize());
     Assert.assertEquals(TxnStore.CLEANING_RESPONSE, rsp.getCompacts().get(0).getState());
     String[][] expected2 = new String[][] {
-        {"1\t2", "t/base_0000003_v0000022/000000_0"},
-        {"4\t5", "t/base_0000003_v0000022/000000_0"},
-        {"5\t6", "t/base_0000003_v0000022/000000_0"},
-        {"8\t8", "t/base_0000003_v0000022/000000_0"}};
-    checkResult(expected2, testQuery, false, "check data after concatenate", LOG);
+        {"1\t2", "t/base_0000003_v0000011/000000_0"},
+        {"4\t5", "t/base_0000003_v0000011/000000_0"},
+        {"5\t6", "t/base_0000003_v0000011/000000_0"},
+        {"8\t8", "t/base_0000003_v0000011/000000_0"}};
+    checkResultAndVectorization(expected2, testQuery, "check data after concatenate", LOG);
   }
 }
