@@ -313,7 +313,7 @@ public class TestFileSinkOperator {
         TFSOStatsPublisher.class.getName());
     jc.set(HiveConf.ConfVars.HIVE_STATS_DEFAULT_AGGREGATOR.varname,
         TFSOStatsAggregator.class.getName());
-    jc.set(HiveConf.ConfVars.HIVESTATSDBCLASS.varname, "custom");
+    jc.set(HiveConf.ConfVars.HIVE_STATS_DBCLASS.varname, "custom");
   }
 
   @After
@@ -404,7 +404,7 @@ public class TestFileSinkOperator {
       DynamicPartitionCtx dpCtx = new DynamicPartitionCtx(partColMap, "Sunday", 100);
       //todo: does this need the finalDestination?
       desc = new FileSinkDesc(basePath, tableDesc, false, 1, false,
-          false, 1, 1, partCols, dpCtx, null, null, false, false, false, false, false, writeType, false);
+          false, 1, 1, partCols, dpCtx, null, false, false, false, false, false, writeType, false);
     } else {
       desc = new FileSinkDesc(basePath, tableDesc, false);
     }
