@@ -180,7 +180,7 @@ public class TestAcidUtils {
         new MockFile("mock:/tbl/part1/_done", 0, new byte[0]),
         new MockFile("mock:/tbl/part1/subdir/000000_0", 0, new byte[0]));
     AcidDirectory dir = AcidUtils.getAcidState(fs, new MockPath(fs, "/tbl/part1"), conf,
-        new ValidReaderWriteIdList("tbl:100:" + Long.MAX_VALUE + ":"), null, false);
+        ValidReaderWriteIdList.fromValue("tbl:100:" + Long.MAX_VALUE + ":"), null, false);
     assertEquals(null, dir.getBaseDirectory());
     assertEquals(0, dir.getCurrentDirectories().size());
     assertEquals(0, dir.getObsolete().size());
@@ -216,7 +216,7 @@ public class TestAcidUtils {
     conf.set(ValidTxnList.VALID_TXNS_KEY,
         new ValidReadTxnList(new long[0], new BitSet(), 1000, Long.MAX_VALUE).writeToString());
     AcidDirectory dir = AcidUtils.getAcidState(fs, new MockPath(fs, "mock:/tbl/part1"), conf,
-        new ValidReaderWriteIdList("tbl:100:" + Long.MAX_VALUE + ":"), null, false);
+        ValidReaderWriteIdList.fromValue("tbl:100:" + Long.MAX_VALUE + ":"), null, false);
     assertEquals(null, dir.getBaseDirectory());
     List<Path> obsolete = dir.getObsolete();
     assertEquals(2, obsolete.size());
@@ -259,7 +259,7 @@ public class TestAcidUtils {
     conf.set(ValidTxnList.VALID_TXNS_KEY,
         new ValidReadTxnList(new long[0], new BitSet(), 1000, Long.MAX_VALUE).writeToString());
     AcidDirectory dir = AcidUtils.getAcidState(fs, new MockPath(fs, "mock:/tbl/part1"), conf,
-        new ValidReaderWriteIdList("tbl:100:" + Long.MAX_VALUE + ":"), null, false);
+        ValidReaderWriteIdList.fromValue("tbl:100:" + Long.MAX_VALUE + ":"), null, false);
     assertEquals("mock:/tbl/part1/base_49", dir.getBaseDirectory().toString());
     List<Path> obsoletes = dir.getObsolete();
     assertEquals(5, obsoletes.size());
@@ -309,7 +309,7 @@ public class TestAcidUtils {
     Path part = new MockPath(fs, "/tbl/part1");
     conf.set(ValidTxnList.VALID_TXNS_KEY,
         new ValidReadTxnList(new long[0], new BitSet(), 1000, Long.MAX_VALUE).writeToString());
-    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, new ValidReaderWriteIdList("tbl:150:"
+    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, ValidReaderWriteIdList.fromValue("tbl:150:"
         + Long.MAX_VALUE + ":"), null, false);
     // Obsolete list should include the two original bucket files, and the old base dir
     List<Path> obsoletes = dir.getObsolete();
@@ -332,7 +332,7 @@ public class TestAcidUtils {
     Path part = new MockPath(fs, "mock:/tbl/part1");
     conf.set(ValidTxnList.VALID_TXNS_KEY,
         new ValidReadTxnList(new long[0], new BitSet(), 1000, Long.MAX_VALUE).writeToString());
-    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, new ValidReaderWriteIdList("tbl:100:"
+    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, ValidReaderWriteIdList.fromValue("tbl:100:"
         + Long.MAX_VALUE + ":"), null, false);
     assertEquals("mock:/tbl/part1/base_50", dir.getBaseDirectory().toString());
     List<Path> obsolete = dir.getObsolete();
@@ -369,7 +369,7 @@ public class TestAcidUtils {
     Path part = new MockPath(fs, "mock:/tbl/part1");
     conf.set(ValidTxnList.VALID_TXNS_KEY,
         new ValidReadTxnList(new long[0], new BitSet(), 1000, Long.MAX_VALUE).writeToString());
-    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, new ValidReaderWriteIdList("tbl:100:"
+    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, ValidReaderWriteIdList.fromValue("tbl:100:"
         + Long.MAX_VALUE + ":"), null, false);
     assertEquals("mock:/tbl/part1/base_50", dir.getBaseDirectory().toString());
     List<Path> obsolete = dir.getObsolete();
@@ -398,7 +398,7 @@ public class TestAcidUtils {
     //hypothetically, txn 50 is open and writing write ID 4
     conf.set(ValidTxnList.VALID_TXNS_KEY,
         new ValidReadTxnList(new long[] {50}, new BitSet(), 1000, 55).writeToString());
-    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, new ValidReaderWriteIdList("tbl:100:4:4"), null,
+    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, ValidReaderWriteIdList.fromValue("tbl:100:4:4"), null,
         false);
     List<AcidUtils.ParsedDelta> delts = dir.getCurrentDirectories();
     assertEquals(2, delts.size());
@@ -423,7 +423,7 @@ public class TestAcidUtils {
     //hypothetically, txn 50 is open and writing write ID 4
     conf.set(ValidTxnList.VALID_TXNS_KEY,
         new ValidReadTxnList(new long[] {50}, new BitSet(), 1000, 55).writeToString());
-    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, new ValidReaderWriteIdList("tbl:100:4:4"), null,
+    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, ValidReaderWriteIdList.fromValue("tbl:100:4:4"), null,
         false);
     List<AcidUtils.ParsedDelta> delts = dir.getCurrentDirectories();
     assertEquals(2, delts.size());
@@ -440,7 +440,7 @@ public class TestAcidUtils {
     Path part = new MockPath(fs, "mock:/tbl/part1");
     conf.set(ValidTxnList.VALID_TXNS_KEY,
         new ValidReadTxnList(new long[0], new BitSet(), 1000, Long.MAX_VALUE).writeToString());
-    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, new ValidCompactorWriteIdList("tbl:4:"
+    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, ValidCompactorWriteIdList.fromValue("tbl:4:"
         + Long.MAX_VALUE), null, false);
     List<AcidUtils.ParsedDelta> delts = dir.getCurrentDirectories();
     assertEquals(1, delts.size());
@@ -459,7 +459,7 @@ public class TestAcidUtils {
     Path part = new MockPath(fs, "mock:/tbl/part1");
     conf.set(ValidTxnList.VALID_TXNS_KEY,
         new ValidReadTxnList(new long[0], new BitSet(), 1000, Long.MAX_VALUE).writeToString());
-    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, new ValidCompactorWriteIdList("tbl:3:"
+    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, ValidCompactorWriteIdList.fromValue("tbl:3:"
         + Long.MAX_VALUE), null, false);
     List<AcidUtils.ParsedDelta> delts = dir.getCurrentDirectories();
     assertEquals(1, delts.size());
@@ -486,7 +486,7 @@ public class TestAcidUtils {
     conf.set(ValidTxnList.VALID_TXNS_KEY,
         new ValidReadTxnList(new long[0], new BitSet(), 1000, Long.MAX_VALUE).writeToString());
     AcidDirectory dir = AcidUtils.getAcidState(fs, new MockPath(fs, "mock:/tbl/part1"), conf,
-        new ValidReaderWriteIdList("tbl:100:" + Long.MAX_VALUE + ":"), null, false);
+        ValidReaderWriteIdList.fromValue("tbl:100:" + Long.MAX_VALUE + ":"), null, false);
     assertEquals("mock:/tbl/part1/base_49", dir.getBaseDirectory().toString());
     List<Path> obsoletes = dir.getObsolete();
     assertEquals(7, obsoletes.size());
@@ -528,7 +528,7 @@ public class TestAcidUtils {
     Path part = new MockPath(fs, "mock:/tbl/part1");
     conf.set(ValidTxnList.VALID_TXNS_KEY,
         new ValidReadTxnList(new long[0], new BitSet(), 1000, Long.MAX_VALUE).writeToString());
-    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, new ValidReaderWriteIdList("tbl:100:"
+    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, ValidReaderWriteIdList.fromValue("tbl:100:"
         + Long.MAX_VALUE + ":"), null, false);
     assertEquals("mock:/tbl/part1/base_50", dir.getBaseDirectory().toString());
     List<Path> obsolete = dir.getObsolete();
@@ -559,7 +559,7 @@ public class TestAcidUtils {
     Path part = new MockPath(fs, "mock:/tbl/part1");
     conf.set(ValidTxnList.VALID_TXNS_KEY,
         new ValidReadTxnList(new long[0], new BitSet(), 1000, Long.MAX_VALUE).writeToString());
-    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, new ValidReaderWriteIdList("tbl:100:"
+    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, ValidReaderWriteIdList.fromValue("tbl:100:"
         + Long.MAX_VALUE + ":"), null, false);
     List<Path> obsolete = dir.getObsolete();
     assertEquals(1, obsolete.size());
@@ -588,7 +588,7 @@ public class TestAcidUtils {
     Path part = new MockPath(fs, "mock:/tbl/part1");
     conf.set(ValidTxnList.VALID_TXNS_KEY,
         new ValidReadTxnList(new long[0], new BitSet(), 1000, Long.MAX_VALUE).writeToString());
-    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, new ValidCompactorWriteIdList("tbl:4:"
+    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, ValidCompactorWriteIdList.fromValue("tbl:4:"
         + Long.MAX_VALUE + ":"), null, false);
     List<AcidUtils.ParsedDelta> delts = dir.getCurrentDirectories();
     assertEquals(2, delts.size());
@@ -613,7 +613,7 @@ public class TestAcidUtils {
     //hypothetically, txn 50 is open and writing write ID 4
     conf.set(ValidTxnList.VALID_TXNS_KEY,
         new ValidReadTxnList(new long[] {50}, new BitSet(), 1000, 55).writeToString());
-    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, new ValidReaderWriteIdList("tbl:100:4:4"), null,
+    AcidDirectory dir = AcidUtils.getAcidState(fs, part, conf, ValidReaderWriteIdList.fromValue("tbl:100:4:4"), null,
         false);
     List<AcidUtils.ParsedDelta> delts = dir.getCurrentDirectories();
     assertEquals(3, delts.size());
