@@ -23,6 +23,7 @@ import java.util.List;
 import org.junit.Assert;
 
 import org.apache.hadoop.hive.conf.HiveConf;
+import org.apache.hadoop.hive.conf.HiveConfForTest;
 import org.apache.hadoop.hive.ql.QueryState;
 import org.apache.hadoop.hive.ql.session.SessionState;
 import org.junit.Before;
@@ -39,8 +40,8 @@ public class TestQBJoinTreeApplyPredicate {
   @BeforeClass
   public static void initialize() {
     queryState =
-        new QueryState.Builder().withHiveConf(new HiveConf(SemanticAnalyzer.class)).build();
-    conf = queryState.getConf();
+        new QueryState.Builder().build();
+    conf = new HiveConfForTest(TestQBJoinTreeApplyPredicate.class);
     SessionState.start(conf);
   }
 
