@@ -20,6 +20,7 @@ import java.util.Collections;
 import java.util.List;
 
 import org.apache.hadoop.hive.conf.HiveConf;
+import org.apache.hadoop.hive.common.IPStackUtils;
 import org.apache.hive.service.cli.session.SessionManager;
 import org.junit.Test;
 import org.junit.rules.ExpectedException;
@@ -42,7 +43,8 @@ public class TestCLIServiceConnectionLimits {
     List<SessionHandle> sessionHandles = new ArrayList<>();
     try {
       for (int i = 0; i < limit + 1; i++) {
-        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar", "127.0.0.1", null);
+        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar",
+            IPStackUtils.resolveLoopbackAddress(), null);
         sessionHandles.add(session);
       }
     } finally {
@@ -63,7 +65,8 @@ public class TestCLIServiceConnectionLimits {
     try {
       // open 5 connections
       for (int i = 0; i < limit / 2; i++) {
-        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar", "127.0.0.1", null);
+        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar",
+            IPStackUtils.resolveLoopbackAddress(), null);
         sessionHandles.add(session);
       }
 
@@ -75,7 +78,8 @@ public class TestCLIServiceConnectionLimits {
 
       // open till limit but not exceed
       for (int i = 0; i < limit; i++) {
-        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "ff", "bar", "127.0.0.1", null);
+        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "ff", "bar",
+            IPStackUtils.resolveLoopbackAddress(), null);
         sessionHandles.add(session);
       }
     } finally {
@@ -95,12 +99,14 @@ public class TestCLIServiceConnectionLimits {
     List<SessionHandle> sessionHandles = new ArrayList<>();
     try {
       for (int i = 0; i < limit + 1; i++) {
-        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, null, "bar", "127.0.0.1", null);
+        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, null, "bar",
+            IPStackUtils.resolveLoopbackAddress(), null);
         sessionHandles.add(session);
       }
 
       for (int i = 0; i < limit + 1; i++) {
-        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "", "bar", "127.0.0.1", null);
+        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "", "bar",
+            IPStackUtils.resolveLoopbackAddress(), null);
         sessionHandles.add(session);
       }
     } finally {
@@ -168,7 +174,8 @@ public class TestCLIServiceConnectionLimits {
     List<SessionHandle> sessionHandles = new ArrayList<>();
     try {
       for (int i = 0; i < limit + 1; i++) {
-        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar", "127.0.0.1", null);
+        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar",
+            IPStackUtils.resolveLoopbackAddress(), null);
         sessionHandles.add(session);
       }
 
@@ -183,7 +190,8 @@ public class TestCLIServiceConnectionLimits {
   @Test
   public void testConnectionLimitPerIpAddress() throws HiveSQLException {
     thrown.expect(HiveSQLException.class);
-    thrown.expectMessage("Connection limit per ipaddress reached (ipaddress: 127.0.0.1 limit: 10)");
+    thrown.expectMessage(String.format("Connection limit per ipaddress reached (ipaddress: %s limit: 10)", 
+        IPStackUtils.resolveLoopbackAddress()));
 
     conf.setIntVar(HiveConf.ConfVars.HIVE_SERVER2_LIMIT_CONNECTIONS_PER_USER, 0);
     conf.setIntVar(HiveConf.ConfVars.HIVE_SERVER2_LIMIT_CONNECTIONS_PER_IPADDRESS, 10);
@@ -192,7 +200,8 @@ public class TestCLIServiceConnectionLimits {
     List<SessionHandle> sessionHandles = new ArrayList<>();
     try {
       for (int i = 0; i < limit + 1; i++) {
-        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar", "127.0.0.1", null);
+        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar",
+            IPStackUtils.resolveLoopbackAddress(), null);
         sessionHandles.add(session);
       }
 
@@ -207,7 +216,8 @@ public class TestCLIServiceConnectionLimits {
   @Test
   public void testConnectionLimitPerUserIpAddress() throws HiveSQLException {
     thrown.expect(HiveSQLException.class);
-    thrown.expectMessage("Connection limit per user:ipaddress reached (user:ipaddress: foo:127.0.0.1 limit: 10)");
+    thrown.expectMessage(String.format("Connection limit per user:ipaddress reached (user:ipaddress: foo:%s limit: 10)",
+        IPStackUtils.resolveLoopbackAddress()));
 
     conf.setIntVar(HiveConf.ConfVars.HIVE_SERVER2_LIMIT_CONNECTIONS_PER_USER, 0);
     conf.setIntVar(HiveConf.ConfVars.HIVE_SERVER2_LIMIT_CONNECTIONS_PER_IPADDRESS, 0);
@@ -216,7 +226,8 @@ public class TestCLIServiceConnectionLimits {
     List<SessionHandle> sessionHandles = new ArrayList<>();
     try {
       for (int i = 0; i < limit + 1; i++) {
-        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar", "127.0.0.1", null);
+        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar",
+            IPStackUtils.resolveLoopbackAddress(), null);
         sessionHandles.add(session);
       }
 
@@ -240,7 +251,8 @@ public class TestCLIServiceConnectionLimits {
     List<SessionHandle> sessionHandles = new ArrayList<>();
     try {
       for (int i = 0; i < limit + 1; i++) {
-        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar", "127.0.0.1", null);
+        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar",
+            IPStackUtils.resolveLoopbackAddress(), null);
         sessionHandles.add(session);
       }
 
@@ -255,7 +267,8 @@ public class TestCLIServiceConnectionLimits {
   @Test
   public void testConnectionMultipleLimitsIPAndUserIP() throws HiveSQLException {
     thrown.expect(HiveSQLException.class);
-    thrown.expectMessage("Connection limit per ipaddress reached (ipaddress: 127.0.0.1 limit: 5)");
+    thrown.expectMessage(String.format("Connection limit per ipaddress reached (ipaddress: %s limit: 5)",
+        IPStackUtils.resolveLoopbackAddress()));
 
     conf.setIntVar(HiveConf.ConfVars.HIVE_SERVER2_LIMIT_CONNECTIONS_PER_USER, 0);
     conf.setIntVar(HiveConf.ConfVars.HIVE_SERVER2_LIMIT_CONNECTIONS_PER_IPADDRESS, 5);
@@ -264,7 +277,8 @@ public class TestCLIServiceConnectionLimits {
     List<SessionHandle> sessionHandles = new ArrayList<>();
     try {
       for (int i = 0; i < limit + 1; i++) {
-        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar", "127.0.0.1", null);
+        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar",
+            IPStackUtils.resolveLoopbackAddress(), null);
         sessionHandles.add(session);
       }
 
@@ -279,7 +293,8 @@ public class TestCLIServiceConnectionLimits {
   @Test
   public void testConnectionMultipleLimitsUserIPAndUser() throws HiveSQLException {
     thrown.expect(HiveSQLException.class);
-    thrown.expectMessage("Connection limit per user:ipaddress reached (user:ipaddress: foo:127.0.0.1 limit: 10)");
+    thrown.expectMessage(String.format("Connection limit per user:ipaddress reached (user:ipaddress: foo:%s limit: 10)",
+        IPStackUtils.resolveLoopbackAddress()));
 
     conf.setIntVar(HiveConf.ConfVars.HIVE_SERVER2_LIMIT_CONNECTIONS_PER_USER, 15);
     conf.setIntVar(HiveConf.ConfVars.HIVE_SERVER2_LIMIT_CONNECTIONS_PER_IPADDRESS, 0);
@@ -288,7 +303,8 @@ public class TestCLIServiceConnectionLimits {
     List<SessionHandle> sessionHandles = new ArrayList<>();
     try {
       for (int i = 0; i < limit + 1; i++) {
-        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar", "127.0.0.1", null);
+        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar",
+            IPStackUtils.resolveLoopbackAddress(), null);
         sessionHandles.add(session);
       }
 
@@ -303,17 +319,20 @@ public class TestCLIServiceConnectionLimits {
   @Test
   public void testConnectionForwardedIpAddresses() throws HiveSQLException {
     thrown.expect(HiveSQLException.class);
-    thrown.expectMessage("Connection limit per ipaddress reached (ipaddress: 194.167.0.3 limit: 10)");
+    thrown.expectMessage(String.format("Connection limit per ipaddress reached (ipaddress: %s limit: 10)", 
+        IPStackUtils.transformToIPv6("194.167.0.3")));
 
     conf.setIntVar(HiveConf.ConfVars.HIVE_SERVER2_LIMIT_CONNECTIONS_PER_USER, 0);
     conf.setIntVar(HiveConf.ConfVars.HIVE_SERVER2_LIMIT_CONNECTIONS_PER_IPADDRESS, 10);
     conf.setIntVar(HiveConf.ConfVars.HIVE_SERVER2_LIMIT_CONNECTIONS_PER_USER_IPADDRESS, 10);
     CLIService service = getService(conf);
-    SessionManager.setForwardedAddresses(Lists.newArrayList("194.167.0.3", "194.167.0.2", "194.167.0.1"));
+    SessionManager.setForwardedAddresses(Lists.newArrayList(IPStackUtils.transformToIPv6("194.167.0.3"),
+        IPStackUtils.transformToIPv6("194.167.0.2"), IPStackUtils.transformToIPv6("194.167.0.1")));
     List<SessionHandle> sessionHandles = new ArrayList<>();
     try {
       for (int i = 0; i < limit + 1; i++) {
-        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar", "194.167.0.1", null);
+        SessionHandle session = service.openSession(CLIService.SERVER_VERSION, "foo", "bar", 
+            IPStackUtils.transformToIPv6("194.167.0.1"), null);
         sessionHandles.add(session);
       }
 
