@@ -81,6 +81,7 @@ import org.apache.hadoop.hive.metastore.datasource.DataSourceProvider;
 import org.apache.hadoop.hive.metastore.datasource.DataSourceProviderFactory;
 import org.apache.hadoop.hive.metastore.events.AbortTxnEvent;
 import org.apache.hadoop.hive.metastore.events.AcidWriteEvent;
+import org.apache.hadoop.hive.metastore.events.CommitTxnEvent;
 import org.apache.hadoop.hive.metastore.events.ListenerEvent;
 import org.apache.hadoop.hive.metastore.messaging.EventMessage;
 import org.apache.hadoop.hive.metastore.metrics.Metrics;
@@ -90,6 +91,7 @@ import org.apache.hadoop.hive.metastore.txn.entities.CompactionState;
 import org.apache.hadoop.hive.metastore.txn.entities.LockInfo;
 import org.apache.hadoop.hive.metastore.txn.entities.MetricsInfo;
 import org.apache.hadoop.hive.metastore.txn.entities.TxnStatus;
+import org.apache.hadoop.hive.metastore.txn.entities.TxnWriteDetails;
 import org.apache.hadoop.hive.metastore.txn.jdbc.commands.*;
 import org.apache.hadoop.hive.metastore.txn.jdbc.functions.*;
 import org.apache.hadoop.hive.metastore.txn.jdbc.queries.*;
@@ -129,6 +131,7 @@ import java.util.Set;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.function.BiPredicate;
+import java.util.stream.Collectors;
 
 import static org.apache.hadoop.hive.metastore.txn.TxnUtils.getEpochFn;
 import static org.apache.hadoop.hive.metastore.utils.MetaStoreUtils.getDefaultCatalog;
@@ -292,11 +295,10 @@ public abstract class TxnHandler implements TxnStore, TxnStore.MutexAPI {
             }
           }
           if (dbProduct == null) {
-            try (Connection dbConn = getDbConn(Connection.TRANSACTION_READ_COMMITTED, connPool)) {
-              determineDatabaseProduct(dbConn);
-            } catch (SQLException e) {
-              LOG.error("Unable to determine database product", e);
-              throw new RuntimeException(e);
+            dbProduct = DatabaseProduct.determineDatabaseProduct(connPool, conf);
+            if (dbProduct.isUNDEFINED()) {
+              String msg = "Unrecognized database product name <" + dbProduct.getProductName() + ">";
+              throw new IllegalStateException(msg);
             }
           }
           if (sqlGenerator == null) {
@@ -375,6 +377,26 @@ public abstract class TxnHandler implements TxnStore, TxnStore.MutexAPI {
         .toOpenTxnsResponse(excludeTxnTypes);
   }
 
+  @Override
+  public List<Long> getOpenTxnForPolicy(List<Long> openTxnList, String replPolicy) {
+
+    if (openTxnList.isEmpty()) {
+      return Collections.emptyList();
+    }
+      List<Long> targetTxnIds = null;
+      try {
+          targetTxnIds = jdbcResource.execute(new GetTargetTxnIdListForPolicyHandler(replPolicy, openTxnList));
+      } catch (MetaException e) {
+          throw new RuntimeException(e);
+      }
+
+      if (targetTxnIds.isEmpty()) {
+      LOG.info("There are no Repl Created open transactions on DR side.");
+    }
+    return targetTxnIds;
+  }
+
+
   /**
    * Retry-by-caller note:
    * Worst case, it will leave an open txn which will timeout.
@@ -494,16 +516,39 @@ public abstract class TxnHandler implements TxnStore, TxnStore.MutexAPI {
 
   @Override
   public void abortTxn(AbortTxnRequest rqst) throws NoSuchTxnException, MetaException, TxnAbortedException {
+    List<TxnWriteDetails> txnWriteDetails = new ArrayList<>();
+    if (transactionalListeners != null) {
+      //Find the write details for this transaction.
+      //Doing it here before the metadata tables are updated below.
+      txnWriteDetails = getWriteIdsForTxnID(rqst.getTxnid());
+    }
     TxnType txnType = new AbortTxnFunction(rqst).execute(jdbcResource); 
     if (txnType != null) {
       if (transactionalListeners != null && (!rqst.isSetReplPolicy() || !TxnType.DEFAULT.equals(rqst.getTxn_type()))) {
-        List<String> dbsUpdated = getTxnDbsUpdated(rqst.getTxnid());
-        MetaStoreListenerNotifier.notifyEventWithDirectSql(transactionalListeners, EventMessage.EventType.ABORT_TXN,
-            new AbortTxnEvent(rqst.getTxnid(), txnType, null, dbsUpdated), jdbcResource.getConnection(), sqlGenerator);
+        notifyCommitOrAbortEvent(rqst.getTxnid(),EventMessage.EventType.ABORT_TXN, txnType, jdbcResource.getConnection(), txnWriteDetails, transactionalListeners);
       }
     }
   }
 
+  public static void notifyCommitOrAbortEvent(long txnId, EventMessage.EventType eventType, TxnType txnType, Connection dbConn,
+                                       List<TxnWriteDetails> txnWriteDetails, List<TransactionalMetaStoreEventListener> transactionalListeners) throws MetaException {
+    List<Long> writeIds = txnWriteDetails.stream()
+            .map(TxnWriteDetails::getWriteId)
+            .collect(Collectors.toList());
+    List<String> databases = txnWriteDetails.stream()
+            .map(TxnWriteDetails::getDbName)
+            .collect(Collectors.toList());
+    ListenerEvent txnEvent;
+    if (eventType.equals(EventMessage.EventType.ABORT_TXN)) {
+      txnEvent = new AbortTxnEvent(txnId, txnType, null, databases, writeIds);
+    } else {
+      txnEvent = new CommitTxnEvent(txnId, txnType, null, databases, writeIds);
+    }
+    MetaStoreListenerNotifier.notifyEventWithDirectSql(transactionalListeners,
+            eventType, txnEvent, dbConn, sqlGenerator);
+  }
+
+
   @Override
   public void abortTxns(AbortTxnsRequest rqst) throws MetaException {
     List<Long> txnIds = rqst.getTxn_ids();
@@ -511,6 +556,13 @@ public abstract class TxnHandler implements TxnStore, TxnStore.MutexAPI {
     if (rqst.isSetErrorCode()) {
       txnErrorMsg = TxnErrorMsg.getTxnErrorMsg(rqst.getErrorCode());
     }
+    HashMap<Long, List<TxnWriteDetails>> txnWriteDetailsMap = new HashMap<>();
+    if (transactionalListeners != null) {
+      //Find the write details for this transaction.
+      //Doing it here before the metadata tables are updated below.
+      for(Long txnId : txnIds)
+        txnWriteDetailsMap.put(txnId, getWriteIdsForTxnID(txnId));
+    }
 
     List<String> queries = new ArrayList<>();
     StringBuilder prefix =
@@ -543,10 +595,8 @@ public abstract class TxnHandler implements TxnStore, TxnStore.MutexAPI {
 
       if (transactionalListeners != null) {
         for (Long txnId : txnIds) {
-          List<String> dbsUpdated = getTxnDbsUpdated(txnId);
-          MetaStoreListenerNotifier.notifyEventWithDirectSql(transactionalListeners,
-              EventMessage.EventType.ABORT_TXN, new AbortTxnEvent(txnId,
-                  nonReadOnlyTxns.getOrDefault(txnId, TxnType.READ_ONLY), null, dbsUpdated), dbConn, sqlGenerator);
+          notifyCommitOrAbortEvent(txnId,EventMessage.EventType.ABORT_TXN,
+                  nonReadOnlyTxns.getOrDefault(txnId, TxnType.READ_ONLY), dbConn, txnWriteDetailsMap.get(txnId), transactionalListeners);
         }
       }
     } catch (SQLException e) {
@@ -1065,22 +1115,6 @@ public abstract class TxnHandler implements TxnStore, TxnStore.MutexAPI {
         (ResultSet rs, int rowNum) -> rs.getTimestamp(1));
   }
 
-  private void determineDatabaseProduct(Connection conn) {
-    try {
-      String s = conn.getMetaData().getDatabaseProductName();
-      dbProduct = DatabaseProduct.determineDatabaseProduct(s, conf);
-      if (dbProduct.isUNDEFINED()) {
-        String msg = "Unrecognized database product name <" + s + ">";
-        LOG.error(msg);
-        throw new IllegalStateException(msg);
-      }
-    } catch (SQLException e) {
-      String msg = "Unable to get database product name";
-      LOG.error(msg, e);
-      throw new IllegalStateException(msg, e);
-    }
-  }
-  
   private void initJdbcResource() {
     if (jdbcResource == null) {
       jdbcResource = new MultiDataSourceJdbcResource(dbProduct, conf, sqlGenerator);
@@ -1138,4 +1172,24 @@ public abstract class TxnHandler implements TxnStore, TxnStore.MutexAPI {
     }
   }
 
+  /**
+   * Returns the databases and writeID updated by txnId.
+   * Queries TXN_TO_WRITE_ID using txnId.
+   *
+   * @param txnId Transaction ID for which write IDs are requested.
+   * @throws MetaException
+   */
+  public List<TxnWriteDetails> getWriteIdsForTxnID(long txnId) throws MetaException {
+    try {
+      return sqlRetryHandler.executeWithRetry(
+              new SqlRetryCallProperties().withCallerId("GetWriteIdsForTxnIDHandler"),
+              () -> jdbcResource.execute(new GetWriteIdsForTxnIDHandler(txnId)));
+    } catch (MetaException e) {
+      throw e;
+    } catch (TException e) {
+      throw new MetaException(e.getMessage());
+    }
+  }
+
+
 }
