@@ -104,7 +104,7 @@ public interface TxnStore extends Configurable {
 
 
   enum MUTEX_KEY {
-    Initiator, Cleaner, HouseKeeper, TxnCleaner,
+    Initiator, Cleaner, HouseKeeper, IcebergHouseKeeper, TxnCleaner,
     CompactionScheduler, MaterializationRebuild
   }
   // Compactor states (Should really be enum)
@@ -132,6 +132,7 @@ public interface TxnStore extends Configurable {
   char MINOR_TYPE = 'i';
   char REBALANCE_TYPE = 'r';
   char ABORT_TXN_CLEANUP_TYPE = 'c';
+  char SMART_OPTIMIZE_TYPE = '*';
 
   String[] COMPACTION_STATES = new String[] {INITIATED_RESPONSE, WORKING_RESPONSE, CLEANING_RESPONSE, FAILED_RESPONSE,
       SUCCEEDED_RESPONSE, DID_NOT_INITIATE_RESPONSE, REFUSED_RESPONSE };
@@ -186,6 +187,11 @@ public interface TxnStore extends Configurable {
   @RetrySemantics.ReadOnly
   GetOpenTxnsResponse getOpenTxns(List<TxnType> excludeTxnTypes) throws MetaException;
 
+  @SqlRetry
+  @Transactional(POOL_TX)
+  @RetrySemantics.ReadOnly
+  List<Long> getOpenTxnForPolicy(List<Long> openTxnList, String replPolicy);
+
   /**
    * Get the count for open transactions.
    * @throws MetaException
