@@ -595,6 +595,8 @@ public class TxnUtils {
         return CompactionType.REBALANCE;
       case TxnStore.ABORT_TXN_CLEANUP_TYPE:
         return CompactionType.ABORT_TXN_CLEANUP;
+      case TxnStore.SMART_OPTIMIZE_TYPE:
+        return CompactionType.SMART_OPTIMIZE;
       default:
         throw new SQLException("Unexpected compaction type " + dbValue);
     }
@@ -610,6 +612,8 @@ public class TxnUtils {
         return TxnStore.REBALANCE_TYPE;
       case ABORT_TXN_CLEANUP:
         return TxnStore.ABORT_TXN_CLEANUP_TYPE;
+      case SMART_OPTIMIZE:
+        return TxnStore.SMART_OPTIMIZE_TYPE;
       default:
         throw new MetaException("Unexpected compaction type " + ct);
     }
