@@ -63,6 +63,7 @@ import org.apache.commons.beanutils.PropertyUtils;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.collections.ListUtils;
 import org.apache.commons.lang3.StringUtils;
+import org.apache.commons.lang3.math.NumberUtils;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileStatus;
 import org.apache.hadoop.fs.Path;
@@ -1555,7 +1556,8 @@ public class MetaStoreServerUtils {
     try {
       return Warehouse.makePartName(getPartCols(table), partition.getValues());
     } catch (MetaException e) {
-      throw new RuntimeException(e);
+      throw new RuntimeException("Invalid partition found, location: " +
+          getDataLocation(table, partition), e);
     }
   }
 
@@ -1636,6 +1638,10 @@ public class MetaStoreServerUtils {
 
   public static String getNormalisedPartitionValue(String partitionValue, String type) {
 
+    if (!NumberUtils.isParsable(partitionValue)) {
+      return partitionValue;
+    }
+
     LOG.debug("Converting '" + partitionValue + "' to type: '" + type + "'.");
 
     if (type.equalsIgnoreCase("tinyint")
