@@ -286,13 +286,6 @@ public class TestTablesGetExists extends MetaStoreClientTest {
     Assert.assertTrue("Comparing tablenames", tables.contains(testTables[5].getTableName()));
     Assert.assertTrue("Comparing tablenames", tables.contains(testTables[6].getTableName()));
 
-    tables = client.getTables("*", "*");
-    Assert.assertEquals("All tables size", 7, tables.size());
-    tables = client.getTables("d*", "*");
-    Assert.assertEquals("All tables size", 7, tables.size());
-    tables = client.getTables("def*", "*");
-    Assert.assertEquals("All tables size", 5, tables.size());
-
     // Look for tables but do not find any
     tables = client.getTables(DEFAULT_DATABASE, "*_not_such_function_*");
     Assert.assertEquals("No such table size", 0, tables.size());
